/**
   @file bayes.h
   @brief ベイズモデル 定義用ヘッダ

   @author tadah_fussy
   @date 2021/03/04 ライブラリとしてファイルを整形
   @date 2021/03/26 名前空間の見直し
**/
#ifndef BAYES_20210326_H
#define BAYES_20210326_H

#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>
#include <set>
#include <map>
#include <string>

#include <math.h>

namespace tadah_fussy
{
  namespace statistics
  {
    /**
       @brief ベルヌーイ・モデルを使ったナイーブ・ベイズ
    **/
    template< class T > class Bernoulli_NaiveBayesClass
    {
      using ClassIndex = std::vector< size_t >::size_type;
      using ClassName = std::map< std::string, ClassIndex >;
      using TrainCount = std::vector< size_t >;
      using Count = std::map< T, TrainCount >;

      ClassName className_;   // クラス名の配列
      TrainCount trainCount_; // 学習パターンの数
      Count count_;           // 各特徴の数

    public:

      /// @brief 学習処理
      ///
      /// @param name クラス名
      /// @param data 学習パターン
      void training( const std::string& name, const std::map< T, size_t >& data );

      /// @brief 発生比率の積の対数を返す
      ///
      /// @param data 対象の特徴ベクトル
      /// @param totalProb クラスごとの発生比率の積の対数を返す変数への参照
      void prob( const std::map< T, size_t >& data, std::map< std::string, double >& totalProb ) const;

      /// @brief 学習結果の出力
      ///
      /// @param os 出力ストリーム
      /// @param nbClass 対象の NaiveBayesClass
      /// @return 出力ストリーム os への参照
      template< class T2 > friend std::ostream& operator<<( std::ostream& os, const Bernoulli_NaiveBayesClass< T2 >& nbClass );
    };

    /**
       @brief 多項モデルを使ったナイーブ・ベイズ
    **/
    template< class T > class Multinominal_NaiveBayesClass
    {
      using ClassIndex = std::vector< size_t >::size_type;
      using ClassName = std::map< std::string, ClassIndex >;
      using TotalCount = std::vector< size_t >;
      using TrainCount = std::vector< size_t >;
      using Count = std::map< T, TrainCount >;

      ClassName className_;   // クラス名の配列
      TotalCount totalCount_; // 要素数の合計
      TrainCount trainCount_; // 学習パターンの数
      Count count_;           // 各特徴の数

    public:

      /// @brief 学習処理
      ///
      /// @param name クラス名
      /// @param data 学習パターン
      void training( const std::string& name, const std::map< T, size_t >& data );

      /// @brief 発生比率の積の対数を返す
      ///
      /// @param data 対象の特徴ベクトル
      /// @param totalProb クラスごとの発生比率の積の対数を返す変数への参照
      void prob( const std::map< T, size_t >& data, std::map< std::string, double >& totalProb ) const;

      /// @brief 学習結果の出力
      ///
      /// @param os 出力ストリーム
      /// @param nbClass 対象の NaiveBayesClass
      /// @return 出力ストリーム os への参照
      template< class T2 > friend std::ostream& operator<<( std::ostream& os, const Multinominal_NaiveBayesClass< T2 >& nbClass );
    };

  } // namespace statistics

} // namespace tadah_fussy

#include "bayes_inline.h"

#endif
