namespace tadah_fussy
{
  namespace statistics
  {
    /*
      Bernoulli_NaiveBayesClass< T >::training : 学習

      const string& name : クラス名
      const map< T, size_t >& data : 学習パターン
    */
    template< class T > void Bernoulli_NaiveBayesClass< T >::training( const std::string& name, const std::map< T, size_t >& data )
    {
      // 未登録クラスの場合の処理
      if ( className_.find( name ) == className_.end() ) {
        auto i = className_.size(); // クラス数(新クラスの添字になる)
        className_[name] = i;
        // 各要素の発生回数を初期化(あらかじめ 1 加算しておく)
        for ( auto it = count_.begin() ; it != count_.end() ; ++it )
          ( it->second ).push_back( 1 );
        trainCount_.push_back( 2 ); // 学習パターンの数にはあらかじめ要素の種類(0,1)の数を加算しておく
      }

      auto index = className_[name]; // 指定したクラスに対する添字
      ++( trainCount_[index] );

      // 学習処理メイン
      for ( auto dataCit = data.begin() ; dataCit != data.end() ; ++dataCit )
        {
          // 学習パターンが count_ 内に登録された位置
          auto cntIt = count_.find( dataCit->first );

          // count_ に登録済みの要素の処理
          if ( cntIt != count_.end() ) {
            ++( ( cntIt->second )[index] );
            // count_ に未登録の要素の処理
          } else {
            count_[dataCit->first] = TrainCount( className_.size(), 1 );
            ++( count_[dataCit->first][index] );
          }
        }
    }

    /*
      Bernoulli_NaiveBayesClass< T >::prob : 発生比率の積の対数を返す

      const map< T, size_t >& data : 対象の特徴ベクトル
      map< string, double >& totalProb : クラスごとの発生比率の積の対数

      count_内の各要素に対して
      P( count_[key] ) = prob( key )     [keyがdataに存在する時]
      1 - prob( key ) [keyがdataに存在しない時]
    */
    template< class T > void Bernoulli_NaiveBayesClass< T >::prob( const std::map< T, size_t >& data, std::map< std::string, double >& totalProb ) const
    {
      // totalProbの初期化
      totalProb.clear();
      for ( auto nameCit = className_.begin() ; nameCit != className_.end() ; ++nameCit )
        totalProb[nameCit->first] = 0;

      for ( auto cntCit = count_.begin() ; cntCit != count_.end() ; ++cntCit )
        {
          const auto& vecCnt = cntCit->second; // 対象要素に対する頻度ベクトルへの参照

          for ( auto nameCit = className_.begin() ; nameCit != className_.end() ; ++nameCit )
            {
              auto i = nameCit->second; // 対象クラスの添字
              // data に対象要素が含まれていれば P、そうでなければ 1 - P
              totalProb[nameCit->first] += ( data.find( cntCit->first ) != data.end() ) ?
                log( (double)( vecCnt[i] ) / (double)( trainCount_[i] ) ) :
                log( 1.0 - (double)( vecCnt[i] ) / (double)( trainCount_[i] ) );
            }
        }

      // 総学習パターン数の計算
      size_t totalSum = 0;
      for ( std::vector< size_t >::size_type i = 0 ; i < trainCount_.size() ; ++i )
        totalSum += trainCount_[i];
      totalSum -= 2 * trainCount_.size(); // 2 だけ余分に加算した分を補正

      // 総数に対する各クラスの学習パターン数から頻度を算出して追加
      for ( auto nameCit = className_.begin() ; nameCit != className_.end() ; ++nameCit )
        totalProb[nameCit->first] += log( (double)( trainCount_[nameCit->second] - 2 ) / (double)( totalSum ) );
    }

    /*
      operator<< : 結果の出力

      ostream& os : 出力ストリーム
      const Bernoulli_NaiveBayesClass<T>& nbClass : 対象の NaiveBayesClass

      戻り値 : 出力ストリーム os への参照
    */
    template< class T > std::ostream& operator<<( std::ostream& os, const Bernoulli_NaiveBayesClass< T >& nbClass )
    {
      // クラス名の出力
      for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
        os << '\t' << nameCit->first;
      os << std::endl;

      // 要素ごとの出現頻度を出力
      for ( auto cntCit = nbClass.count_.begin() ; cntCit != nbClass.count_.end() ; ++cntCit ) {
        os << cntCit->first;
        for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
          os  << '\t' << ( cntCit->second )[nameCit->second] - 1;
        os << std::endl;
      }

      // クラスごとの学習パターン数を出力
      os << "[TOTAL]";
      for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
        os  << '\t' << nbClass.trainCount_[nameCit->second] - 2;
      os << std::endl;

      return( os );
    }

    /*
      Multinominal_Bernoulli_NaiveBayesClass<T>::training : 学習

      const string& name : クラス名
      const map< T, size_t >& data : 学習パターン
    */
    template< class T > void Multinominal_NaiveBayesClass< T >::training( const std::string& name, const std::map< T, size_t >& data )
    {
      // 未登録クラスの場合の処理
      if ( className_.find( name ) == className_.end() ) {
        auto i = className_.size(); // クラス数(新クラスの添字になる)
        className_[name] = i;
        // 各要素の発生回数を初期化(あらかじめ 1 加算しておく)
        for ( auto it = count_.begin() ; it != count_.end() ; ++it )
          ( it->second ).push_back( 1 );
        totalCount_.push_back( count_.size() ); // 総要素数にはあらかじめ要素の種類の数を加算しておく
        trainCount_.push_back( 0 );
      }

      auto index = className_[name]; // 指定したクラスに対する添字
      ++( trainCount_[index] );

      // 学習処理メイン
      for ( auto dataCit = data.begin() ; dataCit != data.end() ; ++dataCit )
        {
          totalCount_[index] += dataCit->second;

          // 学習パターンが count_ 内に登録された位置
          auto cntIt = count_.find( dataCit->first );

          // count_ に登録済みの要素の処理
          if ( cntIt != count_.end() ) {
            ( cntIt->second )[index] += dataCit->second;
            // count_ に未登録の要素の処理
          } else {
            count_[dataCit->first] = TrainCount( className_.size(), 1 );
            count_[dataCit->first][index] += dataCit->second;
            // 総要素数を補正
            for ( std::vector< size_t >::size_type i = 0 ; i < totalCount_.size() ; ++i )
              ++( totalCount_[i] );
          }
        }
    }

    /*
      Multinominal_NaiveBayesClass<T>::prob : 発生比率の積の対数を返す

      const map< T, size_t >& data : 対象の特徴ベクトル
      map< string, double >& totalProb : クラスごとの発生比率の積の対数

      count_内の各要素に対して
      P( count_[key] ) = pow( prob( key ), count( key ) )

    */
    template< class T > void Multinominal_NaiveBayesClass< T >::prob( const std::map< T, size_t >& data, std::map< std::string, double >& totalProb ) const
    {
      // totalProbの初期化
      totalProb.clear();
      for ( auto nameCit = className_.begin() ; nameCit != className_.end() ; ++nameCit )
        totalProb[nameCit->first] = 0;

      for ( auto cntCit = count_.begin() ; cntCit != count_.end() ; ++cntCit )
        {
          const auto& vecCnt = cntCit->second; // 対象要素に対する頻度ベクトルへの参照

          for ( auto nameCit = className_.begin() ; nameCit != className_.end() ; ++nameCit )
            {
              auto i = nameCit->second; // 対象クラスの添字
              // 特徴ベクトル内の対象要素の位置
              auto dataCit = data.find( cntCit->first );
              if ( dataCit != data.end() )
                totalProb[nameCit->first] += log( pow( (double)( vecCnt[i] ) / (double)( totalCount_[i] ), dataCit->second ) );
            }
        }

      // 総学習パターン数の計算
      size_t totalSum = 0;
      for ( std::vector< size_t >::size_type i = 0 ; i < trainCount_.size() ; ++i )
        totalSum += trainCount_[i];

      // 総数に対する各クラスの学習パターン数から頻度を算出して追加
      for ( auto nameCit = className_.begin() ; nameCit != className_.end() ; ++nameCit )
        totalProb[nameCit->first] += log( (double)( trainCount_[nameCit->second] ) / (double)( totalSum ) );
    }

    /*
      operator<< : 結果の出力

      ostream& os : 出力ストリーム
      const Multinominal_NaiveBayesClass<T>& nbClass : 対象の NaiveBayesClass

      戻り値 : 出力ストリーム os への参照
    */
    template< class T > std::ostream& operator<<( std::ostream& os, const Multinominal_NaiveBayesClass< T >& nbClass )
    {
      // クラス名の出力
      for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
        os << '\t' << nameCit->first;
      os << std::endl;

      // 要素ごとの要素数を出力
      for ( auto cntCit = nbClass.count_.begin() ; cntCit != nbClass.count_.end() ; ++cntCit ) {
        os << cntCit->first;
        for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
          os  << '\t' << ( cntCit->second )[nameCit->second] - 1;
        os << std::endl;
      }

      // クラスごとの要素数合計を出力
      os << "[FEATURE TOTAL]";
      for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
        os  << '\t' << nbClass.totalCount_[nameCit->second] - nbClass.count_.size();

      // クラスごとの学習パターン数を出力
      os << std::endl << "[TRAINING TOTAL]";
      for ( auto nameCit = nbClass.className_.begin() ; nameCit != nbClass.className_.end() ; ++nameCit )
        os  << '\t' << nbClass.trainCount_[nameCit->second];

      return( os );
    }

  } // namespace statistics

} // namespace tadah_fussy
