/**
   @file glm.h
   @brief 一般化線形モデル 定義用ヘッダ

   @author tadah_fussy
   @date 2015/12/15 statistics.h から分離
   @date 2016/05/05 クラス構成の見直し(スコア法を統合化)・ドキュメントの整備
   @date 2017/04/07 LinearEquationSystem の廃止に伴う変更
   @date 2019/04/30 GeneralizedLinearModel/GLM_BinomialLogistic/GLM_PoissonRegression の仕様変更
   @date 2019/05/10 最大対数尤度などの統計値を返すメンバ関数を全体に追加
   @date 2019/05/12 最終チェック版
   @date 2021/03/03 細かい部分の修正 ( 型など )
   @date 2021/03/21 名前空間の見直し
**/
#ifndef GLM_20210321_H
#define GLM_20210321_H

#include <common/optimization.h>
#include "statistics.h"
//#define DEBUG

namespace tadah_fussy
{
  namespace statistics
  {
    /**
       @brief 一般化線形モデル名前空間
    **/
    namespace glm
    {
      /**
         @brief 一般化線形モデル構築用基底クラス

         一般化線形モデルを構築してスコア法で係数を求めるためのクラスの基底となる。以下のメンバ関数が必須となる。

         - init ... 係数計算のための連立方程式を初期化する
         - create ... 係数計算のための連立方程式を作成する
         - update ... 係数更新後に他のデータを更新する
         - cleanUp ... 係数計算後にその他のパラメータの計算などを行う

         init はスコア法(反復法)のループ前に呼び出され、主に配列などの領域確保などに使われる。
         また、create は反復処理の中で連立方程式を構築するために利用される。
         連立方程式を解いて係数を更新した後、更新が必要なデータは update で処理する。
         ループ処理が完了した後、cleanUp でその他の必要な処理を行う。
         init は求める係数を保持する配列と連立方程式の両方が、
         create には連立方程式が、update は更新後の係数が引数として渡される。
         cleanUp には引数はない。
      **/
      class Base
      {
      public:

        /// @brief 仮想デストラクタ(何もしない)
        ///
        virtual ~Base() {}

      protected:

        /// @brief 引数・戻り値の型
        using stat_type = distribution::Base::stat_type;

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする。
        ///
        /// @param a 求めた係数を保持する配列へのポインタ
        /// @param var_a 係数の分散を保持する配列へのポインタ(NULLの場合は無視)
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        template< class Solver, class Test >
        void scoringMethod
        ( std::vector< stat_type >* a, std::vector< stat_type >* var_a,
          Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT );

      private:

        /// @brief 係数計算のための連立方程式と係数を保持する配列を初期化する
        ///
        /// スコア法(反復法)のループ前に一回だけ呼び出され、主に配列などの領域確保などに使われる。
        /// 連立方程式 s や求めた係数を保持するための配列 a は必要なサイズが確保されていないため、
        /// この中で初期化を行うことを前提としている。また、必要であれば a の初期値として近似値を
        /// あらかじめ代入することもできる。
        ///
        /// @param sa 係数行列へのポインタ
        /// @param sb 右辺(解ベクトル)へのポインタ
        /// @param a 求める係数を保持するための配列へのポインタ
        virtual void init( SquareMatrix< stat_type >* sa,
                           std::vector< stat_type >* sb,
                           std::vector< stat_type >* a ) = 0;

        /// @brief 係数計算のための連立方程式のを作成する
        ///
        /// 反復処理の中で連立方程式を構築するために利用される。
        ///
        /// @param sa 係数行列へのポインタ
        /// @param sb 右辺(解ベクトル)へのポインタ
        virtual void create( SquareMatrix< stat_type >* sa,
                             std::vector< stat_type >* sb ) = 0;

        /// @brief 係数計算後の更新処理を行う
        ///
        /// 反復処理の中で期待値の更新を行うために利用される。引数の a には反復処理で得られた
        /// 係数の更新値が保持されている。
        ///
        /// @param a 求めた係数を保持した配列への参照
        virtual void update( const std::vector< stat_type >& a ) = 0;

        /// @brief 反復処理完了後に必要な処理を行う
        ///
        /// 係数が得られてからその他のパラメータを計算するときなどに利用される。
        virtual void cleanUp() = 0;
      };

      /**
         @brief 一母数指数型分布族用 I/F

         以下の形で表される確率密度関数を「指数型分布族」と呼ぶ。

         f(y|θ) = h(y)・exp( η(θ)T(y) - A(θ) )

         但し、y は確率変数、θは確率分布が持つパラメータを表す。

         特にθが一変数ならば「一母数指数型分布族」になる。η = η(θ) とし、A(θ) が変数変換によって A(η) で表されるなら、上式は

         f(y|η) = h(y)・exp( η・T(y) - A(η) )

         となる。

         これは、一般化線形モデルにおける確率密度関数に利用することを想定したクラスである。
      **/
      struct ExpFamily_IF
      {
        /// @brief 引数・戻り値の型
        using stat_type = distribution::Base::stat_type;

        /// @brief A(η) を求める
        ///
        /// @param eta ηの値
        /// @return A(η)の値
        virtual stat_type a( stat_type eta ) const = 0;

        /// @brief T(y)の期待値 ( = A(η) の導関数の値 ) を返す
        ///
        /// @param eta ηの値
        /// @return T(y) の期待値
        virtual stat_type average( stat_type eta ) const = 0;

        /// @brief A(η) の導関数の逆関数の値を返す
        ///
        /// η = A'^(-1)( E[T(y)] ) を計算する
        ///
        /// @param mu E[T(y)] の値
        /// @return A(η) の導関数の逆関数の値 (A'^(-1)( E[T(y)] ))
        virtual stat_type aveInv( stat_type mu ) const = 0;

        /// @brief T(y) の分散 ( = A(η) の二階導関数の値 ) を返す
        ///
        /// @param eta ηの値
        /// @return T(y) の分散
        virtual stat_type variance( stat_type eta ) const = 0;

        /// @brief B(y) = log( h(y) ) の値を返す
        ///
        /// @param y y の値
        /// @return B(y)
        virtual stat_type b( stat_type y ) const = 0;

        /// @brief 対数尤度の値を返す
        ///
        /// @param eta ηの値
        /// @param y y の値
        /// @return 対数尤度
        stat_type logLikelihood( stat_type eta, stat_type y ) const
        { return( eta * y - a( eta ) + b( y ) ); }
        //{ return( eta * y - a( eta ) ); }

        /// @brief 属性を表す文字列を返す
        ///
        /// @return 属性
        virtual std::string ident() const = 0;

        /// @brief 仮想デストラクタ(何もしない)
        virtual ~ExpFamily_IF() {}
      };

      /**
         @brief 汎用一母数指数型分布族

         導関数・二階導関数の算出に math_lib::diff_method::Central を、導関数の逆関数の算出に math_lib::solver::Newton をそれぞれ使用する
      **/
      class ExpFamily_Generic : public ExpFamily_IF
      {
        stat_type h_;         // DiffMethod用の極少量
        stat_type threshold_; // Newton用のしきい値
        uint32_t maxCnt_;     // Newton用の最大反復回数

      public:

        /// @brief 導関数の導出に使う極少量を指定して構築
        ///
        /// 導関数の計算には中心差分法 ( math_lib::diff_method::Central ) を利用しているため、分母の極少量 h を指定する必要がある。
        ///
        /// 導関数の逆関数の計算にはNewton法 ( math_lib::solver::Newton ) を利用しているため、反復処理用の最大処理回数 maxCnt としきい値 threshold を指定する必要がある。
        ///
        /// @param h 中心差分法で使う極少量
        /// @param threshold Newton法で使うしきい値
        /// @param maxCnt Newton法で使う最大反復処理回数
        ExpFamily_Generic( stat_type h = DEFAULT_THRESHOLD,
                           stat_type threshold = DEFAULT_THRESHOLD, uint32_t maxCnt = DEFAULT_MAX_COUNT )
          : h_( h ), threshold_( threshold ), maxCnt_( maxCnt )
        {
          const std::string FUNC_NAME = "ExpFamily_Generic constructor";

          error_lib::PNum( h_, FUNC_NAME + " : h " + error_message::PNUM );
          error_lib::PNum( threshold_, FUNC_NAME + " : threshold " + error_message::PNUM );
        }

        // A(η) の導関数 = 期待値 ( E[T(y)] = A'(η) )
        virtual stat_type average( stat_type eta ) const
        {
          using namespace std::placeholders;

          return( math_lib::diff_method::CreateCentral( std::bind( std::mem_fun( &ExpFamily_IF::a ), this, _1 ), h_ )( eta ) );
        }

        // A(η) の導関数の逆関数 ( η = A'^(-1)(E[T(y)]) )
        virtual stat_type aveInv( stat_type mu ) const
        {
          using namespace std::placeholders;

          return( math_lib::solver::Newton( mu, std::bind( std::mem_fun( &ExpFamily_IF::average ), this, _1 ),
                                            std::bind( std::mem_fun( &ExpFamily_IF::variance ), this, _1 ),
                                            1.0, maxCnt_, threshold_ )
                  );
        }

        // A(η) の二階導関数 = 分散 ( V[T(y)] = A''(η) )
        virtual stat_type variance( stat_type eta ) const
        {
          using namespace std::placeholders;

          return( math_lib::diff_method::CreateCentral( std::bind( std::mem_fun( &ExpFamily_IF::average ), this, _1 ), h_ )( eta ) );
        }
      };

      /**
         @brief 一母数指数型分布族(正規分布)

         パラメータとして平均を使用し、標準偏差(分散)は局外パラメータとして扱う。
      **/
      class ExpFamily_NormDist : public ExpFamily_IF
      {
        stat_type var_; // 分散 σ^2

      public:

        /// @brief 標準偏差を指定して構築
        ///
        /// - sigma は正数でなければならない。そうでない場合、例外 exception::NotPositiveNumber< stat_type > を投げる。
        ///
        /// @param sigma 標準偏差(分散ではないことに注意)
        ExpFamily_NormDist( stat_type sigma )
          : var_( std::pow( error_lib::PNum( sigma, "ExpFamily_NormDist constructor : sigma " + error_message::PNUM ), 2 ) ) {}

        // A(η)
        virtual stat_type a( stat_type eta ) const
        { return( ( std::pow( eta, 2 ) * var_ + std::log( 2 * math_lib::Pi< stat_type >() * var_ ) ) / 2 ); }

        // A(η) の導関数 = 期待値 ( E[y] = A'(η) )
        virtual stat_type average( stat_type eta ) const
        { return( eta * var_ ); }

        // A(η) の導関数の逆関数 ( η = A'^(-1)(y) )
        virtual stat_type aveInv( stat_type mu ) const
        { return( mu / var_ ); }

        // A(η) の二階導関数 = 分散 ( V[y] = A''(η) )
        virtual stat_type variance( stat_type eta ) const
        { return( var_ ); }

        // B(y) = log( h(y) )
        virtual stat_type b( stat_type y ) const
        { return( -std::pow( y, 2 ) / ( 2 * var_ ) ); }

        // 属性を表す文字列
        virtual std::string ident() const
        {
          std::stringstream ss;
          ss.setf( std::ios::fixed );
          ss.precision( 3 );
          ss << "Normal Distribution (var = " << var_ << ")";
          return( ss.str() );
        }
      };

      /**
         @brief 一母数指数型分布族(ポアソン分布)
      **/
      struct ExpFamily_Poisson : public ExpFamily_IF
      {
        using stat_type = stat_type;

      public:

        // A(η)
        virtual stat_type a( stat_type eta ) const
        { return( std::exp( eta ) ); }

        // A(η) の導関数 = 期待値 ( E[y] = A'(η) )
        virtual stat_type average( stat_type eta ) const
        { return( std::exp( eta ) ); }

        // A(η) の導関数の逆関数 ( η = A'^(-1)(y) )
        virtual stat_type aveInv( stat_type mu ) const
        { return( std::log( mu ) ); }

        // A(η) の二階導関数 = 分散 ( V[y] = A''(η) )
        virtual stat_type variance( stat_type eta ) const
        { return( std::exp( eta ) ); }

        // B(y) = log( h(y) )
        virtual stat_type b( stat_type y ) const
        { return( -std::lgamma( y + 1 ) ); }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Poisson Distribution" ); }
      };

      /**
         @brief 一母数指数型分布族(二項分布)
      **/
      class ExpFamily_Binomial : public ExpFamily_IF
      {
        stat_type n_; // 総試行回数 N

      public:

        /// @brief 総試行回数を指定して構築
        ///
        /// @param n 総試行回数
        ExpFamily_Binomial( distribution::Base::counter_type n )
          : n_( error_lib::NNNum( n, "ExpFamily_Binomial constructor : n " + error_message::NNNUM ) ) {}

        // A(η)
        virtual stat_type a( stat_type eta ) const
        { return( n_ * std::log( std::exp( eta ) + 1 ) ); }

        // 期待値 ( E[y] = A'(η) )
        stat_type average( stat_type eta ) const
        { return( n_ * ( 1.0 - 1.0 / ( std::exp( eta ) + 1.0 ) ) ); }

        // A(η) の導関数の逆関数 ( η = A'^(-1)(y) )
        virtual stat_type aveInv( stat_type mu ) const
        { return( std::log( mu / ( n_ - mu ) ) ); }

        // A(η) の二階導関数 = 分散 ( V[y] = A''(η) )
        //
        // x→∞のとき、e^x / ( e^x + 1 )^2 = 1 / [ e^x + 2 + ( 1 / e^x ) ] → 0
        virtual stat_type variance( stat_type eta ) const
        { return( ( std::isinf( eta ) ) ? 0 : n_ * std::exp( eta ) / std::pow( std::exp( eta ) + 1, 2 ) ); }

        // B(y) = log( h(y) )
        // 各試行を分解して n = 1 ; y = { 0, 1 } としたとき、h(y) = nCy は必ず 1 となるので無視できる。
        // ここでは、必ずゼロを返す仕様としている。
        virtual stat_type b( stat_type y ) const
        { return( 0 ); }
        //{ return( std::log( MathLib::BinCoef( n_, static_cast< DiscDist::var_type >( y ) ) ) ); }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Binomial Distribution" ); }
      };

      /**
         @brief 連結関数用 I/F

         一般化線形モデルにおける連結関数(Link Function)を定義するためのクラスである。
      **/
      struct LinkFunction_IF
      {
        /// @brief 引数・戻り値の型
        using stat_type = distribution::Base::stat_type;

        /// @brief 連結関数 g(x)
        ///
        /// @param x g(x)を求める対象のx
        /// @return g(x)の値
        virtual stat_type operator()( stat_type x ) const = 0;

        /// @brief 導関数 g'(x)
        ///
        /// @param x g'(x)を求める対象のx
        /// @return g'(x)の値
        virtual stat_type df( stat_type x ) const = 0;

        /// @brief 逆関数 g^-1(y)
        ///
        /// @param y g^-1(y)を求める対象のy
        /// @return g^-1(y)の値
        virtual stat_type invf( stat_type y ) const = 0;

        /// @brief 属性を表す文字列
        ///
        /// @return 属性
        virtual std::string ident() const = 0;
      };

      /**
         @brief 汎用連結関数

         導関数・二階導関数の算出に math_lib::diff_method::Central を、逆関数の計算に math_lib::solver::Newton をそれぞれ使用する
      **/
      class LinkFunction_Generic : public LinkFunction_IF
      {
        stat_type xInit_;     // Newton法での x の初期値
        uint32_t maxCount_;   // Newton法での反復回数最大値
        stat_type threshold_; // Newton法での収束判定しきい値
        stat_type h_;         // DiffMethod用の極少量

      public:

        /// @brief 導関数や逆関数の導出に使う極少量を指定して構築
        ///
        /// 導関数の計算には差分法 ( MathLib::DiffMethod ) を利用しているため、分母の極少量を指定する必要がある。
        ///
        /// 逆関数の計算には Newton 法 ( MathLib::Newton ) を利用しているため、反復処理用の最大処理回数としきい値を指定する必要がある。
        ///
        /// @param xInit Newton 法で使う初期値
        /// @param maxCount Newton 法で使う最大反復処理回数
        /// @param threshold Newton 法で使うしきい値
        /// @param h 差分法で使う極少量
        LinkFunction_Generic( stat_type xInit = 1.0, uint32_t maxCount = 100,
                              stat_type threshold = DEFAULT_THRESHOLD, stat_type h = DEFAULT_THRESHOLD )
          : xInit_( xInit ), maxCount_( maxCount ), threshold_( threshold ), h_( h )
        {
          const std::string FUNC_NAME = "LinkFunction_Generic constructor";

          error_lib::PNum( threshold_, FUNC_NAME + " : threshold " + error_message::PNUM );
          error_lib::PNum( h_, FUNC_NAME + " : h " + error_message::PNUM );
        }

        // 導関数 g'(x)
        virtual stat_type df( stat_type x ) const
        {
          using namespace std::placeholders;

          return( math_lib::diff_method::CreateCentral( std::bind( std::mem_fun( &LinkFunction_IF::operator() ), this, _1 ), h_ )( x ) );
        }

        // 逆関数 g^-1(y)
        virtual stat_type invf( stat_type y ) const
        {
          using namespace std::placeholders;

          return( math_lib::solver::Newton( y, std::bind( std::mem_fun( &LinkFunction_IF::operator() ), this, _1 ),
                                            std::bind( std::mem_fun( &LinkFunction_IF::df ), this, _1 ),
                                            xInit_, maxCount_, threshold_ )
                  );
        }
      };

      /**
         @brief 恒等連結関数

         連結関数として f(x) = x としたもの。
      **/
      struct LinkFunction_Identity : public LinkFunction_IF
      {
        // 連結関数 g(x)
        virtual stat_type operator()( stat_type x ) const
        { return( x ); }

        // 導関数 g'(x)
        virtual stat_type df( stat_type x ) const
        { return( 1 ); }

        // 逆関数 g^-1(y)
        virtual stat_type invf( stat_type y ) const
        { return( y ); }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Identity Function" ); }
      };

      /**
         @brief 対数連結関数

         連結関数として f(x) = log(x) としたもの。
      **/
      struct LinkFunction_Log : public LinkFunction_IF
      {
        // 連結関数 g(x)
        virtual stat_type operator()( stat_type x ) const
        { return( std::log( x ) ); }

        // 導関数 g'(x)
        virtual stat_type df( stat_type x ) const
        { return( 1 / x ); }

        // 逆関数 g^-1(y)
        virtual stat_type invf( stat_type y ) const
        { return( std::exp( y ) ); }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Logarithm Function" ); }
      };

      /**
         @brief プロビット・モデル連結関数

         連結関数としてプロビット・モデルを利用したもの。
         プロビット・モデルは標準正規分布の逆関数である。

         プロビット・モデルは 0 <= x <= 1 で定義され、確率πの関数と見ることができる。

         スコア法では連結関数の x の値は期待値μ = Nπを表すため、利用する際に変換が必要となることに注意。
         具体的には以下のようになる ( N = 1 の場合、変換は不要となる)。

         g(μ) → g(π) = g(μ/N)

         g'(μ) → g'(π) / N

         π = g^-1(ξ) → μ = Ng^-1(ξ)
      **/
      class LinkFunction_ProbitModel : public LinkFunction_IF
      {
        distribution::Normal norm_; // 標準正規分布

      public:

        /// @brief デフォルト・コンストラクタ
        LinkFunction_ProbitModel()
          : norm_( 0, 1 ) {}

        // 連結関数 g(x)
        virtual stat_type operator()( stat_type x ) const
        {
          if ( x < 0 || x > 1 ) return( distribution::Base::nan() );
          if ( x < 0.5 )
            return( -norm_.binSearch( 0.5 - x ) );
          else
            return( norm_.binSearch( x - 0.5 ) );
        }

        // 導関数 g'(x)
        virtual stat_type df( stat_type x ) const
        { return( std::exp( std::pow( (*this)( x ), 2.0 ) * 0.5 ) * std::sqrt( 2 * math_lib::Pi< stat_type >() ) ); }

        // 逆関数 g^-1(y)
        virtual stat_type invf( stat_type y ) const
        { return( norm_.lower_p( y ) ); }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Probit Model Function" ); }
      };

      /**
         @brief ロジット連結関数

         連結関数としてロジット関数を利用したもの。
         ロジット関数は y = log( x / ( 1 - x ) ) [ 0 < x < 1 ] で定義され、確率πの関数と見ることができる。

         スコア法では連結関数の x の値は期待値μ = Nπを表すため、利用する際に変換が必要となることに注意。
         具体的には以下のようになる ( N = 1 の場合、変換は不要となる)。

         g(μ) → g(π) = g(μ/N)

         g'(μ) → g'(π) / N

         π = g^-1(ξ) → μ = Ng^-1(ξ)
      **/
      struct LinkFunction_Logit : public LinkFunction_IF
      {
        // 連結関数 g(x)
        //
        // x→+0 のとき log( x / ( 1 - x ) ) → -∞
        // x↑1 のとき log( x / ( 1 - x ) ) → ∞
        // x<0||x>1 のとき log( x / ( 1 - x ) ) = NaN
        // 下式で上記の値になること確認済
        virtual stat_type operator()( stat_type x ) const
        { return( std::log( x / ( 1 - x ) ) ); }

        // 導関数 g'(x)
        virtual stat_type df( stat_type x ) const
        { return( ( x < 0 || x > 1 ) ? distribution::Base::nan() : 1 / ( x * ( 1 - x ) ) ); }

        // 逆関数 g^-1(y)
        //
        // e^y→±∞ のとき e^y / ( 1 + e^y ) = 1 - 1 / ( 1 + e^y ) → 1
        virtual stat_type invf( stat_type y ) const
        {
          stat_type ey = std::exp( y );
          return( ( std::isinf( ey ) ) ? 1 : ey / ( 1 + ey ) );
        }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Logit Function" ); }
      };

      /**
         @brief Complementary Log-log 連結関数

         連結関数としてComplementary Log-log関数を利用したもの。
         Complementary Log-log関数は y = log( -log( 1 - x ) ) [ 0 < x < 1 ] で定義され、確率πの関数と見ることができる。

         スコア法では連結関数の x の値は期待値μ = Nπを表すため、利用する際に変換が必要となることに注意。
         具体的には以下のようになる ( N = 1 の場合、変換は不要となる)。

         g(μ) → g(π) = g(μ/N)

         g'(μ) → g'(π) / N

         π = g^-1(ξ) → μ = Ng^-1(ξ)
      **/
      struct LinkFunction_Loglog : public LinkFunction_IF
      {
        // 連結関数 g(x)
        virtual stat_type operator()( stat_type x ) const
        { return( std::log( -std::log( 1 - x ) ) ); }

        // 導関数 g'(x)
        //
        // x→+0 のとき 1 / ( x - 1 )log( 1 - x ) → -∞
        // x↑1 のとき 1 / ( x - 1 )log( 1 - x ) → ∞
        // 下式で x→+0 のときは一致するが、x↑1 のときでは NaN になるため場合分け
        virtual stat_type df( stat_type x ) const
        {
          return(
                 ( x < 0 || x > 1 ) ? distribution::Base::nan() :
                 ( x == 1 ) ? distribution::Base::infinity() : 1 / ( ( x - 1 ) * std::log( 1 - x ) )
                 );
        }

        // 逆関数 g^-1(y)
        //
        // e^y→∞ のとき e^( -e^y )→0
        // 値が一致することは確認済
        virtual stat_type invf( stat_type y ) const
        { return( 1 - std::exp( -std::exp( y ) ) ); }

        // 属性を表す文字列
        virtual std::string ident() const
        { return( "Complementary Log-log Function" ); }
      };

      /**
         @brief 一般化線形モデル

         指数型分布族 ExpFamily_IF と連結関数 LinkFunction_IF を使い、線形式の係数をスコア法により求める。

         連結関数 g(μ) を使い、g(μ) = g(E[y]) = ( x, a ) (内積) の形の線形式で表せるとし、
         かつ y が指数型分布族に従うと仮定して係数 a を求める。
         g(μ) = μ (恒等関数) で指数型分布族が正規分布なら、一般化線形モデルは重回帰モデルと一致する。
      **/
      class Generic : public Base
      {
      public:

        /// @brief 独立変数ベクトル・従属変数・指数型分布族・連結関数を指定して構築
        ///
        /// 独立変数ベクトル(X)は、行方向にベクトルの要素が、列方向に各ベクトルが並んだ行列で表される。
        /// ベクトルの数(標本数)は従属変数(Y)の要素数で決まる。そのため、X行列の行数とYの要素数は一致させる必要がある。
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数、従属変数の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< distribution::Base::stat_type >::size_type > を投げる。
        /// - 従属変数の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
        /// - デザイン行列の行数と従属変数の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< distribution::Base::stat_type >::size_type > を投げる。
        ///
        /// @param x 独立変数ベクトル(X)
        /// @param ys, ye 従属変数(Y)の開始・末尾の次の位置
        /// @param pdf 指数型分布族への参照
        /// @param g 連結関数への参照
        template< class In >
        Generic( const Matrix< stat_type >& x, In ys, In ye,
                 const ExpFamily_IF& pdf, const LinkFunction_IF& g )
          : x_( x ), pdf_( &pdf ), g_( &g ),
            la_( stat_type() ), lmax_( stat_type() ), lmin_( stat_type() )
        { initY( ys, ye ); }

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする
        /// ( math_lib::convergence_test::BySum か math_lib::convergence_test::ByMax の利用を想定している)。
        ///
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        /// @return なし
        template< class Solver, class Test >
        void scoringMethod( Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT )
        { Base::scoringMethod( &a_, &var_a_, solver, test, maxCount ); }

        /// @brief 指数型分布族の変更
        ///
        /// @param pdf 変更する指数型分布族への参照
        void setExpFamily( ExpFamily_IF& pdf )
        { pdf_ = &pdf; }

        /// @brief 連結関数の変更
        ///
        /// @param g 変更する連結関数への参照
        void setLinkFunc( LinkFunction_IF& g )
        { g_ = &g; }

        /// @brief 係数値の取得
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 係数値の出力先
        template< class Out >
        void a( Out out ) const
        { std::copy( a_.begin(), a_.end(), out ); }

        /// @brief 係数値の分散の取得
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 係数値の分散の出力先
        template< class Out >
        void var_a( Out out ) const
        { std::copy( var_a_.begin(), var_a_.end(), out ); }

        /// @brief 従属変数の当てはめ値の取得
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 当てはめ値の出力先
        template< class Out >
        void fitValue( Out out ) const
        { std::copy( yExp_.begin(), yExp_.end(), out ); }

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( la_ ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @todo 最大モデルの対数尤度関数は、独立変数が等しいものを一つにまとめ、
        ///       係数の数を独立変数の異なるパラメータの数と等しくなるようにしなければならない。
        ///       現在、全独立変数が異なるものとして計算しているため、近々対応が必要になる。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( lmax_ ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( lmin_ ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( ( lmax() - la() ) * 2 ); }

        /// @brief ピアソン・カイ二乗統計量 X^2 を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return ピアソン・カイ二乗統計量 X^2
        stat_type pearsonChiSq() const
        { return( ( yExp_.size() != y_.size() ) ? 0 :
                  PearsonChiSq( y_.begin(), y_.end(), yExp_.begin(), yExp_.end() ) ); }

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( ( la() - lmin() ) * 2 ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( 1 - la() / lmin() ); }

        /// @brief 逸脱度残差 d を求める
        ///
        /// @note 逸脱度残差は、対数尤度統計量の和の成分を分解して平方根を計算することで得られる。
        ///       指数型分布族によっては和の成分が常に正になるとは限らない。
        ///       負数だった場合、逸脱度残差は NaN となることに注意。
        ///
        /// @param out 逸脱度残差の出力先
        template< class Out >
        void devianceResiduals( Out out ) const;

        /// @brief ピアソン残差 Xi を求める
        ///
        /// @param out ピアソン残差の出力先
        template< class Out >
        void pearsonResiduals( Out out ) const;

        /// @brief 予測値の計算
        ///
        /// - 独立変数ベクトル [ xs, xe ) の範囲が適切でない場合、
        ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
        /// - 独立変数ベクトル [ xs, xe ) のサイズは係数列のサイズと一致している必要がある。
        ///   一致しない場合は例外 exception::NotEqualLength< std::iterator_traits< In >::difference_type > を投げる。
        ///
        /// @note Generic::scoringMethod を実行しなければ係数の推定値は得られないため、
        ///       独立変数ベクトル [ xs, xe ) のサイズが正しくても例外を投げることに注意。
        ///
        /// @param xs 独立変数ベクトルの開始位置
        /// @param xe 独立変数ベクトルの末尾の次の位置
        template< class In >
        stat_type predict( In xs, In xe ) const;

        /// @brief 指数型分布族の属性を返す
        ///
        /// @return 指数型分布族の属性
        std::string pdf() const
        { return( pdf_->ident() ); }

        /// @brief 連結関数の属性を返す
        ///
        /// @return 連結関数の属性
        std::string linkFunc() const
        { return( g_->ident() ); }

      protected:

        using size_type = Matrix< stat_type >::size_type; ///< サイズの型

        // 引数として入力する変数への参照・ポインタ
        const Matrix< stat_type >& x_; ///< 独立変数ベクトルへの参照
        std::vector< stat_type > y_;   ///< 従属変数(観測値)への参照
        const ExpFamily_IF* pdf_;  ///< 指数型分布族へのポインタ
        const LinkFunction_IF* g_; ///< 連結関数へのポインタ

        // 推定値
        std::vector< stat_type > a_;     ///< 求めた係数を保持する配列
        std::vector< stat_type > var_a_; ///< 求めた係数の分散値を保持する配列
        std::vector< stat_type > yExp_;  ///< 従属変数(当てはめ値)

        stat_type la_;   ///< 当てはめ値に対する最大対数尤度
        stat_type lmax_; ///< 最大モデルの最大対数尤度
        stat_type lmin_; ///< 最小モデルの最大対数尤度

        /// @brief 独立変数と従属変数だけを指定して構築
        ///
        /// 独立変数ベクトル(X)は、行方向にベクトルの要素が、列方向に各ベクトルが並んだ行列で表される。
        /// ベクトルの数(標本数)は従属変数(Y)の要素数で決まる。そのため、X行列の行数とYの要素数は一致させる必要がある。
        ///
        /// 指数型分布族と連結関数は後で指定する。派生クラスでの利用を想定しているため protected アクセスである。
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< size_type > を投げる。
        /// - 従属変数の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
        /// - デザイン行列の行数と従属変数の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< size_type > を投げる。
        ///
        /// @param x 独立変数ベクトル(X)
        /// @param ys, ye 従属変数(Y)の開始と末尾の次の位置
        template< class In >
        Generic( const Matrix< stat_type >& x, In ys, In ye )
          : x_( x ), pdf_( 0 ), g_( 0 ),
            la_( stat_type() ), lmax_( stat_type() ), lmin_( stat_type() )
        { initY( ys, ye ); }

      private:

        // 計算に利用する値
        size_type n_; // 独立変数ベクトル xi の数
        size_type p_; // 独立変数ベクトルの要素数
        std::vector< stat_type > mu_;  // y の期待値
        std::vector< stat_type > dxi_; // g'(mu)
        std::vector< stat_type > w_;   // 対角行列の要素

        // g'(mu) と対角行列 W の計算
        void calcTerm();

        // x から係数行列を、y から解を計算し、連立方程式を構築する
        void createLES( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb );

        // 当てはめ値・予測値を計算する
        template< class In1, class In2 >
        stat_type fit( In1 s1, In1 e1, In2 s2 ) const
        { return( g_->invf( std::inner_product( s1, e1, s2, stat_type() ) ) ); }

        // 係数計算のための連立方程式と係数を保持する配列を初期化する
        virtual void init( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb, std::vector< stat_type >* a );

        // 係数計算のための連立方程式を作成する
        virtual void create( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb );

        // 係数計算後の更新処理を行う
        virtual void update( const std::vector< stat_type >& a );

        // 反復処理完了後に必要な処理を行う
        virtual void cleanUp()
        { fit(); calcLa(); calcLmax(); calcLmin(); }

        // 従属変数 y の初期化
        template< class In >
        void initY( In ys, In ye );

        // 当てはめ値の計算
        void fit();

        // 最大対数尤度の計算
        virtual void calcLa();   // 当てはめ値
        virtual void calcLmax(); // 最大モデル
        virtual void calcLmin(); // 最小モデル
      };

    } // namespace glm

    /**
       @brief 二値データから計数データへの変換

       独立変数列ごとに二値データを集計し、総数と成功回数を保持する。
       結果は std::map 形式で保持され(このクラスが std::map からの派生クラスとなる)、
       キーに独立変数、値に総数と成功回数からなる std::pair を持つ。
    **/
    template< class X, class Less = std::less< X > >
    struct Bin2Cnt : public std::map< X, std::pair< distribution::Base::counter_type, distribution::Base::counter_type >, Less >
    {
      /// @brief 計数データの型
      using counter_type = distribution::Base::counter_type;
      /// @brief 反復子の型
      using iterator = typename std::map< X, std::pair< counter_type, counter_type >, Less >::iterator;
      /// @brief 定数反復子の型
      using const_iterator = typename std::map< X, std::pair< counter_type, counter_type >, Less >::const_iterator;

      /// @brief 独立変数列と二値データ列を指定して構築
      ///
      /// - 独立変数列 [ xs, xe ) の指定範囲が有効かを内部でチェックしている。
      ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
      /// - 二値データ列 [ bin_s, bin_e ) の指定範囲が有効かを内部でチェックしている。
      ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
      /// - 独立変数列と二値データ列の要素数が一致していなければならない。
      ///   そうでない場合、例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
      ///
      /// @param xs 独立変数列の開始位置
      /// @param xe 独立変数列の末尾の次の位置
      /// @param bin_s 二値データ列の開始位置
      /// @param bin_e 二値データ列の末尾の次の位置
      template< class In1, class In2 >
      Bin2Cnt( In1 xs, In1 xe, In2 bin_s, In2 bin_e );

      /// @brief 独立変数と二値データのペア列を指定して構築
      ///
      /// データ列の要素は、独立変数をキー、二値データを値とした std::pair であることを想定している。
      ///
      /// - データ列 [ s, e ) の指定範囲が有効かを内部でチェックしている。
      ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
      ///
      /// @param s ペア列の開始位置
      /// @param e ペア列の末尾の次の位置
      template< class In >
      Bin2Cnt( In s, In e );
    };

    namespace glm
    {
      /**
         @brief 二項分布を確率分布とした一般化線形モデル(ロジスティック回帰)

         指数型分布族として各試行ごとに二項分布を定義し、線形式の係数をスコア法により求める。
      **/
      class BinomialLogistic : private Generic
      {
      public:

        /// @brief 独立変数ベクトルと、各試行の総試行回数・成功回数を指定して構築
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< stat_type >::size_type > を投げる。
        /// - 各試行の総試行回数の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
        /// - 各試行の成功回数の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
        /// - 独立変数ベクトルの数(標本数)と総試行回数・成功回数の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< stat_type >::size_type > を投げる。
        /// - 各標本の成功回数はゼロ以上でなければならない。
        ///   そうでない場合、例外 exception::NegativeNumber< stat_type > を投げる。
        /// - 各標本の総試行回数は正数でなければならない。
        ///   そうでない場合、例外 exception::NotPositiveNumber< stat_type > を投げる。
        /// - 各標本の成功回数は総試行回数以下でなければならない。
        ///   そうでない場合、例外 exception::TooGreatNumber< stat_type > を投げる。
        ///
        /// @note 成功回数がゼロ・または総試行回数と等しいとき、推定確率の初期値は 0, 1 となるため、
        ///       計算時に NaN が発生してしまう。これを防止する目的で微小値を加減している。
        ///
        /// @param x 独立変数ベクトルへの参照
        /// @param ns, ne 各試行の総試行回数の開始・末尾の次の位置
        /// @param ys, ye 各試行の成功回数の開始・末尾の次の位置
        /// @param g 連結関数への参照
        /// @param h 微小値(π=0,1のときに計算結果が NaN になるのを防ぐためのしきい値)
        template< class In1, class In2 >
        BinomialLogistic( const Matrix< stat_type >& x, In1 ns, In1 ne, In2 ys, In2 ye,
                          const LinkFunction_IF& g, stat_type h = DEFAULT_THRESHOLD );

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする
        /// ( math_lib::convergence_test::BySum か math_lib::convergence_test::ByMax の利用を想定している)。
        ///
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        /// @return なし
        template< class Solver, class Test >
        void scoringMethod( Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT )
        { Generic::scoringMethod( solver, test, maxCount ); }

        /// @brief 連結関数の変更
        ///
        /// @param g 変更する連結関数への参照
        void setLinkFunc( const LinkFunction_IF& g )
        {
          sf_ = LinkFunction_Sigmoid( g );
          sf_.assign( n_.begin(), n_.end() );
          Generic::setLinkFunc( sf_ );
        }

        /// @brief 係数値の取得
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 係数値の出力先
        template< class Out >
        void a( Out out ) const
        { Generic::a( out ); }

        /// @brief 係数値の分散の取得
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 係数値の分散の出力先
        template< class Out >
        void var_a( Out out ) const
        { Generic::var_a( out ); }

        /// @brief 推定確率を取得する
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 推定確率の出力先
        /// @return なし
        template< class Out >
        void pi( Out out ) const;

        /// @brief 従属変数の当てはめ値の取得
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 当てはめ値の出力先
        template< class Out >
        void fitValue( Out out ) const
        { Generic::fitValue( out ); }

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( Generic::la() ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @todo 最大モデルの対数尤度関数は、独立変数が等しいものを一つにまとめ、
        ///       係数の数を独立変数の異なるパラメータの数と等しくなるようにしなければならない。
        ///       現在、全独立変数が異なるものとして計算しているため、近々対応が必要になる。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( Generic::lmax() ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( Generic::lmin() ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( Generic::logLikelihood() ); }

        /// @brief ピアソン・カイ二乗統計量 X^2 を求める
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return ピアソン・カイ二乗統計量 X^2
        stat_type pearsonChiSq() const;

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( Generic::likelihoodChiSq() ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( Generic::pseudoRSq() ); }

        /// @brief 逸脱度残差 d を求める
        ///
        /// @param out 逸脱度残差の出力先
        template< class Out >
        void devianceResiduals( Out out ) const
        { Generic::devianceResiduals( out ); }

        /// @brief ピアソン残差 Xi を求める
        ///
        /// @note BinomialLogistic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out ピアソン残差の出力先
        /// @return なし
        template< class Out >
        void pearsonResiduals( Out out ) const;

        /// @brief 指数型分布族の属性を返す
        ///
        /// @return 指数型分布族の属性
        std::string pdf() const
        { return( bin_.ident() ); }

        /// @brief 連結関数の属性を返す
        ///
        /// @return 連結関数の属性
        std::string linkFunc() const
        { return( sf_.ident() ); }

      private:

        /*
          一母数指数型分布族(複数パラメータの二項分布)

          A(η)・平均・η(E[y])・分散はそれぞれ、計算結果を返す際に次の試行を計算するよう切り替えられる。
          処理を反復させれば、個々の試行に対する値を自動的に得ることができる。
        */
        class ExpFamily_MultiBinomial : public ExpFamily_IF
        {
        public:

          // 総試行回数のアサイン
          template< class In >
          void assign( In s, In e )
          {
            aCnt_ = aveCnt_ = aveInvCnt_ = varCnt_ = bCnt_ = 0;

            n_.assign( s, e );
          }
          // A(η)
          virtual stat_type a( stat_type eta ) const
          {
            stat_type ans = n_[aCnt_] * std::log( std::exp( eta ) + 1.0 );
            aCnt_ = ( aCnt_ + 1 ) % n_.size();
            return( ans );
          }

          // 期待値 ( E[y] = A'(η) )
          virtual stat_type average( stat_type eta ) const
          {
            stat_type ans = n_[aveCnt_] * eta2Pi( eta );
            aveCnt_ = ( aveCnt_ + 1 ) % n_.size();
            return( ans );
          }

          // A(η) の導関数の逆関数 ( η = A'^(-1)(y) )
          virtual stat_type aveInv( stat_type mu ) const
          {
            stat_type eta = pi2Eta( mu / n_[aveInvCnt_] );
            aveInvCnt_ = ( aveInvCnt_ + 1 ) % n_.size();
            return( eta );
          }

          // A(η) の二階導関数 = 分散 ( V[y] = A''(η) )
          virtual stat_type variance( stat_type eta ) const
          {
            stat_type p = eta2Pi( eta );
            stat_type ans = n_[varCnt_] * p * ( 1.0 - p );
            varCnt_ = ( varCnt_ + 1 ) % n_.size();
            return( ans );
          }

          // B(y) = log( h(y) )
          // 各試行を分解して n = 1 ; y = { 0, 1 } としたとき、h(y) = nCy は必ず 1 となるので無視できる。
          // ここでは、必ずゼロを返す仕様としている。
          // 参考文献 : 一般化線形モデル入門 (共立出版)
          virtual stat_type b( stat_type y ) const
          { return( 0 ); }

          // 属性を表す文字列
          virtual std::string ident() const
          { return( "Binomial Distribution (multiple parameters)" ); }

        private:

          using size_type = std::vector< stat_type >::size_type;

          std::vector< stat_type > n_; // 各 x に対応する二項分布の総試行回数

          // 各関数の利用回数(n_.size()の剰余)
          mutable size_type aCnt_;      // a()
          mutable size_type aveCnt_;    // average()
          mutable size_type aveInvCnt_; // aveInv()
          mutable size_type varCnt_;    // variance()
          mutable size_type bCnt_;      // b()

          // eta2Pi : η = eta に対する π の値を返す
          static stat_type eta2Pi( stat_type eta )
          {
            stat_type pi = 1.0 - 1.0 / ( std::exp( eta ) + 1.0 );
            //if ( pi == 0 ) pi = h_;
            //if ( pi == 1.0 ) pi -= h_;

            return( pi );
          }

          // pi2Eta : π = pi に対する η の値を返す
          static stat_type pi2Eta( stat_type pi )
          {
            return( std::log( pi / ( 1.0 - pi ) ) );
          }
        };

        /*
          シグモイド型連結関数

          シグモイド型関数( ProbitModelFunc, LogitFunc, LoglogFunc など)を利用して、複数の試行に対する
          各試行の関数値 g(x)、導関数値 g'(x)、逆関数 g^-1(y) を返す。
          これらの関数はそれぞれ、計算結果を返す際に次の試行を計算するよう切り替えられる。処理を反復させ
          れば、個々の試行に対する値を自動的に得ることができる。
        */
        class LinkFunction_Sigmoid : public LinkFunction_IF
        {
        public:

          // シグモイド型連結関数 linkFunc を指定して構築
          LinkFunction_Sigmoid( const LinkFunction_IF& linkFunc )
            : linkFunc_( &linkFunc ) {}

          // 総試行回数のアサイン
          template< class In >
          void assign( In s, In e )
          {
            opCnt_ = dfCnt_ = invfCnt_ = 0;
            n_.assign( s, e );
          }

          // 連結関数 g(x)
          virtual stat_type operator()( stat_type x ) const
          {
            stat_type pi = x / n_[opCnt_];
            //if ( pi == 0 ) pi = h_;
            //if ( pi == 1.0 ) pi -= h_;

            stat_type d = (*linkFunc_)( pi );
            opCnt_ = ( opCnt_ + 1 ) % n_.size();
            return( d );
          }

          // 導関数 g'(x)
          virtual stat_type df( stat_type x ) const
          {
            stat_type pi = x / n_[dfCnt_];
            //if ( pi == 0 ) pi = h_;
            //if ( pi == 1.0 ) pi -= h_;

            stat_type d = linkFunc_->df( pi ) / n_[dfCnt_];
            dfCnt_ = ( dfCnt_ + 1 ) % n_.size();
            return( d );
          }

          // 逆関数 g^-1(y)
          virtual stat_type invf( stat_type y ) const
          {
            stat_type d = linkFunc_->invf( y ) * n_[invfCnt_];
            invfCnt_ = ( invfCnt_ + 1 ) % n_.size();
            return( d );
          }

          // 属性を表す文字列
          virtual std::string ident() const
          { return( linkFunc_->ident() ); }

        private:

          using size_type = std::vector< stat_type >::size_type;

          const LinkFunction_IF* linkFunc_;      // 利用する連結関数
          std::vector< stat_type > n_; // 各 x に対応する二項分布の総試行回数

          // 各関数の利用回数(n_.size()の剰余)
          mutable size_type opCnt_;   // operator()()
          mutable size_type dfCnt_;   // df()
          mutable size_type invfCnt_; // invf()
        };

        // 計算に利用する変数
        std::vector< stat_type > n_; // 各試行の総試行回数

        // 指数型分布族と連結関数
        ExpFamily_MultiBinomial bin_; // 各 x 用の二項分布
        LinkFunction_Sigmoid sf_;     // 各 x 用の連結関数

        virtual void calcLmin(); // 最小モデルの最大対数尤度
      };

      /**
         @brief ホズマー・レメショウ・テスト

         連続値に対するロジスティック回帰への当てはめ値を元に独立変数をグループ化して試行回数と成功回数を求め、
         ピアソン・カイ二乗統計量(ホズマー・レメショウ統計量)を計算する。ホズマー・レメショウ統計量は自由度を
         ( グループ数 - 2 ) とするカイ二乗分布に従う。

         連続値の独立変数ベクトル [ xs, xe ) とその成功/失敗を表す二値データ [ bin_s, bin_e ) がある場合、
         statistics::Bin2Cnt クラスと statistics::glm::BinomialLogistic クラスを組み合わせることで以下のようにホズマー・レメショウ・テストを行うことができる。

         @code
         statistics::Bin2Cnt( xs, xe, bin_s, bin_e ); // 二値データから計数データへ
         // 独立変数ベクトルからなる行列 x と総試行回数・成功回数を持つベクトル n, y を作成
         :
         statistics::glm::BinomialLogistic glmBin( x, n.begin(), n.end(), y.begin(), y.end(), logit ); // ロジスティック回帰
         glmBin.scoringMethod( solver, test );   // スコア法
         glmBin.pi( std::back_inserter( pi ) );  // 推定確率を計算する
         statistics::glm::HosmerLemeshowTest( n, y, pi, grpCnt ); // ホズマー・レメショウ・テスト
         @endcode
      **/
      class HosmerLemeshowTest
      {
        using stat_type = distribution::Base::stat_type;
        using counter_type = distribution::Base::counter_type;

        std::vector< std::pair< stat_type, stat_type > > pi_; // グループごとの確率の範囲

        std::vector< stat_type > se_; // グループごとの成功回数期待値
        std::vector< stat_type > so_; // グループごとの成功回数実測値
        std::vector< stat_type > fe_; // グループごとの失敗回数期待値
        std::vector< stat_type > fo_; // グループごとの失敗回数実測値

        stat_type chisqHL_; // X^2HL

      public:

        /// @brief 各試行の総試行回数とその成功回数、確率の期待値を指定して構築
        ///
        /// @param ns, ne 各試行の総試行回数の開始・末尾の次の位置
        /// @param ys, ye 各試行の成功回数の開始・末尾の次の位置
        /// @param pis, pie 各試行の確率の期待値の開始・末尾の次の位置
        /// @param grpCnt 分割するグループ数
        template< class InN, class InY, class InPi >
        HosmerLemeshowTest( InN ns, InN ne, InY ys, InY ye, InPi pis, InPi pie,
                            distribution::Base::counter_type grpCnt = 10 );

        /// @brief 試行の数を返す
        ///
        /// @return 試行の数
        size_t size() const
        { return( pi_.size() ); }

        /// @brief 指定した番号の試行の確率(期待値)の範囲を返す
        ///
        /// @param i 試行の番号
        /// @return 確率(期待値)の範囲
        std::pair< distribution::Base::stat_type, distribution::Base::stat_type > pi( size_t i ) const
        { return( pi_.at( i ) ); }

        /// @brief 指定した番号の成功回数(期待値)を返す
        ///
        /// @param i 試行の番号
        /// @return 成功回数(期待値)
        distribution::Base::stat_type se( size_t i ) const
        { return( se_.at( i ) ); }

        /// @brief 指定した番号の成功回数(実測値)を返す
        ///
        /// @param i 試行の番号
        /// @return 成功回数(実測値)
        distribution::Base::stat_type so( size_t i ) const
        { return( so_.at( i ) ); }

        /// @brief 指定した番号の失敗回数(期待値)を返す
        ///
        /// @param i 試行の番号
        /// @return 失敗回数(期待値)
        distribution::Base::stat_type fe( size_t i ) const
        { return( fe_.at( i ) ); }

        /// @brief 指定した番号の失敗回数(実測値)を返す
        ///
        /// @param i 試行の番号
        /// @return 失敗回数(実測値)
        distribution::Base::stat_type fo( size_t i ) const
        { return( fo_.at( i ) ); }

        /// @brief ホズマー・レメショウ統計量 Χ^2_HL を返す
        ///
        /// @return ホズマー・レメショウ統計量 Χ^2_HL
        distribution::Base::stat_type chisqHL() const
        { return( chisqHL_ ); }

        /// @brief ホズマー・レメショウ統計量 Χ^2_HL の p 値を返す
        ///
        /// @return ホズマー・レメショウ統計量 Χ^2_HL の p 値
        distribution::Base::stat_type p() const
        {
          if ( size() < 3 ) return( distribution::Base::nan() );

          distribution::ChiSquare chi( size() - 2 );
          return( 1.0 - chi.lower_p( chisqHL() ) );
        }
      };

      /**
         @brief 多項分布を確率分布とした一般化線形モデル(名義ロジスティック回帰)
      **/
      class MultinomialLogistic : public Base
      {
      public:

        /// @brief 添字の型
        using size_type = Matrix< stat_type >::size_type;

        /// @brief 独立変数と従属変数を指定して構築
        ///
        /// N は独立変数ベクトルの数、P は独立変数ベクトルの要素数、C はカテゴリの数としたとき、
        /// x は N x P 行列、y は N x C 行列である必要がある。
        ///
        /// - N, P, C のいずれかがゼロ以下だった場合、
        ///   例外 exception::NotPositiveLength< size_type > を投げる。
        /// - y の行数が N と等しくなかった場合、
        ///   例外 exception::NotEqualLength< size_type > を投げる。
        ///
        /// @param x 独立変数(N x p 行列)
        /// @param y 従属変数(各カテゴリの発生回数 ; N x C 行列)
        MultinomialLogistic( const Matrix< stat_type >& x, const Matrix< stat_type >& y );

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする
        /// ( math_lib::convergence_test::BySum か math_lib::convergence_test::ByMax の利用を想定している)。
        ///
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        /// @return なし
        template< class Solver, class Test >
        void scoringMethod( Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT )
        { Base::scoringMethod( &a_, &var_a_, solver, test, maxCount ); }

        /// @brief 推定係数を取得する
        ///
        /// 結果は ( C - 1 ) x P 行列の形で得られる。1 番目の独立変数に対する発生確率を基準とした各独立変数の発生確率の比の対数について、
        ///
        /// log( πik / πi1 ) = Σj{1→p}( αkj・xij ) ( xij は i 番目の独立変数の j 番目の要素 )
        ///
        /// を満たす係数 αkj を返す。
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ正しい結果は得られないことに注意。
        ///       その場合は要素が全てゼロの 1 x P 行列が得られる。
        ///
        /// @param mat 係数を取得するための行列へのポインタ
        /// @return なし
        void a( Matrix< stat_type >* mat ) const
        {
          assert( mat != nullptr );
          mat->assign( a_.begin(), a_.end(), x_.cols(), false );
        }

        /// @brief 推定係数の分散を取得する
        ///
        /// 結果は ( C - 1 ) x P 行列の形で得られる。1 番目の独立変数に対する発生確率を基準とした各独立変数の発生確率の比の対数について、
        ///
        /// log( πik / πi1 ) = Σj{1→p}( αkj・xij ) ( xij は i 番目の独立変数の j 番目の要素 )
        ///
        /// を満たす係数 αkj の分散値を返す。
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ正しい結果は得られないことに注意。
        ///       その場合は要素が全てゼロの 1 x P 行列が得られる。
        ///
        /// @param mat 係数の分散を取得するための行列へのポインタ
        /// @return なし
        void var_a( Matrix< stat_type >* mat ) const
        {
          assert( mat != nullptr );
          mat->assign( var_a_.begin(), var_a_.end(), x_.cols(), false );
        }

        /// @brief 推定確率を取得する
        ///
        /// 結果は N x C 行列の形で得られる。
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ空の行列しか得られないことに注意。
        ///
        /// @param mat 推定確率を取得するための行列へのポインタ
        /// @return なし
        void pi( Matrix< stat_type >* mat ) const
        {
          assert( mat != nullptr );
          *mat = pi_;
        }

        /// @brief 当てはめ値(発生回数の推定値)を取得する
        ///
        /// 結果は N x C 行列の形で得られる。
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ空の行列しか得られないことに注意。
        ///
        /// @param mat 当てはめ値を取得するための行列へのポインタ
        /// @return なし
        void fitValue( Matrix< stat_type >* mat ) const
        {
          assert( mat != nullptr );
          *mat = yExp_;
        }

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( la_ ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        /// @note 最大モデルの対数尤度関数は、独立変数 x が全て相異なることを前提に計算していることに注意。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( lmax_ ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( lmin_ ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( ( lmax() - la() ) * 2 ); }

        /// @brief ピアソン・カイ二乗統計量 X^2 を求める
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return ピアソン・カイ二乗統計量 X^2
        stat_type pearsonChiSq() const
        {
          if ( yExp_.rows() == 0 && yExp_.cols() == 0 ) return( 0 );
          return( PearsonChiSq( y_.begin(), y_.end(), yExp_.begin(), yExp_.end() ) );
        }

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( ( la() - lmin() ) * 2 ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( 1 - la() / lmin() ); }

        /// @brief ピアソン残差 Xi を求める
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ行列の要素は全てゼロとなることに注意。
        ///
        /// @param res ピアソン残差の出力先
        /// @return なし
        void pearsonResiduals( Matrix< stat_type >* res ) const;

        /// @brief オッズ比を求める
        ///
        /// オッズ比は、二つのオッズ ρr1,c = πr1,c / πr1,1 と ρr2,c = πr2,c / πr2,1 の比から求められる。
        ///
        /// - 無効な添字を指定した場合、例外 std::out_of_range を投げる。
        ///
        /// @note glm::MultinomialLogistic::scoringMethod を実行しなければ結果は得られない
        ///       (どの引数を指定しても例外 std::out_of_range が投げられる)
        ///
        /// @return オッズ比
        stat_type oddsRatio( size_type r1, size_type r2, size_type c ) const
        { return( ( pi_.row( r1 ).at( c ) * pi_.row( r2 ).at( 0 ) ) / ( pi_.row( r2 ).at( c ) * pi_.row( r1 ).at( 0 ) ) ); }

      private:

        // 引数として入力される変数への参照
        const Matrix< stat_type >& x_; // 独立変数ベクトル
        const Matrix< stat_type >& y_; // 従属変数(観測値)

        // 計算に利用する変数
        size_type n_; // 独立変数ベクトル x_i の数
        size_type p_; // 独立変数ベクトルの要素数
        size_type c_; // カテゴリ数
        std::vector< stat_type > ni_; // 各独立変数の総数 ( n )
        Matrix< stat_type > pi_;      // 確率の当てはめ値 ( n x c )
        std::vector< stat_type > w_;  // 対角行列 W_kk'

        // 結果として得られる推定値
        std::vector< stat_type > a_;     // 計算した係数を保持する行列
        std::vector< stat_type > var_a_; // 計算した係数の分散値を保持する行列
        Matrix< stat_type > yExp_;       // 当てはめ値

        stat_type la_;   // 当てはめ値による最大対数尤度
        stat_type lmax_; // 最大モデルの最大対数尤度
        stat_type lmin_; // 最小モデルの最大対数尤度

        // 係数計算のための連立方程式と係数を保持する配列を初期化する
        virtual void init( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb, std::vector< stat_type >* a );

        // 係数計算のための連立方程式を作成する
        virtual void create( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb );

        // 係数計算後の更新処理を行う
        virtual void update( const std::vector< stat_type >& a );

        // 反復処理完了後に必要な処理を行う
        virtual void cleanUp();
      };

      /**
         @brief 累積ロジットモデル用基底クラス

         比例オッズモデル glm::ProportionalOddsModel 及び 累積ロジットモデル glm::CumulativeLogitModel のための
         基底クラスであり、単独での利用はできない。
      **/
      class CumulativeLogitModel_Base : public Base
      {
      protected:

        /// @brief サイズの型
        using size_type = Matrix< stat_type >::size_type;

      public:

        // 独立変数と従属変数(各カテゴリの発生回数)を指定して構築
        //
        // x 独立変数
        // y 従属変数(各カテゴリの発生回数)
        CumulativeLogitModel_Base( const Matrix< stat_type >& x, const Matrix< stat_type >& y );

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする
        /// ( math_lib::convergence_test::BySum か math_lib::convergence_test::ByMax の利用を想定している)。
        ///
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        /// @return なし
        template< class Solver, class Test >
        void scoringMethod( Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT )
        { Base::scoringMethod( &a_, &var_a_, solver, test, maxCount ); }

        /// @brief 独立変数ベクトルの数を返す
        ///
        /// @return 独立変数ベクトルの数
        size_t xSize() const
        { return( n_ ); }

        /// @brief 独立変数ベクトルの要素数を返す
        ///
        /// @return 独立変数ベクトルの要素数
        size_t varSize() const
        { return( p_ ); }

        /// @brief カテゴリ数を返す
        ///
        /// @return カテゴリ数
        size_t categorySize() const
        { return( c_ ); }

        /// @brief 当てはめ値(発生回数の推定値)を返す
        ///
        /// @param mat 当てはめ値を取得するための行列へのポインタ
        void fitValue( Matrix< stat_type >* mat ) const
        {
          assert( mat != nullptr );
          *mat = yExp_;
        }

        /// @brief 推定確率を返す
        ///
        /// @param pi 推定確率を取得するための行列へのポインタ
        void pi( Matrix< stat_type >* pi ) const
        {
          assert( pi != nullptr );
          *pi = pi_;
        }

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( la_ ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        /// @note 最大モデルの対数尤度関数は、独立変数 x が全て相異なることを前提に計算していることに注意。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( lmax_ ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( lmin_ ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( ( lmax() - la() ) * 2 ); }

        /// @brief ピアソン・カイ二乗統計量 X^2 を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return ピアソン・カイ二乗統計量 X^2
        stat_type pearsonChiSq() const
        { return( ( ( y_.rows() != yExp_.rows() ) || ( y_.cols() != yExp_.cols() ) ) ? 0 :
                  PearsonChiSq( y_.begin(), y_.end(), yExp_.begin(), yExp_.end() ) ); }

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( ( la_ - lmin_ ) * 2 ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( 1 - la_ / lmin_ ); }

        /// @brief ピアソン残差 Xi を求める
        ///
        /// @note scoringMethod を実行しなければ行列の要素は全てゼロとなることに注意。
        ///
        /// @param res ピアソン残差の出力先
        /// @return なし
        void pearsonResiduals( Matrix< stat_type >* res ) const;

        /// @brief オッズ比を求める
        ///
        /// オッズ比は、二つのオッズ ρr1,c = πr1,c / πr1,1 と ρr2,c = πr2,c / πr2,1 の比から求められる。
        ///
        /// - 無効な添字を指定した場合、例外 std::out_of_range を投げる。
        ///
        /// @note scoringMethod を実行しなければ結果は得られない
        ///       (どの引数を指定しても例外 std::out_of_range が投げられる)
        ///
        /// @return オッズ比
        stat_type oddsRatio( size_type r1, size_type r2, size_type c ) const
        { return( ( pi_.row( r1 ).at( c ) * pi_.row( r2 ).at( 0 ) ) / ( pi_.row( r2 ).at( c ) * pi_.row( r1 ).at( 0 ) ) ); }

      protected:

        const Matrix< stat_type >& x_; ///< 独立変数への参照
        const Matrix< stat_type >& y_; ///< 従属変数(各カテゴリの発生回数)への参照

        std::vector< stat_type > a_;     ///< 係数
        std::vector< stat_type > var_a_; ///< 係数の分散

        std::vector< stat_type > ni_; ///< 各独立変数の総数 ( n )
        Matrix< stat_type > rho_;     ///< ρ値(基準カテゴリの確率との比率)
        Matrix< stat_type > pi_;      ///< 確率の当てはめ値
        Matrix< stat_type > yExp_;    ///< 発生回数の当てはめ値

        // x は n_ x p_ の行列
        // y は n_ x c_ の行列
        size_type n_; ///< 独立変数ベクトルの数
        size_type p_; ///< 独立変数ベクトルの要素数
        size_type c_; ///< カテゴリ数

        stat_type la_;   ///< 当てはめ値から求めた最大対数尤度
        stat_type lmax_; ///< 最大モデルの最大対数尤度
        stat_type lmin_; ///< 最小モデルの最大対数尤度

      private:

        // 求めた係数から基準カテゴリの確率との比率 rho を計算する
        virtual void coef2Rho( const std::vector< stat_type >& a ) = 0;

        // 係数行列を計算する
        virtual void calcCoefMatrix( SquareMatrix< stat_type >* sa ) = 0;

        // 連立方程式の右辺を計算する
        virtual void calcRSide( std::vector< stat_type >* sb ) = 0;

        // 回帰係数の数を返す
        virtual size_t coefSize() const = 0;

        // 係数計算のための連立方程式と係数を保持する配列を初期化する
        virtual void init( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb, std::vector< stat_type >* a );

        // 係数計算のための連立方程式を作成する
        virtual void create( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb );

        // 係数計算後の更新処理を行う
        virtual void update( const std::vector< stat_type >& a );

        // 反復処理完了後に必要な処理を行う
        virtual void cleanUp();

        // 確率の当てはめ値 pi_ を計算する
        void calcPi();
      };

      /**
         @brief 順序ロジスティック(比例オッズモデル)

         潜在変数 Ui がカテゴリ Ck を境界にそれ以上になる累積確率と小さくなる確率の比率 F( Ui >= Ck ) / F( Ui < Ck ) が
         モデル式 exp( Xi・a - Ck ) に従うとするモデルで、定数項以外の係数は全てカテゴリに対して共通とする。
      **/
      class ProportionalOddsModel : public CumulativeLogitModel_Base
      {
      public:

        /// @brief 独立変数と従属変数(各カテゴリの発生回数)を指定して構築
        ///
        /// N は独立変数ベクトルの数、P は定数項を除いた独立変数ベクトルの要素数、C はカテゴリの数としたとき、
        /// x は N x P 行列、y は N x C 行列である必要がある。
        ///
        /// - N, P, C のいずれかがゼロ以下だった場合、
        ///   例外 exception::NotPositiveLength< size_type > を投げる。
        /// - y の行数が N と等しくなかった場合、
        ///   例外 exception::NotEqualLength< size_type > を投げる。
        ///
        /// @param x 独立変数
        /// @param y 従属変数(各カテゴリの発生回数)
        ProportionalOddsModel( const Matrix< stat_type >& x, const Matrix< stat_type >& y )
          : CumulativeLogitModel_Base( x, y ) {}

        /// @brief 定数項以外の項に対する係数を返す
        ///
        /// 定数項以外の項に対する係数は各カテゴリに対して共通である。P - 1 個の係数を返す。
        ///
        /// @param out 係数の出力先
        template< class Out >
        void a( Out out ) const
        { if ( a_.size() >= categorySize() - 1 ) std::copy( a_.begin() + categorySize() - 1, a_.end(), out ); }

        /// @brief 定数項に対する係数を返す
        ///
        /// 定数項に対する係数は各カテゴリに対して異なる。C - 1 個の定数項を返す。
        ///
        /// @param out 係数の出力先
        template< class Out >
        void b( Out out ) const
        { if ( a_.size() >= categorySize() - 1 ) std::copy( a_.begin(), a_.begin() + categorySize() - 1, out ); }

        /// @brief 定数項以外の項に対する係数の分散を返す
        ///
        /// 定数項以外の項に対する係数の分散は各カテゴリに対して共通である。P - 1 個の値を返す。
        ///
        /// @param out 係数の分散の出力先
        template< class Out >
        void var_a( Out out ) const
        { if ( a_.size() >= categorySize() - 1 ) std::copy( var_a_.begin() + categorySize() - 1, var_a_.end(), out ); }

        /// @brief 定数項に対する係数の分散を返す
        ///
        /// 定数項に対する係数の分散は各カテゴリに対して異なる。C - 1 個の値を返す。
        ///
        /// @param out 係数の分散の出力先
        template< class Out >
        void var_b( Out out ) const
        { if ( a_.size() >= categorySize() - 1 ) std::copy( var_a_.begin(), var_a_.begin() + categorySize() - 1, out ); }

      private:

        void calcH11( SquareMatrix< stat_type >* s ); // H11 の計算
        void calcH22( SquareMatrix< stat_type >* s ); // H22 の計算
        void calcH12( SquareMatrix< stat_type >* s ); // H12(H21) の計算

        // 回帰係数の数を返す
        virtual size_type coefSize() const
        { return( c_ + p_ - 1 ); }

        virtual void coef2Rho( const std::vector< stat_type >& a );   // 求めた係数から &rho; を計算する
        virtual void calcCoefMatrix( SquareMatrix< stat_type >* sa ); // 係数行列を計算する
        virtual void calcRSide( std::vector< stat_type >* sb );       // 連立方程式の右辺を計算する
      };

      /**
         @brief 順序ロジスティック(累積ロジットモデル)

         潜在変数 Ui がカテゴリ Ck を境界にそれ以上になる累積確率と小さくなる確率の比率 F( Ui >= Ck ) / F( Ui < Ck ) が
         モデル式 exp( Xi・a - Ck ) に従うとするモデルで、定数項以外の係数も全てカテゴリに対して異なるとする。
      **/
      class CumulativeLogitModel : public CumulativeLogitModel_Base
      {
      public:

        /// @brief 独立変数と従属変数(各カテゴリの発生回数)を指定して構築
        ///
        /// N は独立変数ベクトルの数、P は独立変数ベクトルの要素数、C はカテゴリの数としたとき、
        /// x は N x P 行列、y は N x C 行列である必要がある。
        ///
        /// - N, P, C のいずれかがゼロ以下だった場合、
        ///   例外 exception::NotPositiveLength< size_type > を投げる。
        /// - y の行数が N と等しくなかった場合、
        ///   例外 exception::NotEqualLength< size_type > を投げる。
        ///
        /// @param x 独立変数
        /// @param y 従属変数(各カテゴリの発生回数)
        CumulativeLogitModel( const Matrix< stat_type >& x, const Matrix< stat_type >& y )
          : CumulativeLogitModel_Base( x, y ) {}

        /// @brief 推定係数を取得する
        ///
        /// 結果は ( C - 1 ) x P 行列の形で得られる。
        ///
        /// @note scoringMethod を実行しなければ正しい結果は得られないことに注意。
        ///       その場合は要素が全てゼロの 1 x P 行列が得られる。
        ///
        /// @param mat 係数を取得するための行列へのポインタ
        /// @return なし
        void a( Matrix< stat_type >* mat )
        {
          assert( mat != nullptr );
          mat->assign( a_.begin(), a_.end(), varSize(), false );
        }

        /// @brief 推定係数の分散を取得する
        ///
        /// 結果は ( C - 1 ) x P 行列の形で得られる。
        ///
        /// @note scoringMethod を実行しなければ正しい結果は得られないことに注意。
        ///       その場合は要素が全てゼロの 1 x P 行列が得られる。
        ///
        /// @param mat 係数の分散を取得するための行列へのポインタ
        /// @return なし
        void var_a( Matrix< stat_type >* mat )
        {
          assert( mat != nullptr );
          mat->assign( var_a_.begin(), var_a_.end(), varSize(), false );
        }

      private:

        void calcCoefDiagBlock( SquareMatrix< stat_type >* sa, size_type k );    // 対角ブロックの計算
        void calcCoefNonDiagBlock( SquareMatrix< stat_type >* sa, size_type k ); // 対角以外のブロックの計算

        // 回帰係数の数を返す
        virtual size_type coefSize() const
        { return( ( c_ - 1 ) * p_ ); }

        virtual void coef2Rho( const std::vector< stat_type >& a );   // 求めた係数から &rho; を計算する
        virtual void calcCoefMatrix( SquareMatrix< stat_type >* sa ); // 係数行列を計算する
        virtual void calcRSide( std::vector< stat_type >* sb );       // 連立方程式の右辺を計算する
      };

      /**
         @brief ポアソン回帰

         ポアソン分布を指数型分布族に利用した一般化線形モデル
      **/
      class PoissonRegression : private Generic
      {
      public:

        /// @brief 独立変数と各カテゴリの総数・発生回数を指定して構築
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< stat_type >::size_type > を投げる。
        /// - 各試行の総試行回数の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
        /// - 各試行の成功回数の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
        /// - 独立変数ベクトルの数(標本数)と総試行回数・成功回数の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< stat_type >::size_type > を投げる。
        /// - 各標本の成功回数はゼロ以上でなければならない。
        ///   そうでない場合、例外 exception::NegativeNumber< stat_type > を投げる。
        /// - 各標本の総試行回数は正数でなければならない。
        ///   そうでない場合、例外 exception::NotPositiveNumber< stat_type > を投げる。
        /// - 各標本の成功回数は総試行回数以下でなければならない。
        ///   そうでない場合、例外 exception::TooGreatNumber< stat_type > を投げる。
        ///
        /// @param x 独立変数
        /// @param ns, ne 各カテゴリの総試行回数の開始・末尾の次の位置
        /// @param ys, ye 各カテゴリの発生回数の開始・末尾の次の位置
        template< class In1, class In2 >
        PoissonRegression( const Matrix< stat_type >& x, In1 ns, In1 ne, In2 ys, In2 ye );

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする
        /// ( math_lib::convergence_test::BySum か math_lib::convergence_test::ByMax の利用を想定している)。
        ///
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        /// @return なし
        template< class Solver, class Test >
        void scoringMethod( Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT )
        { Generic::scoringMethod( solver, test, maxCount ); }

        /// @brief 係数値の取得
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 係数値の出力先
        template< class Out >
        void a( Out out ) const
        { Generic::a( out ); }

        /// @brief 係数値の分散の取得
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 係数値の分散の出力先
        template< class Out >
        void var_a( Out out ) const
        { Generic::var_a( out ); }

        /// @brief 従属変数の当てはめ値の取得
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 当てはめ値の出力先
        template< class Out >
        void fitValue( Out out ) const
        { Generic::fitValue( out ); }

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( Generic::la() ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @todo 最大モデルの対数尤度関数は、独立変数が等しいものを一つにまとめ、
        ///       係数の数を独立変数の異なるパラメータの数と等しくなるようにしなければならない。
        ///       現在、全独立変数が異なるものとして計算しているため、近々対応が必要になる。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( Generic::lmax() ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( Generic::lmin() ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( Generic::logLikelihood() ); }

        /// @brief ピアソン・カイ二乗統計量 X^2 を求める
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return ピアソン・カイ二乗統計量 X^2
        stat_type pearsonChiSq() const
        { return( Generic::pearsonChiSq() ); }

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( Generic::likelihoodChiSq() ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( Generic::pseudoRSq() ); }

        /// @brief 逸脱度残差 d を求める
        ///
        /// @param out 逸脱度残差の出力先
        template< class Out >
        void devianceResiduals( Out out ) const
        { Generic::devianceResiduals( out ); }

        /// @brief ピアソン残差 Xi を求める
        ///
        /// @note GLM_PoissonRegression::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out ピアソン残差の出力先
        /// @return なし
        template< class Out >
        void pearsonResiduals( Out out ) const
        { Generic::pearsonResiduals( out ); }

        /// @brief 指数型分布族の属性を返す
        ///
        /// @return 指数型分布族の属性
        std::string pdf() const
        { return( poisson_.ident() ); }

        /// @brief 連結関数の属性を返す
        ///
        /// @return 連結関数の属性
        std::string linkFunc() const
        { return( lf_.ident() ); }

      private:

        /*
          LogFunc_Poisson : 対数連結関数(ポアソン回帰用)

          定数項として試行回数を加える。試行回数は各試行に応じて切り替える
        */
        class LinkFunction_LogFunc_Poisson : public LinkFunction_IF
        {
        public:

          std::vector< stat_type > ni_; // 試行回数

          // デフォルト・コンストラクタ
          LinkFunction_LogFunc_Poisson()
            : opCnt_( 0 ), invfCnt_( 0 ) {}

          // 連結関数 g(x)
          virtual stat_type operator()( stat_type x ) const
          {
            stat_type d = std::log( x ) - std::log( ni_[opCnt_] );
            opCnt_ = ( opCnt_ + 1 ) % ni_.size();
            return( d );
          }

          // 導関数 g'(x)
          virtual stat_type df( stat_type x ) const
          { return( 1 / x ); }

          // 逆関数 g^-1(y)
          virtual stat_type invf( stat_type y ) const
          {
            stat_type d = ni_[invfCnt_] * std::exp( y );
            invfCnt_ = ( invfCnt_ + 1 ) % ni_.size();
            return( d );
          }

          // 属性を表す文字列
          virtual std::string ident() const
          { return( "Logarithm Function ( for Poisson Regression )" ); }

        private:

          // 各関数の利用回数(n_.size()の剰余)
          mutable std::vector< stat_type >::size_type opCnt_;   // operator()()
          mutable std::vector< stat_type >::size_type invfCnt_; // invf()
        };

        ExpFamily_Poisson poisson_;       //ポアソン分布
        LinkFunction_LogFunc_Poisson lf_; // 連結関数

        // 最小モデルの最大対数尤度
        virtual void calcLmin();
      };

      /**
         @brief 生存時間構造体

         打ち切りの有無を湿す bool 値 SurvivalTime::censored を持つ。打ち切りありの場合は SurvivalTime::censored = true とする。
      **/
      struct SurvivalTime
      {
        distribution::Base::stat_type time; ///< @brief 生存時間
        bool censored;            ///< @brief 打ち切りの有無

        /// @brief 生存時間と打ち切りの有無を指定して構築
        ///
        /// @param t 生存時間
        /// @param c 打ち切りの有無
        SurvivalTime( distribution::Base::stat_type t, bool c = false )
          : time( t ), censored( c ) { assert( t >= 0 ); }

        /// @brief 値の比較(Less)用関数オブジェクト
        ///
        /// @param s 比較対象の生存時間
        /// @return 自分の方が生存時間が短ければ true を返す
        bool operator<( const SurvivalTime& s ) const
        { return( time < s.time ); }
      };

      /**
         @brief 生存時間解析

         SurvivalAnalysis_Exponential と SurvivalAnalysis_Weibull の基底であり、単独では利用できない。
      **/
      class SurvivalAnalysis : public Base
      {
      public:

        /// @brief 独立変数と従属変数(生存時間)を指定して構築
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< stat_type >::size_type > を投げる。
        /// - 従属変数(生存時間)の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
        /// - 独立変数ベクトルの数(標本数)と従属変数(生存時間)の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< stat_type >::size_type > を投げる。
        ///
        /// @param x 独立変数ベクトルへの参照
        /// @param ts, te 従属変数(生存時間)の開始・末尾の次の位置
        template< class In >
        SurvivalAnalysis( const Matrix< stat_type >& x, In ts, In te );

        /// @brief スコア法による係数の推定
        ///
        /// 一般化線形モデルをスコア法によって係数推定するための関数
        ///
        /// Solver 型の引数 solver は任意の連立方程式計算用関数オブジェクトである。
        /// また、Test 型の引数 test は前回の解からの収束を判定するための関数オブジェクトで、
        /// 初期化するための init、判定対象の値を収集するための operator()、
        /// 収束判定を行うための isConvergent をメンバ関数として持つことを前提とする
        /// ( math_lib::convergence_test::BySum か math_lib::convergence_test::ByMax の利用を想定している)。
        ///
        /// @param solver 連立方程式を解くための関数
        /// @param test 収束判定用のテスト関数オブジェクト
        /// @param maxCount 反復処理の最大回数
        /// @return なし
        template< class Solver, class Test >
        void scoringMethod( Solver solver, Test test, uint32_t maxCount = DEFAULT_MAX_COUNT )
        { Base::scoringMethod( &a_, &var_a_, solver, test, maxCount ); }

        /// @brief 推定した係数を取得する
        ///
        /// @param out 係数の出力先
        /// @return なし
        template< class Out >
        void a( Out out ) const
        { if ( a_.size() >= x_.cols() ) std::copy( a_.begin(), a_.begin() + x_.cols(), out ); }

        /// @brief 推定係数の分散値を取得する
        ///
        /// @param out 推定係数の分散値の出力先
        /// @return なし
        template< class Out >
        void var_a( Out out ) const
        { if ( var_a_.size() >= x_.cols() ) std::copy( var_a_.begin(), var_a_.begin() + x_.cols(), out ); }

      protected:

        using size_type = Matrix< stat_type >::size_type; ///< サイズの型

        const Matrix< stat_type >& x_; ///< 独立変数ベクトルへの参照
        std::vector< SurvivalTime > t_;                    ///< 従属変数(生存時間)

        std::vector< stat_type > a_;     ///< 係数
        std::vector< stat_type > var_a_; ///< 係数の分散

        /// @brief 独立変数ベクトルの個数を返す
        ///
        /// @return 独立変数ベクトルの個数
        size_type xSize() const
        { return( x_.rows() ); }

        /// @brief 係数計算のための連立方程式と係数を保持する配列を初期化する
        ///
        /// @param sa 係数行列へのポインタ
        /// @param sb 右辺(解ベクトル)へのポインタ
        /// @param a 求める係数を保持するための配列へのポインタ
        virtual void init( SquareMatrix< stat_type >* sa,
                           std::vector< stat_type >* sb,
                           std::vector< stat_type >* a );

      private:

        // 独立変数ベクトルのサイズを返す
        virtual size_type coefSize() const = 0;
      };

      /**
         @brief 指数分布を用いた生存時間解析

         glm::Base からの派生であり glm::Base::scoringMethod を呼び出して利用する。
      **/
      class SurvivalAnalysis_Exponential : public SurvivalAnalysis
      {
      public:

        /// @brief 独立変数と従属変数(生存時間)を指定して構築
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< stat_type >::size_type > を投げる。
        /// - 従属変数(生存時間)の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
        /// - 独立変数ベクトルの数(標本数)と従属変数(生存時間)の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< stat_type >::size_type > を投げる。
        ///
        /// @param x 独立変数ベクトルへの参照
        /// @param ts, te 従属変数(生存時間)の開始・末尾の次の位置
        template< class In >
        SurvivalAnalysis_Exponential( const Matrix< stat_type >& x, In ts, In te )
          : SurvivalAnalysis( x, ts, te ), la_( 0 ), lmax_( 0 ), lmin_( 0 ) {}

        /// @brief 従属変数の当てはめ値の取得
        ///
        /// @note scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 当てはめ値の出力先
        template< class Out >
        void fitValue( Out out ) const
        { std::copy( mu_.begin(), mu_.end(), out ); }

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( la_ ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( lmax_ ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( lmin_ ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( ( lmax() - la() ) * 2 ); }

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( ( la() - lmin() ) * 2 ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( 1 - la() / lmin() ); }

        /// @brief 補正のない残差を求める
        ///
        /// 指数分布の場合、t / μ で求められる。打ち切り有無に対する補正はない。
        ///
        /// @param out 残差の出力先
        /// @return なし
        template< class Out >
        void residuals( Out out ) const;

        /// @brief Cox-Snell 残差を求める
        ///
        /// 指数分布の場合、t / μ + ( 1 - δ )Δ で求められる(Δは任意の定数)。
        ///
        /// @param out 残差の出力先
        /// @param delta 定数Δ
        /// @return なし
        template< class Out >
        void coxSnellResiduals( Out out, stat_type delta ) const;

        /// @brief Martingale 残差を求める
        ///
        /// 指数分布の場合、δ - t / μ で求められる。
        ///
        /// @param out 残差の出力先
        /// @return なし
        template< class Out >
        void martingaleResiduals( Out out ) const;

        /// @brief 逸脱度残差を求める
        ///
        /// 指数分布の場合、sign(t / μ)・{ -2[ t / μ + δln(t / μ) ] }^(1/2) で求められる。
        ///
        /// @param out 残差の出力先
        /// @return なし
        template< class Out >
        void devianceResiduals( Out out ) const;

      private:

        std::vector< stat_type > mu_;   // μ(当てはめ値)

        stat_type la_; // 当てはめ値に対する最大対数尤度
        stat_type lmax_; // 最大モデルに対する最大対数尤度
        stat_type lmin_; // 最小モデルに対する最大対数尤度

        // 係数行列の計算
        void calcCoef( SquareMatrix< stat_type >* sa ) const;
        // 対数尤度の係数による偏微分値を求める
        void calcUj( std::vector< stat_type >* uj ) const ;
        // 連立方程式の右辺を求める
        void calcRSide( const SquareMatrix< stat_type >& sa, std::vector< stat_type >* sb ) const;

        // 係数計算のための連立方程式と係数を保持する配列を初期化する
        virtual void init( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb, std::vector< stat_type >* a );

        // 係数計算のための連立方程式を作成する
        virtual void create( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb );

        // 係数計算後の更新処理を行う
        virtual void update( const std::vector< stat_type >& a );

        // ループ処理後の後始末
        virtual void cleanUp()
        { calcLa(); calcLmax(); calcLmin(); }

        // 当てはめ値に対する最大対数尤度を求める
        void calcLa();

        // 最大モデルに対する最大対数尤度を求める
        void calcLmax();

        // 最小モデルに対する最大対数尤度を求める
        void calcLmin();

        // 独立変数ベクトルのサイズを返す
        virtual size_type coefSize() const
        { return( x_.cols() ); }
      };

      /**
         @brief ワイブル分布を用いた生存時間解析

         glm::Base からの派生であり glm::Base::scoringMethod を呼び出して利用する。

         推定する係数の数は独立変数ベクトルの要素数で決定するが、ワイブル分布を用いる場合は形状パラメータ λ も推定するため、
         スコア法で返される係数とその分散は形状パラメータの分だけ一つ多くなる。形状パラメータは返された配列の末尾にある。
      **/
      class SurvivalAnalysis_Weibull : public SurvivalAnalysis
      {
      public:

        /// @brief 独立変数と従属変数(生存時間)を指定して構築
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< stat_type >::size_type > を投げる。
        /// - 従属変数(生存時間)の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
        /// - 独立変数ベクトルの数(標本数)と従属変数(生存時間)の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< stat_type >::size_type > を投げる。
        ///
        /// @param x 独立変数ベクトルへの参照
        /// @param ts, te 従属変数(生存時間)の開始・末尾の次の位置
        template< class In >
        SurvivalAnalysis_Weibull( const Matrix< stat_type >& x, In ts, In te )
          : SurvivalAnalysis( x, ts, te ), la_( stat_type() ),
            lmax_( stat_type() ), lmin_( stat_type() ) {}

        /// @brief 形状パラメータ λ の推定値を返す
        ///
        /// @return 形状パラーメータ λ
        stat_type lambda() const
        { return( ( a_.size() > 0 ) ? a_.back() : stat_type() ); }

        /// @brief 形状パラメータ λ の分散値を返す
        ///
        /// @return 形状パラーメータ λ の分散値
        stat_type var_lambda() const
        { return( ( var_a_.size() > 0 ) ? var_a_.back() : stat_type() ); }

        /// @brief 従属変数の当てはめ値の取得
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は何も得られないことに注意。
        ///
        /// @param out 当てはめ値の出力先
        template< class Out >
        void fitValue( Out out ) const;

        /// @brief 当てはめ値から求めた最大対数尤度を返す
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 当てはめ値から求めた最大対数尤度
        stat_type la() const { return( la_ ); }

        /// @brief 最大モデルの最大対数尤度を返す
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最大モデルの最大対数尤度
        stat_type lmax() const { return( lmax_ ); }

        /// @brief 最小モデルの最大対数尤度を返す
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 最小モデルの最大対数尤度
        stat_type lmin() const { return( lmin_ ); }

        /// @brief 対数尤度統計量 D を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 対数尤度統計量 D
        stat_type logLikelihood() const
        { return( ( lmax() - la() ) * 2 ); }

        /// @brief 尤度比カイ二乗統計量 C を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常にゼロになる。
        ///
        /// @return 尤度比カイ二乗統計量 C
        stat_type likelihoodChiSq() const
        { return( ( la() - lmin() ) * 2 ); }

        /// @brief マクファデンの疑似 R^2 値を求める
        ///
        /// @note Generic::scoringMethod を実行しなければ結果は常に NaN になる。
        ///
        /// @return マクファデンの疑似 R^2 値
        stat_type pseudoRSq() const
        { return( 1 - la() / lmin() ); }

        /// @brief 補正のない残差を求める
        ///
        /// ワイブル分布の場合、φt^λ で求められる。打ち切り有無に対する補正はない。
        ///
        /// @param out 残差の出力先
        /// @return なし
        template< class Out >
        void residuals( Out out ) const;

        /// @brief Cox-Snell 残差を求める
        ///
        /// ワイブル分布の場合、φt^λ + ( 1 - δ )Δ で求められる(Δは任意の定数)。
        ///
        /// @param out 残差の出力先
        /// @param delta 定数Δ
        /// @return なし
        template< class Out >
        void coxSnellResiduals( Out out, stat_type delta ) const;

        /// @brief Martingale 残差を求める
        ///
        /// ワイブル分布の場合、δ - φt^λ で求められる。
        ///
        /// @param out 残差の出力先
        /// @return なし
        template< class Out >
        void martingaleResiduals( Out out ) const;

        /// @brief 逸脱度残差を求める
        ///
        /// ワイブル分布の場合、sign(φt^λ)・{ -2[ φt^λ + δln(φt^λ) ] }^(1/2) で求められる。
        ///
        /// @param out 残差の出力先
        /// @return なし
        template< class Out >
        void devianceResiduals( Out out ) const;

      private:

        std::vector< stat_type > phi_; // Φ[i]
        std::vector< stat_type > d1_;  // Φ[i] * t[i]^λ
        std::vector< stat_type > d2_;  // ln( t[i] / θ[i] )

        stat_type la_;
        stat_type lmax_;
        stat_type lmin_;

        // 係数行列の計算
        void calcCoef( stat_type lambda, SquareMatrix< stat_type >* sa ) const;
        // 対数尤度の係数による偏微分値を求める
        void calcUj( stat_type lambda, std::vector< stat_type >* uj ) const ;
        // 連立方程式の右辺を求める
        void calcRSide( stat_type lambda, const SquareMatrix< stat_type >& sa, std::vector< stat_type >* sb ) const;

        // スコア法での初期化を行う
        virtual void init( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb, std::vector< stat_type >* a );

        // 係数を求めるための連立方程式を構築する
        virtual void create( SquareMatrix< stat_type >* sa, std::vector< stat_type >* sb );

        // 係数を求めるための連立方程式を構築する
        virtual void update( const std::vector< stat_type >& a );

        // 反復処理完了後に必要な処理を行う
        virtual void cleanUp() { calcLa(); calcLmax(); calcLmin(); }

        // 独立変数ベクトルのサイズを返す
        virtual size_type coefSize() const
        { return( x_.cols() + 1 ); }

        // 当てはめ値に対する最大対数尤度を求める
        void calcLa();

        // 最大モデルに対する最大対数尤度を求める
        void calcLmax();

        // 最小モデルに対するθ、λを求める
        void calcLminParam( stat_type* lambda, stat_type* theta );

        // 最小モデルに対する最大対数尤度を求める
        void calcLmin();
      };

      /**
         @brief カプラン・マイヤー推定値 ( Kaplan-Meier Estimate ) 算出用クラス

         std::map< SurvivalTime, distribution::Base::stat_type > からの派生であり std::map< SurvivalTime, distribution::Base::stat_type > の
         メンバ関数は全て利用することができる。キーは対象の生存時間の範囲の開始を表す。
      **/
      class KMEstimate : public std::map< SurvivalTime, distribution::Base::stat_type >
      {
      public:

        /// @brief 反復子の型
        using std::map< SurvivalTime, distribution::Base::stat_type >::iterator;
        /// @brief 定数反復子の型
        using std::map< SurvivalTime, distribution::Base::stat_type >::const_iterator;
        /// @brief キーの型
        using std::map< SurvivalTime, distribution::Base::stat_type >::key_type;
        /// @brief 値の型
        using std::map< SurvivalTime, distribution::Base::stat_type >::mapped_type;
        /// @brief 要素の型
        using std::map< SurvivalTime, distribution::Base::stat_type >::value_type;

        /// @brief 生存時間の配列を指定して構築
        ///
        /// - 生存時間の配列の指定範囲が有効かを内部でチェックしている。
        ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
        ///
        /// @param s 配列の開始位置
        /// @param e 配列の末尾の次の位置
        template< class In > KMEstimate( In s, In e );
      };

    } // namespace glm

  } // namespace statistics

} // namespace tadah_fussy

#include "glm_inline.h"

#endif
