/**
   @file quantification.h
   @brief 数量化法ルーチン定義用ヘッダ

   @author tadah_fussy
   @date 2020/10/04 新規作成
   @date 2021/03/26 名前空間の変更
**/
#ifndef QUANTIFICATION_20210326_H
#define QUANTIFICATION_20210326_H

#include "common/eigen.h"
#include "statistics.h"
#include <set>

namespace tadah_fussy
{
  namespace statistics
  {
    namespace quantification
    {
      template< class X, class VALUE >
      using item_type = std::map< X, VALUE >;                 ///< アイテムの型
      template< class X, class VALUE >
      using list_type = std::vector< item_type< X, VALUE > >; ///< アイテムリストの型
      using stat_type = distribution::Base::stat_type;        ///< 数量の型
      using size_type = size_t;                               ///< 添字の型

      /**
         @brief 数量化 I 類
      **/
      template< class X >
      class Class1
      {
        using x_item_type = item_type< X, uint32_t >;  // 独立変数用のアイテムの型
        using x_list_type = list_type< X, uint32_t >;  // 独立変数用のアイテムリストの型
        using a_item_type = item_type< X, stat_type >; // カテゴリごとの係数の型
        using a_list_type = list_type< X, stat_type >; // 全アイテムの係数リストの型

        a_list_type a_;                  // 係数
        x_list_type category_;           // カテゴリリスト
        stat_type my_;                   // y の平均値
        SquareMatrix< stat_type > s_;    // 共分散行列
        SquareMatrix< stat_type > invS_; // 共分散行列の逆行列

      public:

        /// @brief 独立変数と従属変数を指定して構築
        ///
        /// 独立変数ベクトル(X)は、一行に一つのベクトルの要素が、列ごとに各ベクトルが並んだ行列で表される。
        /// ベクトルの数(標本数)は従属変数(Y)の要素数で決まる。そのため、X行列の行数とYの要素数は一致
        /// させる必要がある。
        ///
        /// 独立ベクトルの各要素はカテゴリ(分類)データとして扱われ、内部で同一データごとに集計される。
        /// データの型は文字列・数値データのいずれも利用できるが、浮動小数点数などは少しでも値が異なれば
        /// 違うデータとして扱われるため注意すること。
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はどちらもゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< X >::size_type > を投げる。
        /// - 従属変数の要素の範囲が不正な(開始と終了が逆になっている)場合、
        ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
        /// - デザイン行列の行数と従属変数の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< X >::size_type > を投げる。
        /// - 共分散行列の逆行列が得られなかった場合、また回帰係数の推定値が得られなかった(連立方程式の計算に失敗した)場合、
        ///   例外 std::runtime_error を投げる。
        ///
        /// @param x 独立変数(X)を保持する行列
        /// @param ys, ye 従属変数(Y)のデータ列先頭と末尾の次
        /// @param solver 連立方程式を求めるソルバー
        template< class In, class Solver >
        Class1( const Matrix< X >& x, In ys, In ye, Solver solver );

        /// @brief アイテム数を返す
        ///
        /// @return アイテム数
        size_type itemSize() const
        { return( category_.size() ); }

        /// @brief 指定したアイテムのカテゴリ数を返す
        ///
        /// - アイテム番号が範囲外だった場合、例外 std::out_of_range を投げる。
        ///
        /// @param item アイテム番号
        /// @return カテゴリ数
        size_type categorySize( size_type item ) const
        { return( category_.at( item ).size() ); }

        /// @brief 回帰係数(数量)を返す
        ///
        /// - アイテム番号が範囲外だった場合、std::out_of_range 例外を投げる。
        /// - 指定したカテゴリが存在しなかった場合、std::out_of_range 例外を投げる。
        ///
        /// @param item アイテムの番号
        /// @param category カテゴリの番号
        /// @return 回帰係数(数量)
        stat_type a( size_type item, const X& category ) const;

        /// @brief カテゴリ名を返す
        ///
        /// - アイテム番号が範囲外だった場合、std::out_of_range 例外を投げる。
        /// - 指定したカテゴリが存在しなかった場合、std::out_of_range 例外を投げる。
        ///
        /// @param item アイテムの番号
        /// @param category カテゴリの番号
        /// @return カテゴリ名
        X category( size_type item, size_type category ) const;

        /// @brief 従属変数の予測値を返す
        ///
        /// @param xs, xe 独立変数の先頭と末尾の次の位置
        /// @return 予測値
        template< class In >
        stat_type expY( In xs, In xe ) const;

        /// @brief 重相関係数を返す
        ///
        /// @return 重相関係数
        stat_type mcc() const
        { return( std::sqrt( 1.0 - 1.0 / ( s_[0][0] * invS_[0][0] ) ) ); }

        /// @brief 指定したアイテム番号と y の間の偏相関係数を返す
        ///
        /// @param item アイテム番号
        /// @return 偏相関係数
        stat_type partialCC( size_type item ) const;
      };

      /**
         @brief 数量化 II 類
      **/
      template< class X, class Y >
      class Class2
      {
        using x_item_type = item_type< X, uint32_t >;  // 独立変数用のアイテムの型
        using x_list_type = list_type< X, uint32_t >;  // 独立変数用のアイテムリストの型
        using y_type = item_type< Y, uint32_t >;       // 従属変数用のカテゴリリストの型
        using bi_type = item_type< Y, stat_type >;     // カテゴリごとの y 値の型
        using a_item_type = item_type< X, stat_type >; // カテゴリごとの係数の型
        using a_list_type = list_type< X, stat_type >; // 全アイテムの係数リストの型

        a_list_type a_;                  // 係数
        x_list_type xCategory_;          // x のアイテムリスト
        y_type yCategory_;               // y のカテゴリリスト
        bi_type bi_;                     // 予測値(yに対する数量)
        SquareMatrix< stat_type > s_;    // 共分散行列
        SquareMatrix< stat_type > invS_; // 共分散行列の逆行列

      public:

        /// @brief 独立変数と従属変数を指定して構築
        ///
        /// 独立変数ベクトル(X)は、一行に一つのベクトルの要素が、列ごとに各ベクトルが並んだ行列で表される。
        /// ベクトルの数(標本数)は従属変数(Y)の要素数で決まる。そのため、X行列の行数とYの要素数は一致
        /// させる必要がある。
        ///
        /// 独立ベクトルの各要素はカテゴリ(分類)データとして扱われ、内部で同一データごとに集計される。
        /// データの型は文字列・数値データのいずれも利用できるが、浮動小数点数などは少しでも値が異なれば
        /// 違うデータとして扱われるため注意すること。
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はどちらもゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< X >::size_type > を投げる。
        /// - 従属変数の要素の範囲が不正な(開始と終了が逆になっている)場合、
        ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
        /// - デザイン行列の行数と従属変数の要素数が一致していなければならない。
        ///   そうでない場合、例外 exception::NotEqualLength< Matrix< X >::size_type > を投げる。
        /// - 共分散行列の逆行列が得られなかった場合、また回帰係数の推定値が得られなかった(連立方程式の計算に失敗した)場合、
        ///   例外 std::runtime_error を投げる。
        ///
        /// @param x 独立変数(X)を保持する行列
        /// @param ys, ye 従属変数(Y)のデータ列先頭と末尾の次
        /// @param solver 連立方程式を求めるソルバー
        template< class In, class Solver >
        Class2( const Matrix< X >& x, In ys, In ye, Solver solver );

        /// @brief アイテム数を返す
        ///
        /// @return アイテム数
        size_type itemSize() const
        { return( xCategory_.size() ); }

        /// @brief 指定したアイテムのカテゴリ数を返す
        ///
        /// - アイテム番号が範囲外だった場合、例外 std::out_of_range を投げる。
        ///
        /// @param item アイテム番号
        /// @return カテゴリ数
        size_type categorySize( size_type item ) const
        { return( xCategory_.at( item ).size() ); }

        /// @brief 回帰係数(数量)を返す
        ///
        /// アイテム数や、各アイテムのカテゴリ数より大きな添字を指定した場合は std::out_of_range 例外を投げる。
        ///
        /// @param item アイテムの番号
        /// @param category カテゴリ
        /// @return 回帰係数(数量)
        stat_type a( size_type item, const X& category ) const;

        /// @brief 指定したアイテム・カテゴリ番号のカテゴリの値を返す
        ///
        /// @param item アイテム番号
        /// @param category カテゴリ番号
        /// @return カテゴリの値
        X category( size_type item, size_type category ) const;

        /// @brief 指定した独立変数から予測値(数量)を求める
        ///
        /// @param s, e 独立変数の開始と末尾の次の位置
        /// @return 予測値(数量)
        template< class In >
        stat_type expY( In s, In e ) const;

        /// @brief 指定した独立変数から判別を行う
        ///
        /// @param s, e 独立変数の開始と末尾の次の位置
        /// @return 判別結果 ( 最近傍にある y の値 )
        template< class In >
        Y discriminant( In s, In e ) const;

        /// @brief 重相関係数を返す
        ///
        /// @return 重相関係数
        stat_type mcc() const
        { return( std::sqrt( 1.0 - 1.0 / ( s_[0][0] * invS_[0][0] ) ) ); }

        /// @brief 指定したアイテム番号と y の間の偏相関係数を返す
        ///
        /// @param item アイテム番号
        /// @return 偏相関係数
        stat_type partialCC( size_type item ) const;
      };

      /**
         @brief 数量化 III 類
      **/
      class Class3
      {
        std::vector< std::vector< stat_type > > xi_; // 各要素の数量
        std::vector< std::vector< stat_type > > yj_; // 各カテゴリの数量
        std::vector< stat_type > lambda_;            // 固有値

      public:

        /// @brief 各要素のカテゴリに対する反応を指定して構築
        ///
        /// 引数 data は、一行に一つの要素のカテゴリに対する反応が、一列に一つのカテゴリの要素ごとの反応が並んだ行列で表される。
        ///
        /// - 独立変数ベクトルの数(標本数)とその中の要素数はどちらもゼロより大きくなければならない。
        ///   そうでない場合は例外 exception::NotPositiveLength< Matrix< bool >::size_type > を投げる。
        ///
        /// @param data 各要素のカテゴリ毎の反応を保持する行列
        Class3( const Matrix< bool >& data );

        /// @brief 要素数を返す
        ///
        /// @return 要素数
        size_type dataSize() const;

        /// @brief カテゴリ数を返す
        ///
        /// @return カテゴリ数
        size_type categorySize() const;

        /// @brief 固有値の数を返す
        ///
        /// @return 固有値の数
        size_type lambdaSize() const
        { return( lambda_.size() ); }

        /// @brief 指定した要素の数量を返す
        ///
        /// 固有値数・要素数より大きな添字を指定した場合は std::out_of_range 例外を投げる。
        ///
        /// @param k 固有値の順位 ( 降順 )
        /// @param i 要素の番号
        /// @return 指定した要素の数量
        stat_type x( size_type k, size_type i ) const
        { return( xi_.at( k ).at( i ) ); }

        /// @brief 指定したカテゴリの数量を返す
        ///
        /// 固有値数・カテゴリ数より大きな添字を指定した場合は std::out_of_range 例外を投げる。
        ///
        /// @param k 固有値の順位 ( 降順 )
        /// @param j カテゴリの番号
        /// @return 指定したカテゴリの数量
        stat_type y( size_type k, size_type j ) const
        { return( yj_.at( k ).at( j ) ); }

        /// @brief 指定した順位の固有値を返す
        ///
        /// 固有値の数より大きな添字を指定した場合は std::out_of_range 例外を投げる。
        ///
        /// @param k 固有値の順位 ( 降順 )
        /// @return 指定した順位の固有値
        stat_type lambda( size_type k ) const
        { return( lambda_.at( k ) ); }
      };

      /**
         @brief 数量化 IV 類
      **/
      class Class4
      {
        std::vector< std::vector< stat_type > > xi_; // 各要素の数量
        std::vector< stat_type > lambda_;            // 固有値

      public:

        /// @brief 親近性を指定して構築
        ///
        /// @param e 親近性を保持する行列
        Class4( const SquareMatrix< stat_type >& e );

        /// @brief 要素数を返す
        ///
        /// @return 要素数
        size_type dataSize() const;

        /// @brief 固有値の数を返す
        ///
        /// @return 固有値の数
        size_type lambdaSize() const
        { return( lambda_.size() ); }

        /// @brief 指定した数量を返す
        ///
        /// 固有値数・要素数より大きな添字を指定した場合は std::out_of_range 例外を投げる。
        ///
        /// @param k 固有値の順位 ( 降順 )
        /// @param i 要素の番号
        /// @return 指定した数量
        stat_type x( size_type k, size_type i ) const
        { return( xi_.at( k ).at( i ) ); }

        /// @brief 指定した順位の固有値を返す
        ///
        /// 固有値の数より大きな添字を指定した場合は std::out_of_range 例外を投げる。
        ///
        /// @param k 固有値の順位 ( 降順 )
        /// @return 指定した順位の固有値
        stat_type lambda( size_type k ) const
        { return( lambda_.at( k ) ); }
      };

    } // namespace quantification

  } // namespace statistics

} // namespace tadah_fussy

#include "quantification_inline.h"

#endif
