#include "regression.h"

namespace tadah_fussy
{
  namespace statistics
  {
    namespace regression
    {
      using std::string;
      using std::vector;
      using std::cout;
      using std::endl;

      using stat_type = distribution::Base::stat_type;

      namespace
      {
        /*
          MultMatByVec : 行列 m と ベクトル is の積を計算し、os に返す

          結果は必ず Normalize される。
        */
        template< class In, class Out >
        void MultMatByVec( const Matrix< stat_type >& m, In is, Out os )
        {
          matrix_lib::multMV( m, is, os );
          math_lib::Normalize( os, os + m.rows() );
        }

        /*
          MultTransByVec : 行列 m の転置と ベクトル is の積を計算し、os に返す

          needToNorm が true なら結果は Normalize される。
        */
        template< class In, class Out >
        void MultTransByVec( const Matrix< stat_type >& m, In is, Out os, bool needToNorm )
        {
          matrix_lib::multVM( is, m, os );
          if ( needToNorm )
            math_lib::Normalize( os, os + m.cols() );
        }

        /*
          SubMat : 列ベクトル [ rs, re ) と行ベクトル [ cs, ce ) の積を計算し(結果は行列になる)、m から減算する
        */
        template< class Row, class Column >
        void SubMat( Row rs, Row re, Column cs, Column ce, Matrix< stat_type >* m )
        {
          auto i = m->begin();
          for ( ; cs != ce ; ++cs )
            for ( Row r = rs ; r != re ; ++r )
              *i++ -= *r * *cs;
        }

        /*
          MultMatByTrans : src とその転置行列の積を計算し、dest に返す
        */
        void MultMatByTrans( const Matrix< stat_type >& src, SymmetricMatrix< stat_type >* dest )
        {
          for ( Matrix< stat_type >::size_type r = 0 ; r < src.rows() ; ++r ) {
            auto row = src.row( r );
            for ( Matrix< stat_type >::size_type c = r ; c < src.rows() ; ++c ) {
              auto column = src.row( c );
              (*dest)[r][c] = std::inner_product( row, row.end(), column, stat_type() );
            }
          }
        }

      } // namespace

      /*
        NIPALS::operator() : PLS 回帰メインルーチン
      */
      void NIPALS::operator()( Matrix< stat_type >* a, size_t latentVecs, stat_type e, uint32_t maxCnt )
      {
        assert( a != nullptr );

        const string FUNC_NAME = "tadah_fussy::statistics::regression::NIPALS::operator()";

        // 潜在ベクトルの個数は X のサイズより小さいことを前提とする
        if ( latentVecs > x_.cols() )
          throw exception::TooGreatNumber< size_t >
            ( FUNC_NAME + " : latentVecs must NOT be more than column size of x.", latentVecs, x_.cols() );
        if ( latentVecs > x_.rows() )
          throw exception::TooGreatNumber< size_t >
            ( FUNC_NAME + " : latentVecs must NOT be more than row size of x.", latentVecs, x_.rows() );

        pT_.assign( latentVecs, x_.cols() );
        qT_.assign( latentVecs, y_.cols() );

        // x_, y_ は更新されるためコピーして使う
        Matrix< stat_type > x( x_ );
        Matrix< stat_type > y( y_ );
        for ( size_t cnt = 0 ; cnt < latentVecs ; ++cnt )
          iterProc( &x, &y, cnt, e, maxCnt );

        SymmetricMatrix< stat_type > pTp( pT_.rows() ); // Pt・P
        MultMatByTrans( pT_, &pTp );

        SquareMatrix< stat_type > pTpM;                 // ( Pt・P )^-1
        matrix_lib::Inverse( pTp, &pTpM, math_lib::solver::LUdecomposition_Crout() );

        Matrix< stat_type > gT;                         // Gt ( G = ( Pt・P )^-1・Pt )
        matrix_lib::mult( pTpM, pT_, &gT );
        gT.transpose();
        for ( vector< stat_type >::size_type c = 0 ; c < gT.cols() ; ++c )
          for ( vector< stat_type >::size_type r = 0 ; r < gT.rows() ; ++r )
            gT[r][c] *= b_[c];

        matrix_lib::mult( gT, qT_, a );                  // A = Gt・Qt

#ifdef DEBUG
        cout << "PtP = " << pTp << endl;
        cout << "PtP^-1 = " << pTpM << endl;
        cout << "Gt = " << gT << endl;
        cout << "GtB = " << gT << endl;
        cout << "Qt = " << qT_ << endl;
#endif
      }

      /*
        NIPALS::iterProc : 反復処理により u, t, v, w を求める関数

        u = Xt・w
        t = X・u
        v = Yt・t
        w = Y・v

        を繰り返す
      */
      void NIPALS::iterProc( Matrix< stat_type >* x, Matrix< stat_type >* y,
                             size_t latentVecIdx, stat_type e, uint32_t maxCnt )
      {
        using Vec = vector< stat_type >;

        const string FUNC_NAME = "tadah_fussy::statistics::regression::NIPALS::iterProc";

#ifdef DEBUG
        cout << "*** iterative processing start ***" << endl;

        cout << "x = " << *x << endl;
        cout << "y = " << *y << endl;
#endif

        Vec u( x->cols(), 1.0 / std::sqrt( static_cast< stat_type >( x->cols() ) ) ); // u ベクトル
        vector< Vec > t( 2, Vec( x->rows() ) ); // t ベクトル(収束判定のため前の値を保持できるよう二つ分確保)
        Vec v( y->cols() ); // v ベクトル
        Vec w( y->rows() ); // w ベクトル

        Vec::size_type tIdx = 0; // t ベクトルの現在の計算対象 (0/1)
        uint32_t cnt; // 反復処理回数
        for ( cnt = 0 ; cnt < maxCnt ; ++cnt ) {
          Vec& ct = t[tIdx]; // t ベクトルの現在の計算対象
          MultMatByVec( *x, u.begin(), ct.begin() );         // t = X・u
          MultTransByVec( *y, ct.begin(), v.begin(), true ); // v = Yt・t
          MultMatByVec( *y, v.begin(), w.begin() );           // w = Y・v
          MultTransByVec( *x, w.begin(), u.begin(), true );   // u = Xt・w
#ifdef DEBUG
          matrix_lib::PrintArray( cout, ct.begin(), ct.end(), "t=" ); cout << endl;
          matrix_lib::PrintArray( cout, v.begin(), v.end(), "v=" ); cout << endl;
          matrix_lib::PrintArray( cout, w.begin(), w.end(), "w=" ); cout << endl;
          matrix_lib::PrintArray( cout, u.begin(), u.end(), "u=" ); cout << endl << endl;
#endif

          // 収束チェック
          math_lib::convergence_test::ByMax< stat_type > test( e );
          for ( Vec::size_type i = 0 ; i < t[tIdx ^ 1].size() ; ++i )
            test( t[tIdx ^ 1][i], ct[i] );
          if ( test.isConvergent() ) break;
          tIdx ^= 1; // t ベクトルの現在の計算対象を反転
        }
        if ( cnt >= maxCnt )
          throw exception::ExcessLimit( FUNC_NAME + " : Not convergent ( ExceptionExcessLimit ).", maxCnt );

        Vec& ct = t[tIdx]; // t ベクトルの現在の計算対象

        // 回帰係数の登録
        b_.push_back( std::inner_product( ct.begin(), ct.end(), w.begin(), stat_type() ) );
        // w は回帰係数を元に予測値にする
        for ( Vec::size_type i = 0 ; i < w.size() ; ++i )
          w[i] = b_.back() * ct[i];
        auto pTRow = pT_.row( latentVecIdx );
        MultTransByVec( *x, ct.begin(), pTRow, false ); // p = Xt・t
        auto qTRow = qT_.row( latentVecIdx );
        MultTransByVec( *y, w.begin(), qTRow, false );  // q = Yt・w

        SubMat( ct.begin(), ct.end(), pTRow, pTRow.end(), x ); // X = X - t・pt
        SubMat( w.begin(), w.end(), qTRow, qTRow.end(), y );   // Y = Y - w・qt

#ifdef DEBUG
        cout << "b = " << b_.back() << endl;
        MatrixLib::PrintArray( cout, pTRow, pTRow.end(), "p=" ); cout << endl;
        MatrixLib::PrintArray( cout, qTRow, qTRow.end(), "q=" ); cout << endl;
        cout << "x="  << *x << endl;
        cout << "y="  << *y << endl;
#endif
      }

      namespace
      {
        /*
          MultTransByMat : mat1 の転置行列と mat2 の積を計算し、dest に返す
        */
        void MultTransByMat( const Matrix< stat_type >& mat1, const Matrix< stat_type >& mat2, Matrix< stat_type >* dest )
        {
          for ( Matrix< stat_type >::size_type r = 0 ; r < mat1.cols() ; ++r ) {
            auto row = mat1.column( r );
            for ( Matrix< stat_type >::size_type c = 0 ; c < mat2.cols() ; ++c ) {
              auto column = mat2.column( c );
              (*dest)[r][c] = std::inner_product( row, row.end(), column, stat_type() );
            }
          }
        }

        /*
          MultMatrix : mat1 と mat2 の積を計算し、dest に返す

          dest は正方行列である ( mat1 の行数と mat2 の列数が等しい ) ことを前提とする
        */
        void MultMatrix( const Matrix< stat_type >& mat1, const Matrix< stat_type >& mat2, SquareMatrix< stat_type >* dest )
        {
          assert( mat1.rows() == mat2.cols() );

          for ( Matrix< stat_type >::size_type r = 0 ; r < mat1.rows() ; ++r ) {
            auto row = mat1.row( r );
            for ( Matrix< stat_type >::size_type c = 0 ; c < mat2.cols() ; ++c ) {
              auto column = mat2.column( c );
              (*dest)[r][c] = std::inner_product( row, row.end(), column, stat_type() );
            }
          }
        }

      } // namespace

      /*
        SIMPLS::operator() : PLS 回帰メインルーチン
      */
      void SIMPLS::operator()( Matrix< stat_type >* a, size_t latentVecs, stat_type e, uint32_t maxCnt )
      {
        const string FUNC_NAME = "tadah_fussy::statistics::regression::SIMPLS::operator()";

        assert( a != nullptr );

        // 潜在ベクトルの個数は X のサイズより小さいことを前提とする
        if ( latentVecs > x_.cols() )
          throw exception::TooGreatNumber< size_t >
            ( FUNC_NAME + " : latentVecs must NOT be more than column size of x.", latentVecs, x_.cols() );
        if ( latentVecs > x_.rows() )
          throw exception::TooGreatNumber< size_t >
            ( FUNC_NAME + " : latentVecs must NOT be more than row size of x.", latentVecs, x_.rows() );

        u_.assign( x_.cols(), latentVecs );
        pT_.assign( latentVecs, x_.cols() );
        qT_.assign( latentVecs, y_.cols() );

        // x_, y_ は更新されるためコピーして使う
        Matrix< stat_type > x( x_ );
        Matrix< stat_type > y( y_ );
        for ( size_t cnt = 0 ; cnt < latentVecs ; ++cnt )
          iterProc( &x, &y, cnt, e, maxCnt );

        // Pt・U
        SquareMatrix< stat_type > pTu( pT_.rows(), u_.cols() );
        MultMatrix( pT_, u_, &pTu );

        // ( Pt・U )^-1
        SquareMatrix< stat_type > pTuM;
        matrix_lib::Inverse( pTu, &pTuM, math_lib::solver::LUdecomposition_Crout() );

        // U・( Pt・U )^-1
        Matrix< stat_type > upTuM;
        matrix_lib::mult( u_, pTuM, &upTuM );

        // A = U・( Pt・U )^-1・Qt
        matrix_lib::mult( upTuM, qT_, a );

#ifdef DEBUG
        cout << "PtU = " << pTu << endl;
        cout << "PtU^-1 = " << pTuM << endl;
        cout << "U(PtU^-1) = " << upTuM << endl;
#endif
      }

      /*
        SIMPLS::iterProc : 反復処理により u, t を求める関数

        u は Xt・Y の第一左特異ベクトル
        t = X・u / ||X・u||
        p = X・t
        q = Y・t
      */
      void SIMPLS::iterProc( Matrix< stat_type >* x, Matrix< stat_type >* y,
                             size_t latentVecIdx, stat_type e, uint32_t maxCnt )
      {
        using Vec = vector< stat_type >;

#ifdef DEBUG
        cout << "*** iterative processing [" << latentVecIdx << "] ***" << endl;

        cout << "x = " << *x << endl;
        cout << "y = " << *y << endl;
#endif

        // S = Xt・Y
        Matrix< stat_type > s( x->cols(), y->cols() );
        MultTransByMat( *x, *y, &s );

        // S・St = ( Xt・Y )・( Xt・Y )t
        SymmetricMatrix< stat_type > ssT( s.rows() );
        MultMatByTrans( s, &ssT );

        // S = Xt・Y の第一左特異ベクトル ( S・St の第一固有ベクトル ) を u に求める
        Vec u( x->cols(), 1.0 / std::sqrt( static_cast< stat_type >( x->cols() ) ) );
        math_lib::convergence_test::ByMax< stat_type > test( e );
        math_lib::eigen_lib::PowerIter( ssT, &u, test, maxCnt );
#ifdef DEBUG
        matrix_lib::PrintArray( cout, u.begin(), u.end(), "u=" ); cout << endl;
#endif
        for ( typename Vec::size_type i = 0 ; i < u.size() ; ++i )
          u_[i][latentVecIdx] = u[i];

        // t = Xu
        Vec t( x->rows() );
        MultMatByVec( *x, u.begin(), t.begin() );
#ifdef DEBUG
        matrix_lib::PrintArray( cout, t.begin(), t.end(), "t=" ); cout << endl;
#endif

        // p = Xt・t
        Matrix< stat_type >::iterator row = pT_.row( latentVecIdx );
        MultTransByVec( *x, t.begin(), row, false );
        SubMat( t.begin(), t.end(), row, row.end(), x ); // X = X - t・pt
#ifdef DEBUG
        matrix_lib::PrintArray( cout, row, row.end(), "p=" ); cout << endl;
#endif

        // q = Yt・t
        row = qT_.row( latentVecIdx );
        MultTransByVec( *y, t.begin(), row, false );
        SubMat( t.begin(), t.end(), row, row.end(), y );   // Y = Y - t・qt
#ifdef DEBUG
        matrix_lib::PrintArray( cout, row, row.end(), "q=" ); cout << endl;

        cout << "x="  << *x << endl;
        cout << "y="  << *y << endl;
#endif
      }

    } // namespace regression

  } // namespace statistics

} // namespace tadah_fussy
