/**
   @file statistics_inline.h
   @brief 確率・統計ルーチンインライン・テンプレート関数用ヘッダ

   @author tadah_fussy
**/
namespace tadah_fussy
{
  namespace statistics
  {
    /**
       標本平均
    **/

    /// @brief 標本平均を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// データがない場合、結果はデータの要素の初期値(通常はゼロ)が返される。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた平均値
    template< class Res, class In >
    Res Mean( In s, In e )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::Mean";

      Res cnt = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );
      Res ans = math_lib::Sum< Res >( s, e );

      if ( cnt > 0 ) ans /= cnt;

      return( ans );
    }

    /// @brief 標本平均を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @return 求めた平均値
    template< class Res, class Container >
    Res MeanAll( const Container& c )
    {
      return( Mean< Res >( c.begin(), c.end() ) );
    }

    /// @brief データを変換した上での標本平均を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Op はデータ変換用の単項関数オブジェクトの型を表す。
    ///
    /// データがない場合、結果はデータの要素の初期値(通常はゼロ)が返される。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param op データ変換用単項関数
    /// @return 求めた平均値
    template< class Res, class In, class Op >
    Res TransMean( In s, In e, Op op )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::TransMean";

      Res cnt = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );
      Res ans = math_lib::Sum< Res >( s, e, math_lib::OpBind( op ) );

      if ( cnt > 0 ) ans /= cnt;

      return( ans );
    }

    /// @brief データを変換した上での標本平均を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Op はデータ変換用の単項関数オブジェクトの型を表す。
    ///
    /// データがない場合、結果はデータの要素の初期値(通常はゼロ)が返される。
    ///
    /// @param c データ列への参照
    /// @param op データ変換用単項関数
    /// @return 求めた平均値
    template< class Res, class Container, class Op >
    Res TransMeanAll( const Container& c, Op op )
    {
      return( TransMean< Res >( c.begin(), c.end(), op ) );
    }

    /**
       調和平均
    **/

    /// @brief 調和平均(逆数の平均の逆数)を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた調和平均値
    template< class Res, class In >
    Res HarmonicMean( In s, In e )
    { return( Res( 1 ) / TransMean< Res >( s, e, Reciprocal< Res >() ) ); }

    /// @brief 調和平均(逆数の平均の逆数)を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @return 求めた調和平均値
    template< class Res, class Container >
    Res HarmonicMeanAll( const Container& c )
    {
      return( HarmonicMean< Res >( c.begin(), c.end() ) );
    }

    /**
       偏差平方和
    **/

    /// @brief 偏差の平方和を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた偏差平方和
    template< class Res, class In >
    Res DevSq( In s, In e )
    {
      return( math_lib::Sum< Res >( s, e, math_lib::OpBind( Deviation< Res >( Mean< Res >( s, e ) ) ) ) );
    }

    /// @brief 偏差の平方和を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @return 求めた偏差平方和
    template< class Res, class Container >
    Res DevSqAll( const Container& c )
    {
      return( DevSq< Res >( c.begin(), c.end() ) );
    }

    /**
       順序統計量の推定量
    **/

    /**
       中央値(メディアン)
    **/

    /// @brief 中央値(メディアン)を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めた中央値 (データがない場合は要素の初期値)
    template< class Res, class In, class Less >
    Res Median( In s, In e, Less less )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::Median";

      error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );

      std::vector< Res > x( s, e ); // 要素を x にコピー
      auto i = x.size(); // 要素数
      if ( i == 0 ) return( Res() );
      if ( i == 1 ) return( *s );

      std::sort( x.begin(), x.end(), less );

      if ( i % 2 != 0 ) {
        return( x[i/2] );
      } else {
        return( ( x[i/2-1] + x[i/2] ) / Res( 2 ) );
      }
    }

    /// @brief 中央値(メディアン)を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた中央値 (データがない場合は要素の初期値)
    template< class Res, class In >
    Res Median( In s, In e )
    { return( Median< Res >( s, e, std::less< Res >() ) ); }

    /// @brief 中央値(メディアン)を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// @param c データ列への参照
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めた中央値 (データがない場合は要素の初期値)
    template< class Res, class Container, class Less >
    Res MedianAll( const Container& c, Less less )
    {
      return( Median< Res >( c.begin(), c.end(), less ) );
    }

    /// @brief 中央値(メディアン)を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param c データ列への参照
    /// @return 求めた中央値 (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res MedianAll( const Container& c )
    { return( MedianAll< Res >( c, std::less< Res >() ) ); }

    /**
       m-分位数
    **/

    /// @brief m-分位数を求める
    ///
    /// テンプレート引数の In, Out はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// - データ列 [ is, ie ) の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    /// - 求めた m-分位数を登録する配列 [ os, oe ) の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< Out >::difference_type > を投げる。
    ///
    /// @param is 対象のデータ列の開始反復子
    /// @param ie 対象のデータ列の末尾の次の反復子
    /// @param os 求めた m-分位数を登録する配列の開始反復子
    /// @param oe 求めた m-分位数を登録する配列の末尾の次の反復子
    /// @param less データ比較用二項関数オブジェクト
    /// @return データがない場合は false
    template< class In, class Out, class Less >
    bool Quantile( In is, In ie, Out os, Out oe, Less less )
    {
      using value_type = typename std::iterator_traits< Out >::value_type; // 値の型

      const std::string FUNC_NAME = "tadah_fussy::statistics::Quantile";
      // 等分する数を求める
      size_t qSize = error_lib::ArrayLength( os, oe, FUNC_NAME + " : [ os, oe ) " + error_message::NEGATIVE_LENGTH );
      if ( qSize++ == 0 ) return( true );

      // 階級の数 (データの数 - 1) を求める
      size_t xSize = error_lib::ArrayLength( is, ie, FUNC_NAME + " : [ is, ie ) " + error_message::NEGATIVE_LENGTH );
      if ( xSize-- == 0 ) return( false );

      // データをコピーしてソートする
      std::vector< value_type > x( is, ie );
      std::sort( x.begin(), x.end(), less );

      size_t qi = 1;
      for ( ; os != oe ; ++os ) {
        size_t xi = xSize * qi / qSize; // qi 番目の分位数にあたるデータの添字
        *os = x[xi];
        if ( xi < xSize ) *os += ( x[xi + 1] - x[xi] ) * ( xSize * qi - qSize * xi ) / qSize;
        ++qi;
      }

      return( true );
    }

    /// @brief m-分位数を求める
    ///
    /// テンプレート引数の In, Out はコンテナクラスの iterator の型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// - データ列 [ is, ie ) の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    /// - 求めた m-分位数を登録する配列 [ os, oe ) の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< Out >::difference_type > を投げる。
    ///
    /// @param is 対象のデータ列の開始反復子
    /// @param ie 対象のデータ列の末尾の次の反復子
    /// @param os 求めた m-分位数を登録する配列の開始反復子
    /// @param oe 求めた m-分位数を登録する配列の末尾の次の反復子
    /// @return データがない場合は false
    template< class In, class Out >
    bool Quantile( In is, In ie, Out os, Out oe )
    { return( Quantile( is, ie, os, oe, std::less< typename std::iterator_traits< Out >::value_type >() ) ); }

    /**
       最大・最小値
    **/

    /// @brief 最大値と最小値を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    /// min または max への参照先が未定義の場合は無視されるが、戻り値は true となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param min 求めた最小値を登録する変数へのポインタ
    /// @param max 求めた最大値を登録する変数へのポインタ
    /// @param less データ比較用二項関数オブジェクト
    /// @return データがない場合は false
    template< class Res, class In, class Less >
    bool ExtremeValues( In s, In e, Res* min, Res* max, Less less )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::ExtremeValues";

      if ( error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH ) == 0 ) return( false );

      if ( min != nullptr ) *min = *s;
      if ( max != nullptr ) *max = *s;
      while ( ++s != e ) {
        if ( min != nullptr )
          if ( less( *s, *min ) ) *min = *s;
        if ( max != nullptr )
          if ( less( *max, *s ) ) *max = *s;
      }

      return( true );
    }

    /// @brief 最大値と最小値を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// min または max への参照先が未定義の場合は無視されるが、戻り値は true となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param min 求めた最小値を登録する変数へのポインタ
    /// @param max 求めた最大値を登録する変数へのポインタ
    /// @return データがない場合は false
    template< class Res, class In >
    bool ExtremeValues( In s, In e, Res* min, Res* max )
    { return( ExtremeValues< Res >( s, e, min, max, std::less< typename std::iterator_traits< In >::value_type >() ) ); }

    /// @brief 最大値と最小値を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    /// min または max への参照先が未定義の場合は無視されるが、戻り値は true となる。
    ///
    /// @param c データ列への参照
    /// @param min 求めた最小値を登録する変数へのポインタ
    /// @param max 求めた最大値を登録する変数へのポインタ
    /// @param less データ比較用二項関数オブジェクト
    /// @return データがない場合は false
    template< class Res, class Container, class Less >
    bool ExtremeValuesAll( const Container& c, Res* min, Res* max, Less less )
    {
      return( ExtremeValues< Res >( c.begin(), c.end(), min, max, less ) );
    }

    /// @brief 最大値と最小値を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// min または max への参照先が未定義の場合は無視されるが、戻り値は true となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param c データ列への参照
    /// @param min 求めた最小値を登録する変数へのポインタ
    /// @param max 求めた最大値を登録する変数へのポインタ
    /// @return データがない場合は false
    template< class Res, class Container >
    bool ExtremeValuesAll( const Container& c, Res* min, Res* max )
    { return( ExtremeValuesAll< Res >( c, min, max, std::less< typename Container::value_type >() ) ); }

    /**
       ミッドレンジ
    **/

    /// @brief ミッドレンジ ( min + max ) / 2 を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めたミッドレンジ (データがない場合は要素の初期値)
    template< class Res, class In, class Less >
    Res MidRange( In s, In e, Less less )
    {
      Res min, max;
      if ( ! ExtremeValues< Res >( s, e, &min, &max, less ) )
        return( Res() );

      return( ( min + max ) / Res( 2 ) );
    }

    /// @brief ミッドレンジ ( min + max ) / 2 を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めたミッドレンジ (データがない場合は要素の初期値)
    template< class Res, class In >
    Res MidRange( In s, In e )
    { return( MidRange< Res >( s, e, std::less< typename std::iterator_traits< In >::value_type >() ) ); }

    /// @brief ミッドレンジ ( min + max ) / 2 を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// @param c データ列への参照
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めたミッドレンジ (データがない場合は要素の初期値)
    template< class Res, class Container, class Less >
    Res MidRangeAll( const Container& c, Less less )
    {
      return( MidRange< Res >( c.begin(), c.end(), less ) );
    }

    /// @brief ミッドレンジ ( min + max ) / 2 を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param c データ列への参照
    /// @return 求めたミッドレンジ (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res MidRangeAll( const Container& c )
    { return( MidRangeAll< Res >( c, std::less< typename Container::value_type >() ) ); }

    /**
       範囲
    **/

    /// @brief 範囲 max - min を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めた範囲 (データがない場合は要素の初期値)
    template< class Res, class In, class Less >
    Res Range( In s, In e, Less less )
    {
      Res min, max;
      if ( ! ExtremeValues< Res >( s, e, &min, &max, less ) )
        return( Res() );

      return( max - min );
    }

    /// @brief 範囲 max - min を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた範囲 (データがない場合は要素の初期値)
    template< class Res, class In >
    Res Range( In s, In e )
    { return( Range< Res >( s, e, std::less< typename std::iterator_traits< In >::value_type >() ) ); }

    /// @brief 範囲 max - min を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// @param c データ列への参照
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めた範囲 (データがない場合は要素の初期値)
    template< class Res, class Container, class Less >
    Res RangeAll( const Container& c, Less less )
    {
      return( Range< Res >( c.begin(), c.end(), less ) );
    }

    /// @brief 範囲 max - min を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param c データ列への参照
    /// @return 求めた範囲 (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res RangeAll( const Container& c )
    { return( RangeAll< Res >( c, std::less< typename Container::value_type >() ) ); }

    /**
       ミッドヒンジ
    **/

    /// @brief ミッドヒンジを求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めたミッドヒンジ (データがない場合は要素の初期値)
    template< class Res, class In, class Less >
    Res MidHinge( In s, In e, Less less )
    {
      std::vector< Res > q( 3 );
      if ( ! Quantile( s, e, q.begin(), q.end(), less ) )
        return( Res() );

      return( ( q[0] + q[2] ) / Res( 2 ) );
    }

    /// @brief ミッドヒンジを求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めたミッドヒンジ (データがない場合は要素の初期値)
    template< class Res, class In >
    Res MidHinge( In s, In e )
    { return( MidHinge< Res >( s, e, std::less< Res >() ) ); }

    /// @brief ミッドヒンジを求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用二項関数オブジェクトの型を表す。
    ///
    /// @param c データ列への参照
    /// @param less データ比較用二項関数オブジェクト
    /// @return 求めたミッドヒンジ (データがない場合は要素の初期値)
    template< class Res, class Container, class Less >
    Res MidHingeAll( const Container& c, Less less )
    {
      return( MidHinge< Res >( c.begin(), c.end(), less ) );
    }

    /// @brief ミッドヒンジを求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param c データ列への参照
    /// @return 求めたミッドヒンジ (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res MidHingeAll( const Container& c )
    { return( MidHingeAll< Res >( c, std::less< Res >() ) ); }

    /**
       分散値
    **/

    /**
       標本分散
    **/

    /// @brief 標本分散を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた標本分散 (データがない場合は要素の初期値)
    template< class Res, class In >
    Res SampleVariance( In s, In e )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::SampleVariance";

      auto sz = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e )" + error_message::NEGATIVE_LENGTH );
      if ( sz < 1 ) return( Res() );

      return( DevSq< Res >( s, e ) / Res( sz ) );
    }

    /// @brief 標本分散を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @return 求めた標本分散 (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res SampleVarianceAll( const Container& c )
    {
      return( SampleVariance< Res >( c.begin(), c.end() ) );
    }

    /**
       不偏分散
    **/

    /// @brief 不偏分散を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた不偏分散 (データ一つ以下の場合は要素の初期値)
    template< class Res, class In >
    Res UnbiasedVariance( In s, In e )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::UnbiasedVariance";

      auto sz = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );
      if ( sz < 2 ) return( Res() );

      return( DevSq< Res >( s, e ) / Res( sz - 1 ) );
    }

    /// @brief 不偏分散を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @return 求めた不偏分散 (データ一つ以下の場合は要素の初期値)
    template< class Res, class Container >
    Res UnbiasedVarianceAll( const Container& c )
    {
      return( UnbiasedVariance< Res >( c.begin(), c.end() ) );
    }

    /**
       四分偏差
    **/

    /// @brief 四分偏差を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用関数オブジェクトの型を表す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param less データ比較用関数オブジェクト
    /// @return 求めた四分偏差 (データがない場合は要素の初期値)
    template< class Res, class In, class Less >
    Res QuantileDeviation( In s, In e, Less less )
    {
      std::vector< Res > q( 3 );
      if ( ! Quantile( s, e, q.begin(), q.end(), less ) )
        return( Res() );

      return( ( q.back() - q.front() ) / Res( 2 ) );
    }

    /// @brief 四分偏差を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた四分偏差 (データがない場合は要素の初期値)
    template< class Res, class In >
    Res QuantileDeviation( In s, In e )
    { return( QuantileDeviation< Res >( s, e, std::less< Res >() ) ); }

    /// @brief 四分偏差を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Less はデータ比較用関数オブジェクトの型を表す。
    ///
    /// @param c データ列への参照
    /// @param less データ比較用関数オブジェクト
    /// @return 求めた四分偏差 (データがない場合は要素の初期値)
    template< class Res, class Container, class Less >
    Res QuantileDeviationAll( const Container& c, Less less )
    {
      return( QuantileDeviation< Res >( c.begin(), c.end(), less ) );
    }

    /// @brief 四分偏差を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// データ比較用二項関数オブジェクトとして std::less を利用。
    ///
    /// @param c データ列への参照
    /// @return 求めた四分偏差 (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res QuantileDeviationAll( const Container& c )
    { return( QuantileDeviationAll< Res >( c, std::less< Res >() ) ); }

    /**
       平均偏差
    **/

    /// @brief 平均偏差を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return 求めた平均偏差 (データがない場合は要素の初期値)
    template< class Res, class In >
    Res MeanDeviation( In s, In e )
    { return( TransMean< Res >( s, e, AbsDeviation< Res >( Mean< Res >( s, e ) ) ) ); }

    /// @brief 平均偏差を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @return 求めた平均偏差 (データがない場合は要素の初期値)
    template< class Res, class Container >
    Res MeanDeviationAll( const Container& c )
    {
      return( MeanDeviation< Res >( c.begin(), c.end() ) );
    }

    /**
       標準化積率
    **/

    /// @brief 標準化積率 ( 平均差と標準偏差の商の r 乗の和 ) を求める
    ///
    /// N 個のデータ列 {x} の平均を m、標準偏差をσとしたとき、標準化積率は
    /// Σ pow( ( x - m ) / σ, r ) / N
    /// で求められる
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In はコンテナクラスの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param r 次数
    /// @return 標準化積率 ( 標準偏差がゼロの場合は要素の初期値を返す )
    template< class Res, class In >
    Res StdMoment( In s, In e, distribution::Base::counter_type r )
    {
      using value_type = typename std::iterator_traits< In >::value_type;

      Res m = Mean< Res >( s, e );
      Res sd = std::sqrt( TransMean< Res >( s, e, Deviation< Res >( m ) ) );
      if ( sd == 0 ) return( Res() );

      return( TransMean< Res >( s, e, Moment< Res >( m, sd, value_type( r ) ) ) );
    }

    /// @brief 標準化積率 ( 平均差と標準偏差の商の r 乗の和 ) を求める
    ///
    /// データ列 {x} の平均を m、標準偏差をσとしたとき、標準化積率は
    /// Σ pow( ( x - m ) / σ, r )
    /// で求められる
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// @param c データ列への参照
    /// @param r 次数
    /// @return 標準化積率 ( 標準偏差がゼロの場合は要素の初期値を返す )
    template< class Res, class Container >
    Res StdMomentAll( const Container& c, distribution::Base::counter_type r )
    {
      return( StdMoment< Res >( c.begin(), c.end(), r ) );
    }

    /**
       二変数の積和
    **/

    /// @brief x, y の xa, ya との差の積和を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @param xa x データ列との差を計算する a
    /// @param ya y データ列との差を計算する a
    /// @return 計算結果
    template< class Res, class In1, class In2 >
    Res ProductSum( In1 xs, In1 xe, In2 ys, In2 ye, Res xa, Res ya )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::ProductSum";

      error_lib::CheckBivariable( xs, xe, ys, ye,
                                  FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : length of [ xs, xe ) " + error_message::NOT_EQUAL_LENGTH + " length of [ ys, ye )" );

      Res ans = Res();
      for ( ; xs != xe ; ++xs, ++ys )
        ans += ( *xs - xa ) * ( *ys - ya );

      return( ans );
    }

    /// @brief x, y の a との差の積和を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container1, Container2 はコンテナの型であることが前提となる。
    ///
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< Container1::const_iterator::difference_type > を投げる。
    ///
    /// @param xc 対象の x データ列
    /// @param yc 対象の y データ列
    /// @param xa x データ列との差を計算する a
    /// @param ya y データ列との差を計算する a
    /// @return 計算結果
    template< class Res, class Container1, class Container2 >
    Res ProductSumAll( const Container1& xc, const Container2& yc, Res xa, Res ya )
    {
      return( ProductSum( xc.begin(), xc.end(), yc.begin(), yc.end(), xa, ya ) );
    }

    /// @brief x, y の平均差の積和を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @return 計算結果
    template< class Res, class In1, class In2 >
    Res ProductSumDev( In1 xs, In1 xe, In2 ys, In2 ye )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::ProductSumDev";

      error_lib::CheckBivariable( xs, xe, ys, ye,
                                  FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : length of [ xs, xe ) " + error_message::NOT_EQUAL_LENGTH + " length of [ ys, ye )" );

      Res mx = Mean< Res >( xs, xe );
      Res my = Mean< Res >( ys, ye );

      return( ProductSum( xs, xe, ys, ye, mx, my ) );
    }

    /// @brief x, y の平均差の積和を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container1, Container2 はコンテナの型であることが前提となる。
    ///
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< Container1::const_iterator::difference_type > を投げる。
    ///
    /// @param xc 対象の x データ列
    /// @param yc 対象の y データ列
    /// @return 計算結果
    template< class Res, class Container1, class Container2 >
    Res ProductSumDevAll( const Container1& xc, const Container2& yc )
    {
      return( ProductSumDev< Res >( xc.begin(), xc.end(), yc.begin(), yc.end() ) );
    }

    /**
       標本共分散
    **/

    /// @brief 標本共分散を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @return 標本共分散(データがない場合はゼロを返す)
    template< class Res, class In1, class In2 >
    Res Covariance( In1 xs, In1 xe, In2 ys, In2 ye )
    {
      Res proSumDev = ProductSumDev< Res >( xs, xe, ys, ye );

      auto n = std::distance( xs, xe );
      if ( n == Res() ) return( Res() );

      return( proSumDev / Res( n ) );
    }

    /// @brief 標本共分散を求める
    ///
    /// テンプレート引数の Res は戻り値の型を表す。
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< Container1::const_iterator::difference_type > を投げる。
    ///
    /// @param xc 対象の x データ列
    /// @param yc 対象の y データ列
    /// @return 標本共分散
    template< class Res, class Container1, class Container2 >
    Res CovarianceAll( const Container1& xc, const Container2& yc )
    {
      return( Covariance< Res >( xc.begin(), xc.end(), yc.begin(), yc.end() ) );
    }

    /**
       U 統計量
    **/

    /// @brief U 値を求める
    ///
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    /// テンプレート引数の Less と Cmp はデータ比較用二項関数オブジェクトの型を表す。
    /// less は x データ列のソートで利用し、cmp は x, y の大小比較で利用する。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    ///
    /// @note x データ列のソートはデータをいったんコピーして行うため、元のデータには影響しない。
    ///         コピーに伴う処理時間がオーバーヘッドとなることに注意。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @param less データ x の大小比較用二項関数オブジェクト
    /// @param cmp データ x, y の大小比較用二項関数オブジェクト
    /// @return U 値
    template< class In1, class In2, class Less, class Cmp >
    distribution::Base::stat_type UValue( In1 xs, In1 xe, In2 ys, In2 ye, Less less, Cmp cmp )
    {
      using X = typename std::iterator_traits< In1 >::value_type;

      const std::string FUNC_NAME = "tadah_fussy::statistics::UValue";

      error_lib::CheckArrayLength( xs, xe, FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH );
      error_lib::CheckArrayLength( ys, ye, FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH );

      std::vector< X > x( xs, xe );
      std::sort( x.begin(), x.end(), less );

      // y の各要素より大きな x の要素数を求め、その和 U 値としてを計算
      distribution::Base::stat_type u = 0; // U値
      for ( In2 y = ys ; y != ye ; ++y ) {
        auto i = lower_bound( x.begin(), x.end(), *y, cmp );
        u += std::distance( i, x.end() );
        // x,y の要素が等しい場合は 0.5 と換算
        while ( i != x.end() )
          if ( *i++ == *y ) u -= 0.5;
      }

      return( u );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    ///
    /// @note x, y の比較には、x の型を X としたとき std::less< X > が使われることに注意。
    /// @note x データ列のソートはデータをいったんコピーして行うため、元のデータには影響しない。
    ///       コピーに伴う処理時間がオーバーヘッドとなることに注意。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @return U 値
    template< class In1, class In2 >
    distribution::Base::stat_type UValue( In1 xs, In1 xe, In2 ys, In2 ye )
    {
      using X = typename std::iterator_traits< In1 >::value_type;
      return( UValue( xs, xe, ys, ye, std::less< X >(), std::less< X >() ) );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の Container1, Container2 はコンテナの型であることが前提となる。
    /// テンプレート引数の Less と Cmp はデータ比較用二項関数オブジェクトの型を表す。
    /// less は x データ列のソートで利用し、cmp は x, y の大小比較で利用する。
    ///
    /// @note x データ列のソートはデータをいったんコピーして行うため、元のデータには影響しない。
    ///       コピーに伴う処理時間がオーバーヘッドとなることに注意。
    ///
    /// @param x 対象の x データ列
    /// @param y 対象の y データ列
    /// @param less データ x の大小比較用二項関数オブジェクト
    /// @param cmp データ x, y の大小比較用二項関数オブジェクト
    /// @return U 値
    template< class Container1, class Container2, class Less, class Cmp >
    distribution::Base::stat_type UValueAll( const Container1& x, const Container2& y, Less less, Cmp cmp )
    {
      return( UValue( x.begin(), x.end(), y.begin(), y.end(), less, cmp ) );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の Container1, Container2 はコンテナの型であることが前提となる。
    ///
    /// @note x, y の比較には、x の型を X としたとき std::less< X > が使われることに注意。
    /// @note x データ列のソートはデータをいったんコピーして行うため、元のデータには影響しない。
    ///       コピーに伴う処理時間がオーバーヘッドとなることに注意。
    ///
    /// @param x 対象の x データ列
    /// @param y 対象の y データ列
    /// @return U 値
    template< class Container1, class Container2 >
    distribution::Base::stat_type UValueAll( const Container1& x, const Container2& y )
    {
      using X = typename Container1::value_type;
      return( UValueAll( x, y, std::less< X >(), std::less< X >() ) );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の In はコンテナの iterator の型であることが前提となる。
    /// テンプレート引数の Pred は要素に対して真偽判定を行う叙述関数の型を表す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// Pred によって真を返す値を x, 偽を返す値を y として U 値を求める。
    /// x が小さい傾向にあるほど U 値は低くなる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @param op データの真偽判定用関数
    /// @return U 値
    template< class In, class Pred >
    distribution::Base::stat_type UValueByBool( In s, In e, Pred op )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::UValueByBool";

      error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );

      // x, y 各要素数を先に計算
      distribution::Base::counter_type xCnt = 0;
      distribution::Base::counter_type yCnt = 0;
      for ( In i = s ; i != e ; ++i ) {
        if ( op( *i ) )
          ++xCnt;
        else
          ++yCnt;
      }

      if ( xCnt == 0 || yCnt == 0 ) return( 0 );

      // y の各要素より大きな x の要素数を求め、その和を計算
      distribution::Base::stat_type u = 0; // U値
      for ( ; s != e ; ++s ) {
        if ( op( *s ) ) {
          --xCnt;
        } else {
          u += xCnt;
          if ( --yCnt == 0 ) break;
        }
      }

      return( u );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の In はコンテナの iterator の型であることが前提となる。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// 真値を x, 偽値を y として U 値を求める。x が小さい傾向にあるほど U 値は低くなる。
    ///
    /// @param s 対象のデータ列の開始反復子
    /// @param e 対象のデータ列の末尾の次の反復子
    /// @return U 値
    template< class In >
    distribution::Base::stat_type UValueByBool( In s, In e )
    {
      using value_type = typename std::iterator_traits< In >::value_type;
      using namespace std::placeholders;

      return( UValueByBool( s, e, std::bind( std::equal_to< value_type >(), _1, ! value_type() ) ) );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// テンプレート引数の Pred は要素に対して真偽判定を行う叙述関数の型を表す。
    ///
    /// Pred によって真を返す値を x, 偽を返す値を y として U 値を求める。
    /// x が小さい傾向にあるほど U 値は低くなる。
    ///
    /// @param c 対象のデータ列への参照
    /// @param op データの真偽判定用関数
    /// @return U 値
    template< class Container, class Pred >
    distribution::Base::stat_type UValueByBoolAll( const Container& c, Pred op )
    {
      return( UValueByBool( c.begin(), c.end(), op ) );
    }

    /// @brief U 値を求める
    ///
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    ///
    /// 真値を x, 偽値を y として U 値を求める。x が小さい傾向にあるほど U 値は低くなる。
    ///
    /// @param c 対象のデータ列への参照
    /// @return U 値
    template< class Container >
    distribution::Base::stat_type UValueByBoolAll( const Container& c )
    {
      return( UValueByBool( c.begin(), c.end() ) );
    }

    /**
       符号付き順位和
    **/

    /// @brief 符号付き順位和を求める
    ///
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    /// テンプレート引数の Less と Eq はデータ比較用二項関数オブジェクトの型を表す。
    /// less は差分 x - y のソートで利用し、eq は差分 x - y が等しいかどうかを判定するために利用する。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
    ///
    /// @note less は差分の絶対値を比較することを想定している。処理の中では絶対値変換を行なっていないことに注意。
    /// @note eq は、差分どうしが等しいかをチェックする他に、差分がゼロかをチェックする時にも利用される。
    /// @note 差分に対する型は x データ列の要素の型が採用される。x, y の型が異なる場合は注意。
    ///       また、x データ列の型が符号なしの場合は正しい結果が得られない。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @param cnt 差分がゼロ以外の要素の数を取得する変数へのポインタ(NULLなら無視)
    /// @param less 差分の大小比較用二項関数オブジェクト
    /// @param eq 差分の等値判定用二項関数オブジェクト
    /// @return 符号付き順位和
    template< class In1, class In2, class Less, class Eq >
    distribution::Base::stat_type SRankValue( In1 xs, In1 xe, In2 ys, In2 ye, size_t* cnt, Less less, Eq eq )
    {
      using X = typename std::iterator_traits< In1 >::value_type;
      using stat_type = distribution::Base::stat_type;

      const std::string FUNC_NAME = "tadah_fussy::statistics::SRankValue";

      error_lib::CheckBivariable( xs, xe, ys, ye,
                                  FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : length of [ xs, xe ) " + error_message::NOT_EQUAL_LENGTH + " length of [ ys, ye )" );

      // x, y の差分 x - y を計算
      std::vector< X > diff;
      for ( ; xs != xe ; ++xs, ++ys ) {
        X d = *xs - *ys;
        if ( ! eq( d, X() ) )
          diff.push_back( d );
      }
      if ( cnt != nullptr )
        *cnt = diff.size();

      // データを昇順で並べる
      // 差の絶対値で並べ替えする必要があることに注意
      sort( diff.begin(), diff.end(), less );

      // 各要素の順位を計算
      using Ranks = std::vector< stat_type >;
      Ranks rank( diff.size() ); // 差の順位
      for ( Ranks::size_type i = 0 ; i < rank.size() ; ) {
        // 絶対値が等しいグループの末尾を探索
        Ranks::size_type j = i + 1;
        for ( ; j < rank.size() ; ++j )
          if ( ! eq( std::abs( diff[i] ), std::abs( diff[j] ) ) )
            break;
        // 順位和の計算(値が等しい要素は等分配)
        stat_type rankSum = 0; // 順位和
        for ( Ranks::size_type k = i ; k < j ; ++k )
          rankSum += k + 1;
        rankSum /= j - i;
        // 順位和を配列に代入
        for ( ; i < j ; ++i )
          rank[i] = rankSum;
      }

      // 正値に対する順位和の計算
      stat_type sp = 0;
      for ( Ranks::size_type i = 0 ; i < diff.size() ; ++i )
        if ( diff[i] > 0 ) sp += rank[i];

      // Δs = 正値の順位和(sp) - 負値の順位和(sm) を計算
      // sp + sm = cnt( cnt + 1 ) / 2 より
      // Δs = sp - sm = sp - { cnt( cnt + 1 ) / 2 - sp }
      //    = 2sp - cnt( cnt + 1 ) / 2
      return( ( sp * 4.0 - ( diff.size() * ( diff.size() + 1 ) ) ) / 2.0 );
    }

    /// @brief 符号付き順位和を求める
    ///
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    ///
    /// - x データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - y データ列の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
    ///
    /// @note 差分に対する型は x データ列の要素の型が採用される。x, y の型が異なる場合は注意。
    /// @note x, y でデータ数が一致しない場合、少ない側にそろえられることに注意。
    ///
    /// @param xs 対象の x データ列の開始反復子
    /// @param xe 対象の x データ列の末尾の次の反復子
    /// @param ys 対象の y データ列の開始反復子
    /// @param ye 対象の y データ列の末尾の次の反復子
    /// @param cnt 差分がゼロ以外の要素の数を取得する変数へのポインタ(NULLなら無視)
    /// @return 符号付き順位和
    template< class In1, class In2 >
    distribution::Base::stat_type SRankValue( In1 xs, In1 xe, In2 ys, In2 ye, size_t* cnt )
    {
      using X = typename std::iterator_traits< In1 >::value_type;
      return( SRankValue( xs, xe, ys, ye, cnt, numeric_lib::LessByAbs< X >(), std::equal_to< X >() ) );
    }

    /// @brief 符号付き順位和を求める
    ///
    /// テンプレート引数の Container1, Container2 はコンテナの型であることが前提となる。
    /// テンプレート引数の Less と Eq はデータ比較用二項関数オブジェクトの型を表す。
    /// less は差分 x - y のソートで利用し、eq は差分 x - y が等しいかどうかを判定するために利用する。
    ///
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< Container1::const_iterator::difference_type > を投げる。
    ///
    /// @note less は差分の絶対値を比較することを想定している。処理の中では絶対値変換を行なっていないことに注意。
    /// @note eq は、差分どうしが等しいかをチェックする他に、差分がゼロかをチェックする時にも利用される。
    /// @note 差分に対する型は x データ列の要素の型が採用される。x, y の型が異なる場合は注意。
    /// @note x, y でデータ数が一致しない場合、少ない側にそろえられることに注意。
    ///
    /// @param x 対象の x データ列
    /// @param y 対象の y データ列
    /// @param cnt 差分がゼロ以外の要素の数を取得する変数へのポインタ(NULLなら無視)
    /// @param less 差分の大小比較用二項関数オブジェクト
    /// @param eq 差分の等値判定用二項関数オブジェクト
    /// @return 符号付き順位和
    template< class Container1, class Container2, class Less, class Eq >
    distribution::Base::stat_type SRankValueAll( const Container1& x, const Container2& y, size_t* cnt, Less less, Eq eq )
    {
      return( SRankValue( x.begin(), x.end(), y.begin(), y.end(), cnt, less, eq ) );
    }

    /// @brief 符号付き順位和を求める
    ///
    /// テンプレート引数の Container1, Container2 はコンテナの型であることが前提となる。
    ///
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< Container1::const_iterator::difference_type > を投げる。
    ///
    /// @note 差分に対する型は x データ列の要素の型が採用される。x, y の型が異なる場合は注意。
    /// @note x, y でデータ数が一致しない場合、少ない側にそろえられることに注意。
    ///
    /// @param x 対象の x データ列
    /// @param y 対象の y データ列
    /// @param cnt 差分がゼロ以外の要素の数を取得する変数へのポインタ(NULLなら無視)
    /// @return 符号付き順位和
    template< class Container1, class Container2 >
    distribution::Base::stat_type SRankValueAll( const Container1& x, const Container2& y, size_t* cnt )
    {
      return( SRankValue( x.begin(), x.end(), y.begin(), y.end(), cnt ) );
    }

    /**
       正規化処理
    **/

    /// @brief データを正規化する
    ///
    /// テンプレート引数の In はコンテナの iterator の型であることが前提となる。
    /// 標準偏差がゼロである場合は正規化できないため何も処理せず false を返す。
    ///
    /// - データ列の長さ( s に対する e の相対位置 )が負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
    ///
    /// @note 平均値と標準偏差を計算するときの型はコンテナの値の型と同一とする。そのため、
    ///       整数型を渡した場合は大きな誤差が発生することに注意。
    ///
    /// @param s : データ列の開始位置
    /// @param e : データ列の末尾の次の位置
    /// @return 正規化できない場合 false を返す
    template< class In >
    bool Normalize( In s, In e )
    {
      using value_type = typename std::iterator_traits< In >::value_type;

      const std::string FUNC_NAME = "tadah_fussy::statistics::Normalize";

      value_type sz = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );
      if ( sz <= 0 ) return( false );

      value_type mu = math_lib::Sum< value_type >( s, e ) / sz; // 平均値
      value_type sigma = std::sqrt( math_lib::Sum< value_type >( s, e,
                                                                 math_lib::OpBind( Deviation< value_type >( mu ) ) ) / sz ); // 標準偏差

      if ( sigma == 0 )
        return( false );

      for ( ; s != e ; ++s )
        *s = ( *s - mu ) / sigma;

      return( true );
    }

    /// @brief データを正規化する
    ///
    /// テンプレート引数の Container はコンテナの型であることが前提となる。
    /// 平均値と標準偏差を計算するときの型はコンテナの値の型と同一とする。そのため、
    /// 整数型を渡した場合は大きな誤差が発生することに注意。
    /// 標準偏差がゼロである場合は正規化できないため何も処理せず false を返す。
    ///
    /// @param container : データ列
    /// @return 正規化できない場合 false を返す
    template< class Container >
    bool NormalizeAll( Container& container )
    {
      return( Normalize( container.begin(), container.end() ) );
    }

    /**
       共分散行列
    **/

    /// @brief 共分散行列を求める
    ///
    /// 引数として渡されるデータ列は、ある列に同種のデータが並ぶ ( 行が一つの標本の独立変数である ) とする。
    ///
    /// @param data データ列
    /// @param cov 求めた共分散行列を保持する対称行列へのポインタ
    /// @return なし
    template< class T, class U >
    void CovarianceMatrix( const Matrix< T >& data, SymmetricMatrix< U >* cov )
    {
      assert( cov != nullptr );

      // 行列に対する各型
      using size_type = typename Matrix< T >::size_type;

      size_type m = data.cols(); // データ種の数
      cov->resize( m );

      if ( m == 0 ) return;

      // 共分散行列の要素を計算する
      for ( size_type c1 = 0 ; c1 < m ; ++c1 ) {
        auto col1 = data.column( c1 );
        ( *cov )[c1][c1] = SampleVariance< U >( col1, col1.end() );
        for ( size_type c2 = c1 + 1 ; c2 < m ; ++c2 ) {
          auto col2 = data.column( c2 );
          ( *cov )[c1][c2] = Covariance< U >( col1, col1.end(), col2, col2.end() );
        }
      }
    }

    /**
       ピアソン・カイ二乗統計量
    **/

    /// @brief ピアソン・カイ二乗統計量を求める
    ///
    /// テンプレート引数の In1, In2 はコンテナの iterator の型であることが前提となる。
    ///
    /// - データ列 [ os, oe ) の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In1 >::difference_type > を投げる。
    /// - データ列 [ es, ee ) の長さが負数(開始と終了が逆)なら、
    ///   例外 exception::NegativeLength< std::iterator_traits< In2 >::difference_type > を投げる。
    /// - 二つのデータ列の要素数は一致していなければならない。
    ///   そうでない場合は例外 exception::NotEqualLength< std::iterator_traits< In1 >::difference_type > を投げる。
    ///
    /// @param os, oe 観測値の開始・末尾の次の位置
    /// @param es, ee 期待値の開始・末尾の次の位置
    /// @return ピアソン・カイ二乗統計量
    template< class In1, class In2 >
    distribution::Base::stat_type PearsonChiSq( In1 os, In1 oe, In2 es, In2 ee )
    {
      using Obj = typename std::iterator_traits< In1 >::value_type;
      using Exp = typename std::iterator_traits< In2 >::value_type;
      using stat_type = distribution::Base::stat_type;

      const std::string FUNC_NAME = "tadah_fussy::statistics::PearsonChiSq";

      struct ChiSquare
      {
        stat_type operator()( Obj obj, Exp exp )
        {
          if ( exp == Exp() )
            return( ( obj == Obj() ) ? stat_type() : distribution::Base::infinity() );
          else
            return( std::pow( stat_type( obj - exp ), stat_type( 2 ) ) / exp );
        }
      };

      error_lib::CheckBivariable( os, oe, es, ee,
                                  FUNC_NAME + " : [ os, oe ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : [ es, ee ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : length of [ os, oe ) " + error_message::NOT_EQUAL_LENGTH + " length of [ es, ee )" );

      return( std::inner_product( os, oe, es, stat_type(), std::plus< stat_type >(), ChiSquare() ) );
    }

    namespace distribution
    {
      /*
        H::CalcRankSq : 順位の二乗の平均を求める

        順列 perm が各集合ごとに固めて並んでいるとみなし、各集合の順位から Sc / σ^2 を求める。
      */
      template< class T >
      class H::CalcRankSq
      {
        using const_iterator = typename std::vector< T >::const_iterator;
        // 順列の開始位置
        const_iterator perm_;

      public:

        // 順列の開始位置を指定して構築
        CalcRankSq( const_iterator perm )
          : perm_( perm ) {}

        // 現在の集団の順位の二乗平均を返す。perm_ は次の集団に対する順列の位置に更新される。
        distribution::Base::stat_type operator()( T t )
        {
          using stat_type = distribution::Base::stat_type;

          if ( t == T() ) return( T() );

          stat_type sum = math_lib::Sum< stat_type >( perm_, perm_ + t ); // 順位和
          perm_ += t; // 順列の更新

          return( sum * sum / t );
        }
      };

      /*
        H コンストラクタ

        [ s, e ] は各集合の個数を表すデータ列
      */
      template< class In >
      H::H( In s, In e, distribution::Base::stat_type threshold )
        : std::map< distribution::Base::stat_type, distribution::Base::stat_type, numeric_lib::ApproxLess< distribution::Base::stat_type > >
        ( numeric_lib::ApproxLess< distribution::Base::stat_type >( threshold ) )
      {
        // 要素数を表す型 ( In の value_type )
        using count_type = typename std::iterator_traits< In >::value_type;
        // 戻り値の型
        using stat_type = distribution::Base::stat_type;

        const std::string FUNC_NAME = "tadah_fussy::statistics::distribution::H constructor";

        error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );
        for ( In i = s ; i < e ; ++i )
          error_lib::NNNum( *i, FUNC_NAME + " : all data in [ s, e ) " + error_message::NNNUM );

        count_type sum = math_lib::Sum< count_type >( s, e ); // 要素数の総和
        if ( sum <= count_type() ) return;

        // 順列は 1 から始まる自然数列で初期化する
        std::vector< count_type > perm( sum );
        for ( count_type c = 1 ; c <= sum ; ++c )
          perm[c - 1] = c;
        stat_type permCnt = std::tgamma( stat_type( sum + 1 ) ); // 順列の総数

        // 各要素の順位の平均と分散
        stat_type ave = stat_type( sum * ( sum + 1 ) * ( sum + 1 ) ) / 4.0;
        stat_type var = stat_type( sum * ( sum + 1 ) ) / 12.0;

        // Sc / σ^2 (確率変数) を求めてコンテナに登録する
        do {
          // 確率変数を求める
          // CalcRankSq は各集合の順列から順位の二乗平均を求める。
          // これを [ s, e ] の間で繰り返すことで全集合の順位の二乗平均和が得られる。
          stat_type x = ( math_lib::Sum< stat_type >( s, e, math_lib::OpBind( CalcRankSq< count_type >( perm.begin() ) ) ) - ave ) / var;
          // 求めた確率変数に対する頻度をインクリメントする
          iterator it = find( x );
          if ( it == end() )
            (*this)[x] = 1;
          else
            it->second += 1;
        } while ( next_permutation( perm.begin(), perm.end() ) );

        // 和を確率密度に変換
        for ( iterator it = begin() ; it != end() ; ++it )
          it->second /= permCnt;
      }

      /*
        Multinomial コンストラクタ

        [ s, e ) : 各事象の確率
        n : 総試行回数
      */
      template< class In >
      Multinomial::Multinomial( In s, In e, var_type n )
        : n_( n )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::distribution::Multinomial constructor";

        error_lib::PNum( n_, FUNC_NAME + " : n " + error_message::PNUM );
        error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );

        p_.assign( s, e );
        p_.push_back( 1.0 - math_lib::Sum< stat_type >( s, e ) );

        for ( auto i = p_.begin() ; i != p_.end() ; ++i )
          error_lib::In01( *i, FUNC_NAME + " : all probability value " + error_message::IN01 );
      }

      /*
        NegativeMultinomial コンストラクタ

        [ s, e ) : 各事象の確率
        s : 事象の起こらない回数
      */
      template< class In >
      NegativeMultinomial::NegativeMultinomial( In ps, In pe, var_type s )
        : s_( s )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::distribution::NegativeMultinomial constructor";

        error_lib::PNum( s, FUNC_NAME + " : s " + error_message::PNUM );
        error_lib::CheckArrayLength( ps, pe, FUNC_NAME + " : [ ps, pe ) " + error_message::NEGATIVE_LENGTH );

        p_.assign( ps, pe );
        p0_ = error_lib::PNum( error_lib::In01( 1.0 - math_lib::Sum< stat_type >( ps, pe ),
                                                FUNC_NAME + " : 1 - ( sum of [ ps, pe ) ) " + error_message::IN01 ),
                               FUNC_NAME + " : ( 1 - sum of [ ps, pe ) ) " + error_message::PNUM );

        for ( auto i = p_.begin() ; i != p_.end() ; ++i )
          error_lib::In01( *i, FUNC_NAME + " : all probability value " + error_message::IN01 );
      }

    } // namespace distribution

    /* ----------------------------
       ヒストグラム作成用関数群
       ---------------------------- */

    /*
      Histogram< T, Frequency >::accumulate : データ列 [ s, e ) から度数を計算する
    */
    template< class T, class Frequency >
    template< class In >
    void Histogram< T, Frequency >::accumulate( In s, In e, bool setNewData )
    {
      const std::string FUNC_NAME = "Histogram< T, Frequency >::accumulate";

      error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );

      for ( ; s != e ; ++s ) {
        auto it = this->find( *s );
        if ( it == std::map< T, Frequency >::end() ) {
          if ( setNewData ) (*this)[*s] = 1;
        } else {
          ++( it->second );
        }
      }
    }

    /**
       @brief ヒストグラム用名前空間

       Histogram クラスを使ってヒストグラムを作成するためのヘルパ関数など
    **/
    namespace histogram
    {
      /// @brief 階級をあらかじめ追加する
      ///
      /// - データ列の指定範囲が有効かを内部でチェックしている。
      ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
      ///
      /// @note 前に入っていた階級は消去を行わないことに注意。
      ///
      /// @param s 階級として使うデータ列の先頭の反復子
      /// @param e 階級として使うデータ列の末尾の次の反復子
      /// @param hist 追加する対象の Histogram オブジェクトへのポインタ
      /// @return なし
      template< class In, class T, class Frequency >
      void AddClass( In s, In e, Histogram< T, Frequency >* hist )
      {
        assert( hist != nullptr );

        const std::string FUNC_NAME = "tadah_fussy::statistics::histogram::AddClass";

        error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );

        for ( ; s != e ; ++s )
          hist->insert( *s, Frequency() );
      }

      /// @brief 階級をあらかじめ追加する(RangeClass追加用)
      ///
      /// 隣り合うデータから RangeClass を作成して追加する。
      ///
      /// - データ列の指定範囲が有効かを内部でチェックしている。
      ///   有効でない場合は例外 exception::NegativeLength< std::iterator_traits< In >::difference_type > を投げる。
      ///
      /// @note 前に入っていた階級は消去を行わないことに注意。
      ///
      /// @param s 範囲を持つ配列の先頭の反復子
      /// @param e 範囲を持つ配列の末尾の次の反復子
      /// @param hist 追加する対象の Histogram オブジェクトへのポインタ
      /// @return なし
      template< class In, class T, class Frequency >
      void AddRangeClass( In s, In e, Histogram< RangeClass< T >, Frequency >* hist )
      {
        assert( hist != nullptr );

        const std::string FUNC_NAME = "tadah_fussy::statistics::histogram::AddRangeClass";

        error_lib::CheckArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );

        for ( In sL = s++ ; s != e ; ++sL , ++s )
          hist->insert( RangeClass< T >( sL, s ), Frequency() );
      }

      /// @brief 各範囲の幅を指定して階級を追加する
      ///
      /// min からスタートして range ずつ増加させながら cnt 個の階級を作成する
      ///
      /// @note 前に入っていた階級は消去を行わないことに注意。
      ///
      /// @param min 範囲の最小値
      /// @param range 各範囲の幅
      /// @param cnt 階級の数
      /// @param hist 追加する対象の Histogram オブジェクトへのポインタ
      /// @return なし
      template< class T, class Frequency >
      void AddRangeByStep( T min, T range, Frequency cnt,
                           Histogram< RangeClass< T >, Frequency >* hist )
      {
        using pair = typename std::pair< RangeClass< T >, Frequency >;

        assert( hist != nullptr );

        for ( Frequency i = 0 ; i < cnt ; ++i ) {
          hist->insert( pair( RangeClass< T >( min, min + range ), Frequency() ) );
          min += range;
        }
      }

      /// @brief 全体の範囲の幅を指定して階級を追加する
      ///
      /// value1 と value2 を境界とする範囲を cnt 個に分割して階級を作成する。
      ///
      /// @note 前に入っていた階級は消去を行わないことに注意。
      ///
      /// @param value1 範囲の最小値(または最大値)
      /// @param value2 範囲の最大値(または最小値)
      /// @param cnt 階級の数
      /// @param hist 追加する対象の Histogram オブジェクトへのポインタ
      /// @return なし
      template< class T, class Frequency >
      void AddRangeByCount( T value1, T value2, Frequency cnt,
                            Histogram< RangeClass< T >, Frequency >* hist )
      {
        assert( hist != nullptr );

        if ( value1 > value2 ) std::swap( value1, value2 );
        T range = ( value2 - value1 ) / cnt;
        AddRangeByStep( value1, range, cnt, hist );
      }

      // 値 value を文字列にして返す
      //
      // 値は classFormat の定義に従って整形される。
      template< class T >
      std::string ClassName::operator()( T value ) const
      {
        std::ostringstream oss;

        oss.copyfmt( classFormat );
        oss << value;

        return( oss.str() );
      }

      // 値 value を文字列にして返す(RangeClass用)
      //
      // 範囲値は classFormat の定義に従って整形される。
      template< class T >
      std::string ClassName::operator()( RangeClass< T > value ) const
      {
        std::ostringstream oss;

        oss.copyfmt( classFormat );
        oss << value.lower << "-" << value.upper;

        return( oss.str() );
      }

      // 度数の最大値 maxFreq と階級名の最大文字数 maxClassLen を求める
      // cit 上の度数・階級名が最大値より大きければ更新する
      template< class T, class Frequency >
      void Print< T, Frequency >::getMaxSize
      ( const_iterator cit, std::streamsize* maxFreq,
        std::string::size_type* maxClassLen ) const
      {
        if ( Frequency( *maxFreq ) < cit->second )
          *maxFreq = cit->second;
        std::string name = className( cit->first );
        if ( *maxClassLen < name.length() )
          *maxClassLen = name.length();
      }

      // ヒストグラム(一つの階級に対する度数グラフ)を文字列に変換して返す
      // "name |***... freq" の形式に変換される
      template< class T, class Frequency >
      std::string Print< T, Frequency >::classData
      ( const T& name, const Frequency& freq,
        std::streamsize unitSize,
        std::string::size_type maxClassLen ) const
      {
        std::ostringstream oss;

        // 階級の幅の表示
        oss.width( maxClassLen );
        oss << className( name ) << " |";

        // ヒストグラムの表示
        oss.width( unitSize + 1 );
        oss.fill( '*' );
        oss << " " << freq;

        return( oss.str() );
      }

      // ヒストグラムを os に出力する
      template< class T, class Frequency >
      void Print< T, Frequency >::operator()
        ( const Histogram< T, Frequency >& hist,
          std::ostream& os,
          std::streamsize maxRange ) const
      {
        if ( hist.size() == 0 ) return;

        // 度数と階級名の文字数の最大値を取得する
        std::streamsize maxFreq = 0;
        std::string::size_type maxClassLen = 0;
        for ( auto cit = hist.begin() ; cit != hist.end() ; ++cit )
          getMaxSize( cit, &maxFreq, &maxClassLen );

        for ( auto cit = hist.begin() ; cit != hist.end() ; ++cit ) {
          // 度数の表示桁数
          std::streamsize unitSize = double( maxRange ) * cit->second / maxFreq + 0.5;
          os << classData( cit->first, cit->second, unitSize, maxClassLen ) << std::endl;
        }
      }

      // 度数(maxFreq)と階級名の文字数(maxClassLen)の最大値を取得し、度数でソートしたコンテナ(pareto)を作成する
      template< class T, class Frequency >
      void PrintPareto< T, Frequency >::getMaxSize
      ( const_iterator cit, std::streamsize* maxFreq,
        std::string::size_type* maxClassLen, pareto_type* pareto ) const
      {
        Print< T, Frequency >::getMaxSize( cit, maxFreq, maxClassLen );
        pareto->insert( std::pair< Frequency, T >( cit->second, cit->first ) );
      }

      // ヒストグラム hist を度数順に os へ出力する(パレート図)
      template< class T, class Frequency >
      void PrintPareto< T, Frequency >::operator()
        ( const Histogram< T, Frequency >& hist,
          std::ostream& os,
          std::streamsize maxRange ) const
      {
        // 頻度順に並べた結果を保持する multimap
        pareto_type sortedFreq;

        if ( hist.size() == 0 ) return;

        // 度数と階級名の文字数の最大値を取得する
        std::streamsize maxFreq = 0;
        std::string::size_type maxClassLen = 0;
        for ( auto cit = hist.begin() ; cit != hist.end() ; ++cit )
          getMaxSize( cit, &maxFreq, &maxClassLen, &sortedFreq );

        for ( auto crit = sortedFreq.rbegin() ; crit != sortedFreq.rend() ; ++crit ) {
          // 度数の表示桁数
          std::streamsize unitSize = double( maxRange ) * crit->first / maxFreq + 0.5;
          os << this->classData( crit->second, crit->first, unitSize, maxClassLen ) << std::endl;
        }
      }

    } // namespace histogram

    /************************************************************************
       要約統計量オブジェクト
    ************************************************************************/

    // データ列 [ s, e ) を指定して構築
    template< class T >
    template< class In >
    BaseMoment1< T >::BaseMoment1( In s, In e )
      : count_( 0 ), sum_( value_type() )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::BaseMoment1< T > constructor";

      count_ = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH );
      sum_ = math_lib::Sum< value_type >( s, e, math_lib::SumOp< value_type >() );
    }

    /*
      MomentPair1 コンストラクタ : データ列 [ s1, e1 ) [ s2, e2 ) を指定して構築
    */
    template< class In1, class In2 >
    MomentPair1::MomentPair1( In1 s1, In1 e1, In2 s2, In2 e2 )
      : BaseMoment1< value_type >()
    {
      using math_lib::Sum;

      const std::string FUNC_NAME = "MomentPair1 constructor";

      count_ = error_lib::CheckBivariable( s1, e1, s2, e2,
                                           FUNC_NAME + " : [ s1, e1 ) " + error_message::NEGATIVE_LENGTH,
                                           FUNC_NAME + " : [ s2, e2 ) " + error_message::NEGATIVE_LENGTH,
                                           FUNC_NAME + " : length of [ s1, e1 ) " + error_message::NOT_EQUAL_LENGTH + " length of [ s2, e2 )" );
      sum_ = value_type( Sum< stat_type >( s1, e1 ), Sum< stat_type >( s2, e2 ) );
    }

    /*
      Moment2 コンストラクタ
    */
    template< class In >
    Moment2::Moment2( In s, In e ) : BaseMoment2< Moment1 >( s, e )
    {
      using func = value_type ( * )( value_type, value_type );
      using math_lib::Sum;
      using math_lib::OpBind;
      using namespace std::placeholders;

      sumSq_ = Sum< value_type >( s, e, OpBind( std::bind( std::ptr_fun( static_cast< func >( std::pow ) ), _1, 2.0 ) ) );
    }

    /*
      MomentPair2 コンストラクタ : データ列 [ s1, e1 ) [ s2, e2 ) を指定して構築
    */
    template< class In1, class In2 >
    MomentPair2::MomentPair2( In1 s1, In1 e1, In2 s2, In2 e2 )
      : proSum_( 0 )
    {
      using func = stat_type ( * )( stat_type, stat_type );
      using math_lib::Sum;
      using math_lib::OpBind;
      using namespace std::placeholders;

      moment1_ = MomentPair1( s1, e1, s2, e2 );

      stat_type sumSq1 = Sum< stat_type >( s1, e1, OpBind( std::bind( std::ptr_fun( static_cast< func >( std::pow ) ), _1, 2.0 ) ) );
      stat_type sumSq2 = Sum< stat_type >( s2, e2, OpBind( std::bind( std::ptr_fun( static_cast< func >( std::pow ) ), _1, 2.0 ) ) );
      sumSq_ = value_type( sumSq1, sumSq2 );

      while ( s1 != e1 )
        proSum_ += *s1++ * *s2++;
    }

    /*
      MomentPair2 コンストラクタ : データ列 [ s, e ) を指定して構築

      データ列は std::pair を要素とするコンテナであることを想定している。
    */
    template< class In >
    MomentPair2::MomentPair2( In s, In e )
      : BaseMoment2< MomentPair1 >( s, e ), proSum_( 0 )
    {
      using math_lib::Sum;
      using math_lib::OpBind;
      using math_lib::SumOp;

      if ( moment1_.count() == 0 ) return;

      sumSq_ = Sum< value_type >( s, e, OpBind< value_type >( &powPair, SumOp< value_type >() ) );
      proSum_ = Sum< stat_type >( s, e, OpBind( ptr_fun( product ) ) );
    }

    /************************************************************************
       区間推定
    ************************************************************************/

    namespace interval_estimation
    {
      template< class In >
      VarN::VarN( In s, In e, distribution::Base::stat_type mu )
      {
        const std::string FUNC_NAME = "IntervalEstimation_VarN constructor";

        count_ = error_lib::PNum< counter_type >
          ( error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH ),
            FUNC_NAME + " : length of [ s, e ) " + error_message::PNUM );

        Deviation< stat_type > dev( mu );
        sumSq_ = math_lib::Sum< stat_type >( s, e, math_lib::OpBind( dev ) );
      }

    }

    /************************************************************************
        検定
    ************************************************************************/

    namespace test
    {
      /*
        ChiSquare コンストラクタ ( 等確率分布用 )
      */
      template< class T >
      ChiSquare::ChiSquare( const Histogram< T, distribution::Base::counter_type >& hist )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::test::ChiSquare constructor";

        error_lib::GT< counter_type >( hist.size(), 1, FUNC_NAME + " : size of hist " + error_message::GT + " 1" );

        counter_type total = 0; // 総標本数

        for ( auto i = hist.begin() ; i != hist.end() ; ++i ) {
          obj_.push_back( i->second );
          total += i->second;
        }

        auto n = hist.size(); // 級数
        stat_type r = stat_type( total ) / n; // 期待値
        exp_.assign( n, r );

        //calcChiSquare();
        chiSquare_ = PearsonChiSq( obj_.begin(), obj_.end(), exp_.begin(), exp_.end() );
        calcP();
      }

      /*
        RepMeasANOVA コンストラクタ
      */
      template< class In >
      RepMeasANOVA::RepMeasANOVA( In s, In e, counter_type classCnt, counter_type rowSize, counter_type colSize )
        : data_( rowSize, colSize * classCnt ), classCnt_( classCnt ), cols_( colSize ),
          mr_( rowSize ), mc_( colSize ), mrc_( rowSize, colSize )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::test::RepMeasANOVA constructor";

        error_lib::GT< counter_type >( classCnt, 1, FUNC_NAME + " : classCnt " + error_message::GT + " 1" );
        error_lib::GT< counter_type >( rowSize, 1, FUNC_NAME + " : rowSize " + error_message::GT + " 1" );
        error_lib::GT< counter_type >( colSize, 1, FUNC_NAME + " : colSize " + error_message::GT + " 1" );
        error_lib::CheckArrayLength( s, e, classCnt * rowSize * colSize,
                                     FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH,
                                     FUNC_NAME + " : length of [ s, e ) " + error_message::NOT_EQUAL_LENGTH + "classCnt * rowSize * colSize" );

        for ( counter_type n = 0 ; n < classCnt ; ++n ) {
          for ( counter_type r = 0 ; r < rowSize ; ++r ) {
            auto row = data_.row( r );
            row += n * colSize;
            for ( counter_type c = 0 ; c < colSize ; ++c ) {
              assert( s != e ); // 念のためチェック
              *row++ = *s++;
            }
          }
        }

        calcMean();
        calcVar();
      }

      /*
        MannWhitney コンストラクタ : データ列を指定して構築
      */
      template< class In1, class In2, class Less, class Cmp >
      MannWhitney::MannWhitney( In1 s1, In1 e1, In2 s2, In2 e2, Less less, Cmp cmp )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::test::MannWhitney constructor";

        m_ = error_lib::ArrayLength( s1, e1, FUNC_NAME + " : [ s1, e1 ) " + error_message::NEGATIVE_LENGTH );
        n_ = error_lib::ArrayLength( s2, e2, FUNC_NAME + " : [ s1, e1 ) " + error_message::NEGATIVE_LENGTH );
        u_ = UValue( s1, e1, s2, e2, less, cmp );

        isRightSide_ = ( u_ >= stat_type( m_ * n_ ) / 2.0 );

        if ( isRightSide_ )
          u_ = stat_type( m_ * n_ ) - u_;
      }
      template< class In1, class In2 >
      MannWhitney::MannWhitney( In1 s1, In1 e1, In2 s2, In2 e2 )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::test::MannWhitney constructor";

        m_ = error_lib::ArrayLength( s1, e1, FUNC_NAME + " : [ s1, e1 ) " + error_message::NEGATIVE_LENGTH );
        n_ = error_lib::ArrayLength( s2, e2, FUNC_NAME + " : [ s1, e1 ) " + error_message::NEGATIVE_LENGTH );
        u_ = UValue( s1, e1, s2, e2 );

        isRightSide_ = ( u_ >= stat_type( m_ * n_ ) / 2.0 );

        if ( isRightSide_ )
          u_ = stat_type( m_ * n_ ) - u_;
      }

      /*
        Wilcoxon コンストラクタ : データ列を指定して構築
      */
      template< class In1, class In2, class Less, class Eq >
      Wilcoxon::Wilcoxon( In1 s1, In1 e1, In2 s2, In2 e2, Less less, Eq eq )
      { sRank_ = SRankValue( s1, e1, s2, e2, &n_, less, eq ); }
      template< class In1, class In2 >
      Wilcoxon::Wilcoxon( In1 s1, In1 e1, In2 s2, In2 e2 )
      { sRank_ = SRankValue( s1, e1, s2, e2, &n_ ); }

      /*
        CalcAveSq クラス
      */
      //template< class T, class Less >
      //struct Test_KruskalWallis<T,Less>::CalcAveSq
      struct KruskalWallis::CalcAveSq
      {
        stat_type operator()( const std::vector< stat_type >& v )
        { return( std::pow( math_lib::SumAll< stat_type >( v ), 2 ) / v.size() ); }
      };

      /*
        KruskalWallis::createRank : データ列 data から順位 rank を作成する。

        R 値と要素の総計も同時に計算する
      */
      template< class T, class Less >
      //void Test_KruskalWallis<T,Less>::createRank
      void KruskalWallis::createRank
      ( const std::vector< std::vector< T > >& data, Less less,
        std::vector< std::vector< stat_type > >* rank,
        stat_type* r, counter_type* totalCnt )
      {
        using MapRank = std::map< stat_type, std::vector< distribution::Base::counter_type >, Less >;

        const std::string FUNC_NAME = "tadah_fussy::statistics::test::KruskalWallis::createRank";

        MapRank mapRank( less ); // 昇順にデータを並べるためのバッファ
        *totalCnt = 0;           // 全要素数を初期化

        // バッファ mapRank にデータをコピー(同時に並べ替え)
        for ( typename std::vector< std::vector< T > >::size_type i = 0 ; i < data.size() ; ++i ) {
          *totalCnt += error_lib::PNum< counter_type >
            ( data[i].size(), FUNC_NAME + " : all class must have at least one data" );
          for ( typename std::vector< T >::size_type j = 0 ; j < data[i].size() ; ++j ) {
            auto it = mapRank.find( data[i][j] );
            if ( it == mapRank.end() )
              mapRank.insert( typename MapRank::value_type( data[i][j], typename MapRank::mapped_type( 1, i ) ) );
            else
              ( it->second ).push_back( i );
          }
        }

        // 順位の初期化
        rank->assign( data.size(), std::vector< stat_type >() );

        if ( *totalCnt == 0 ) return;

        stat_type curRank = 1; // 現在の末尾の順位
        *r = 0; // 補正値 r の初期化
        for ( auto cit = mapRank.begin() ; cit != mapRank.end() ; ++cit ) {
          const auto& v = cit->second;
          typename MapRank::mapped_type::size_type cnt = v.size();
          stat_type avgRank = curRank + stat_type( cnt - 1 ) / 2.0; // 平均順位

          // rankに平均順位を登録
          for ( typename MapRank::mapped_type::size_type i = 0 ; i < cnt ; ++i )
            (*rank)[v[i]].push_back( avgRank );

          // 末尾の順位と補正値を更新
          curRank += v.size();
          *r += cnt * ( cnt - 1 ) * ( cnt + 1 );
        }

        // 補正値 r の計算
        stat_type d = *totalCnt * ( *totalCnt - 1 ) * ( *totalCnt + 1 );
        *r = ( d - *r ) / d;
      }

      /*
        KruskalWallis::init : 初期化ルーチン
      */
      template< class T, class Less >
      void KruskalWallis::init( const std::vector< std::vector< T > >& data, Less less )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::test::KruskalWallis::init";

        classCnt_ = data.size();
        error_lib::GT< counter_type >( classCnt_, 1, FUNC_NAME + " : size of data " + error_message::GT + " 1" );

        std::vector< std::vector< stat_type > > rank;

        createRank( data, less, &rank, &r_, &totalCnt_ );
        calcH( rank );
      }

      /*
        createRank : データから順位を求める(Friedman検定用)

        const std::vector< std::vector<double> >& data : 各集団の要素
        std::vector< std::vector<double> >& rank : 求める順位
        double& r : 同順位が存在した場合の補正値

        戻り値 : true ... 計算成功 ; false ... データが一つもない ; 要素数が等しくない群が存在する
      */
      template< class In, class Less >
      void Friedman::createRank( In s, In e, Less less )
      {
        using MapRank = std::map< stat_type, std::vector< counter_type >, Less >;
        using PairRank = std::pair< stat_type, std::vector< counter_type > >;

        const std::string FUNC_NAME = "tadah_fussy::statistics::test::Friedman::createRank";

        error_lib::CheckArrayLength( s, e, dataCnt_ * classCnt_,
                                     FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH,
                                     FUNC_NAME + " : length of [ s, e ) " + error_message::NOT_EQUAL_LENGTH + "dataCnt * classCnt" );

        r_ = 0; // 補正値 r の初期化

        // バッファにデータをコピー(同時に並べ替え)
        for ( counter_type i = 0 ; i < classCnt_ ; ++i ) {
          MapRank mapRank( less ); // 昇順にデータを並べるためのバッファ
          for ( counter_type j = 0 ; j < dataCnt_ ; ++j ) {
            auto it = mapRank.find( *s );
            if ( it == mapRank.end() )
              mapRank.insert( PairRank( *s, std::vector< counter_type >( 1, j ) ) );
            else
              ( it->second ).push_back( j );
            ++s;
          }

          stat_type curRank = 1; // 現在の末尾の順位
          for ( auto cit = mapRank.begin() ; cit != mapRank.end() ; ++cit ) {
            const std::vector< counter_type >& v = cit->second;
            counter_type cnt = v.size();
            stat_type avgRank = curRank + stat_type( cnt - 1 ) / 2.0; // 平均順位

            // rankに平均順位を登録
            for ( counter_type j = 0 ; j < cnt ; ++j )
              rank_[i][v[j]] = avgRank;

            // 末尾の順位と補正値を更新
            curRank += cnt;
            r_ += cnt * ( cnt - 1 ) * ( cnt + 1 );
          }
        }

        // 補正値 r_ の計算(集団数・要素数のどちらか一方がゼロなら r_ = 0 のままとなる)
        if ( classCnt_ > 0 && dataCnt_ > 0 ) {
          double d = classCnt_ * dataCnt_ * ( dataCnt_ + 1 ) * ( dataCnt_ - 1 );
          r_ = ( d - r_ ) / d;
        }
      }

    } // namespace test

    namespace regression
    {
      /*
        Singular コンストラクタ
      */
      template< class In1, class In2 >
      Singular::Singular( In1 xs, In1 xe, In2 ys, In2 ye )
        : summary_( xs, xe, ys, ye )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::regression::Singular constructor";

        error_lib::GT< counter_type >( summary_.count(), 2, FUNC_NAME + " : size of data " + error_message::GT + " 2" );

        // 回帰係数の推定値
        est_a_ = ( summary_.var1() != 0 ) ? summary_.covariance() / summary_.var1() : distribution::Base::nan();
        est_b_ = ( std::isnan( est_a_ ) ) ? distribution::Base::nan() : summary_.mean2() - est_a_ * summary_.mean1();

        // 誤差項の偏差の平方和
        se_ = 0;
        for ( counter_type i = 0 ; i < summary_.count() ; ++i )
          se_ += std::pow( *ys++ - expY( *xs++ ), 2 );
      }

      /*
        Multiple::estimateCoef : 回帰係数の推定

        x : 独立変数ベクトル(X)を保持する行列
        [ ys, ye ) 従属変数(Y)のデータ列
        solver 連立方程式計算用の関数
      */
      template< class X, class In, class Solver >
      void Multiple::estimateCoef( const Matrix< X >& x, In ys, In ye, Solver solver )
      {
        // 共分散行列の作成
        SymmetricMatrix< stat_type > cov;
        CovarianceMatrix( x, &cov );

        // 共分散行列の逆行列の計算
        matrix_lib::Inverse( cov, &s_inv_, solver );

        // 回帰係数の推定値
        SquareMatrix< stat_type > les( cov );
        std::vector< stat_type > ans( varSize() );
        for ( counter_type c = 0 ; c < varSize() ; ++c ) {
          auto col = x.column( c );
          ans[c] = Covariance< stat_type >( col, col.end(), ys, ye );
        }
        solver( les, ans.begin() );

        stat_type sumEstY = 0;
        for ( counter_type i = 0 ; i < varSize() ; ++i ) {
          est_a_.push_back( ans[i] );
          sumEstY += summary_x_[i].mean() * est_a_[i];
        }
        est_a0_ = summary_y_.mean() - sumEstY;
      }

      /*
        Multiple::calcSSE : 誤差項の平方和を求める

        xs : 独立変数ベクトル(X)のデータ列開始位置
        [ ys, ye ) 従属変数(Y)のデータ列
      */
      template< class X, class In >
      void Multiple::calcSSE( const Matrix< X >& x, In ys )
      {
        for ( counter_type r = 0 ; r < sampleSize() ; ++r ) {
          auto row = x.row( r );
          est_y_.push_back( expY( row, row.end() ) );
          se_ += std::pow( *ys++ - est_y_.back(), 2 );
        }
      }

      /*
        Multiple コンストラクタ : 行列 x と [ ys, ye ) で重回帰分析を行う
      */
      template< class X, class In, class Solver >
      Multiple::Multiple( const Matrix< X >& x, In ys, In ye, Solver solver )
        : se_( 0 )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::regression::Multiple constructor";

        error_lib::CheckLinearModel( x, ys, ye,
                                     FUNC_NAME + " : x " + error_message::NOT_POSITIVE_LENGTH,
                                     FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH,
                                     FUNC_NAME + " : size of x " + error_message::NOT_EQUAL_LENGTH + " [ ys, ye )" );

        summary_y_ = Moment2( ys, ye );
        summary_x_.resize( x.cols() );

        // X に対する統計量の計算
        for ( counter_type i = 0 ; i < varSize() ; ++i ) {
          auto xi = x.column( i );
          summary_x_[i] = Moment2( xi, xi.end() );
        }

        // 回帰係数の推定
        estimateCoef( x, ys, ye, solver );
        // 誤差項の平方和を求める
        calcSSE( x, ys );
      }

      /*
        Multiple::expY : 独立変数ベクトル [ xs, xe ) に対する従属変数の推定値を返す
      */
      template< class In >
      distribution::Base::stat_type Multiple::expY( In xs, In xe ) const
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::regression::Multiple::expY";

        error_lib::CheckArrayLength( xs, xe, varSize(),
                                     FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH,
                                     FUNC_NAME + " : length of [ xs, xe ) " + error_message::NOT_EQUAL_LENGTH + "varSize()" );

        return( std::inner_product( est_a_.begin(), est_a_.end(), xs, est_a0_ ) );
      }

      /*
        Multiple::createSampleOfPartialCC_ :
        独立変数ベクトルの集合 x から k 番目の要素を抽出したベクトル xk と、
        k 番目の要素を除いたベクトルの集合 xl を作成する。偏相関係数を求める処理の中で
        利用する。

        データ列 [ ys ,ye ) は従属変数のデータ列で、その数の取得と値のチェックのみに利用する。
      */
      template< class X >
      void Multiple::createSampleOfPartialCC_
      ( const Matrix< X >& x, counter_type k,
        std::vector< X >* xk, Matrix< X >* xl )
      {
        // k 番目の標本を作成
        auto csit = x.column( k );
        xk->assign( csit, csit.end() );

        // k 番目の独立変数を除いた標本を作成
        xl->assign( x.rows(), x.cols() - 1 );
        auto it = xl->begin();
        for ( typename Matrix< X >::size_type c = 0 ; c < x.cols() ; ++c ) {
          if ( c == k ) continue;
          for ( csit = x.column( c ) ; csit != csit.end() ; ++csit )
            *it++ = *csit;
        }
      }

      /*
        Multiple::calcPartialCC_ :
        独立変数ベクトルの k 番目の要素に対する偏相関係数を求める。

        xk : 独立変数ベクトルから k 番目の要素を抽出したベクトル
        xl : 独立変数ベクトルから k 番目の要素を除いたベクトルの集合
        [ ys ,ye ) : 従属変数のデータ列
        mrc_y : y と xl の重回帰モデル
        mrc_xk : xk と xl の重回帰モデル
      */
      template< class X, class In >
      distribution::Base::stat_type Multiple::calcPartialCC_
      ( const Multiple& mrc_y, const Multiple& mrc_xk,
        const std::vector< X >& xk, const Matrix< X >& xl, In ys )
      {
        stat_type syk = 0; // xk と y の標本共分散
        stat_type vk = 0;  // xk の分散
        stat_type vy = 0;  // y の分散

        for ( typename Matrix< X >::size_type r = 0 ; r < xl.rows() ; ++r ) {
          auto i_xl = xl.row( r );
          stat_type ek = xk[r] - mrc_xk.expY( i_xl, i_xl.end() );
          stat_type ey = *ys++ - mrc_y.expY( i_xl, i_xl.end() );
          syk += ek * ey;
          vk += ek * ek;
          vy += ey * ey;
        }

        return( syk / std::sqrt( vk * vy ) );
      }

      /*
        Multiple::calcPartialCC_ :
        独立変数ベクトルの k 番目の要素に対する偏相関係数を求める。

        xk : 独立変数ベクトルから k 番目の要素を抽出したベクトル
        xl : 独立変数ベクトルから k 番目の要素を除いたベクトルの集合
        [ ys ,ye ) : 従属変数のデータ列
        xSize : xl のベクトルの要素数
        mrc_y : y と xl の重回帰モデル
      */
      template< class X, class In >
      distribution::Base::stat_type Multiple::calcPartialCC_
      ( const Multiple& mrc_y,
        const std::vector< X >& xk, const Matrix< X >& xl, In ys, In ye )
      {
        using size_type = typename Matrix< X >::size_type;

        size_type xSize = xl.cols();
        MomentPair2 sumXkY( xk.begin(), xk.end(), ys, ye );

        std::vector< stat_type > vec_sk( xSize ); // xk と xl の共分散のベクトル
        std::vector< stat_type > vec_sy( xSize ); // y と xl の共分散のベクトル
        for ( size_type i = 0 ; i < xSize ; ++i ) {
          auto csit = xl.column( i );
          vec_sk[i] = Covariance< stat_type >( csit, csit.end(), xk.begin(), xk.end() );
          vec_sy[i] = Covariance< stat_type >( csit, csit.end(), ys, ye );
        }

        std::vector< stat_type > vec_Vsk( xSize, 0 ); // 共分散行列の逆行列と vec_sk の積
        std::vector< stat_type > vec_Vsy( xSize, 0 ); // 共分散行列の逆行列と vec_sy の積
        for ( size_type r = 0 ; r < xSize ; ++r ) {
          for ( size_type c = 0 ; c < xSize ; ++c ) {
            vec_Vsk[r] += mrc_y.s_inv( r, c ) * vec_sk[c];
            vec_Vsy[r] += mrc_y.s_inv( r, c ) * vec_sy[c];
          }
        }

        // 各内積の値
        stat_type syVsk = 0;
        stat_type skVsk = 0;
        stat_type syVsy = 0;
        for ( size_type i = 0 ; i < xSize ; ++i ) {
          syVsk += vec_sy[i] * vec_Vsk[i];
          skVsk += vec_sk[i] * vec_Vsk[i];
          syVsy += vec_sy[i] * vec_Vsy[i];
        }

        return( ( sumXkY.covariance() - syVsk ) / std::sqrt( ( sumXkY.var2() - syVsy ) * ( sumXkY.var1() - skVsk ) ) );
      }

    } //namespace regression

    /*
      RankCorrCoef::getRank データ列 [ s, e ) を順位データ rank に変換する
    */
    template< class In, class Less >
    void RankCorrCoef::getRank( In s, In e, std::vector< stat_type >* rank, Less less )
    {
      assert( rank != nullptr );

      // データ型の宣言
      using value_type = typename std::iterator_traits< In >::value_type; // データ列の型

      // 内部処理用マップの型 ( キーはデータ、値は配列の位置を表す )
      using mmap = typename std::multimap< value_type, size_type, Less >;

      // mmap の各種の型
      using pair = typename mmap::value_type;
      using mmap_cit = typename mmap::const_iterator;
      using mmap_diff = typename mmap::difference_type;

      const std::string FUNC_NAME = "tadah_fussy::statistics::RankCorrCoef::getRank";

      auto sz = error_lib::ArrayLength( s, e, FUNC_NAME + " : [ s, e ) " + error_message::NEGATIVE_LENGTH ); // データ列のサイズ
      rank->resize( sz );

      /*
        データのソート
        multimapへの登録によってキー順にソートされる
      */
      mmap orderedData( less );
      size_type index = 0;
      for ( In i = s ; i != e ; ++i )
        orderedData.insert( pair( *i, index++ ) );

      // 順位への変換
      mmap_cit cit = orderedData.begin();
      mmap_diff r = 0; // 現在の順位
      while ( cit != orderedData.end() ) {
        mmap_cit ub = orderedData.upper_bound( cit->first ); // キーが等しい範囲の末尾
        mmap_diff diff = std::distance( cit, ub ); // キーが等しい要素の個数
        // 平均順位の計算
        stat_type rankAvg = stat_type( r * 2 + diff - 1 ) / 2.0;
        // 平均順位を対象位置に登録
        while ( cit != ub ) {
          (*rank)[cit->second] = rankAvg;
          ++cit;
        }
        r += diff;
      }
    }

    /*
      RankCorrCoef コンストラクタ データ列 [ xs, xe ) と [ ys, ye ) から構築

      less1, less2 はそれぞれ データ列 [ xs, xe ) と [ ys, ye ) の比較用関数オブジェクトである
    */
    template< class In1, class In2, class Less1, class Less2 >
    RankCorrCoef::RankCorrCoef( In1 xs, In1 xe, In2 ys, In2 ye, Less1 less1, Less2 less2 )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::RankCorrCoef constructor";

      error_lib::CheckBivariable( xs, xe, ys, ye,
                                  FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : length of [ xs, xe ) " + error_message::NOT_EQUAL_LENGTH + " length of [ ys, ye )" );

      // 順位の計算
      getRank( xs, xe, &x_, less1 );
      getRank( ys, ye, &y_, less2 );
    }

    /*
      RankCorrCoef コンストラクタ データ列 [ xs, xe ) と [ ys, ye ) から構築
    */
    template< class In1, class In2 >
    RankCorrCoef::RankCorrCoef( In1 xs, In1 xe, In2 ys, In2 ye )
    {
      const std::string FUNC_NAME = "tadah_fussy::statistics::RankCorrCoef constructor";

      error_lib::CheckBivariable( xs, xe, ys, ye,
                                  FUNC_NAME + " : [ xs, xe ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : [ ys, ye ) " + error_message::NEGATIVE_LENGTH,
                                  FUNC_NAME + " : length of [ xs, xe ) " + error_message::NOT_EQUAL_LENGTH + " length of [ ys, ye )" );

      // 順位の計算
      getRank( xs, xe, &x_ );
      getRank( ys, ye, &y_ );
    }

    /*
      FactorAnalysis::calcSMC : 各データ列について (分散 - SMC) を算出して d に返す

      * 求めた値は D の対角成分の初期値として利用できる。
      * データ列が正規化されているため 1 - SMC になる。
      */
    template< class Solver >
    void FactorAnalysis::calcSMC( std::vector< stat_type >* d, Solver solver ) const
    {
      assert( d != nullptr );

      d->clear();

      size_type n = data_.rows(); // 独立変数の数
      if ( n == 0 ) return;
      size_type p = data_.cols(); // データ種の数

      Matrix< stat_type > partMat( n, p - 1 );
      for ( size_type r = 0 ; r < n ; ++r )
        for ( size_type c = 1 ; c < p ; ++c )
          partMat[r][c - 1] = data_[r][c];

      d->resize( p );
      for ( size_type i = 0 ; i < p ; ++i ) {
        auto y = data_.column( i ); // i 番目の標本を y とする
        regression::Multiple mrc( partMat, y, y.end(), solver );
        (*d)[i] = SampleVariance< stat_type >( y, y.end() ) - std::pow( mrc.mcc(), 2 );
        // 次の i 番目の独立変数を除いた標本を作成
        if ( i < p - 1 )
          for ( auto it = partMat.column( i ) ; it != it.end() ; ++it )
            *it = *y++;
      }
    }

    /*
      FactorAnalysis::estimateFScoreCoef : 因子負荷量 a を使い、因子得点の線形式の係数を b に求める
    */
    template< class Solver >
    void FactorAnalysis::estimateFScoreCoef( const Matrix< stat_type >& a, Matrix< stat_type >* b, Solver solver ) const
    {
      assert( b != nullptr );

      const std::string FUNC_NAME = "tadah_fussy::statistics::FactorAnalysis::estimateFScoreCoef";

      size_type p = classCnt(); // データ種のサイズ(vのサイズ,aの行数)
      error_lib::EQ( a.rows(), p, FUNC_NAME + " : a.rows() " + error_message::EQ + " size of class" );

      size_type m = a.cols(); // 因子負荷量の数

      SquareMatrix< stat_type > inv; // 共分散行列の逆行列
      matrix_lib::Inverse( cov_, &inv, solver );
      //PrintMatrix( std::cout, inv );
      b->assign( m, p );
      for ( size_type i = 0 ; i < m ; ++i ) {
        for ( size_type j = 0 ; j < p ; ++j ) {
          auto row = inv.row( j );
          auto col = a.column( i );
          (*b)[i][j] = std::inner_product( row, row.end(), col, stat_type() );
        }
      }
    }

  } // namespace statistics

} // namespace tadah_fussy
