#include "svd.h"

namespace tadah_fussy
{
  namespace statistics
  {
    using std::vector;
    using std::multimap;

    using stat_type = distribution::Base::stat_type;

    /*
      PrincipalComponentAnalysis::operator() : 主成分分析を行う

      e : 対角化を行うときに、非対角成分をゼロと判断するためのしきい値
      maxCnt : 対角化を行うときの反復処理最大回数

      戻り値 : なし
    */
    void PrincipalComponentAnalysis::operator()( stat_type e, uint32_t maxCnt )
    {
      SquareMatrix< stat_type > r; // 上三角行列(対角成分が固有値になる)

      // 対象行列の対角化
      math_lib::eigen_lib::Householder_DoubleShiftQR< stat_type >( cov_, &r, &q_, e, maxCnt );

      evTotal_ = 0; // 固有値の総計
      for ( SquareMatrix< stat_type >::size_type i = 0 ; i < r.size() ; ++i ) {
        // 固有ベクトルの位置を値として登録しておく
        ev_.insert( std::make_pair( r[i][i], i ) );
        evTotal_ += r[i][i];
      }
    }

    /*
      PrincipalComponentAnalysis::contribution : i 番目のデータ種の寄与率を返す
    */
    stat_type PrincipalComponentAnalysis::contribution( size_t i ) const
    {
      if ( i >= ev_.size() )
        throw std::out_of_range( "PrincipalComponentAnalysis::contribution : Specified index is too large" );

      if ( evTotal_ == 0 ) return( distribution::Base::nan() ); // 総和がゼロなら nan

      // 対象のデータ種を指す反復子
      auto cit = ev_.begin();
      std::advance( cit, i );

      return( cit->first / evTotal_ );
    }

    /*
      PrincipalComponentAnalysis::eigenvalue : i 番目のデータ種の固有値を返す
    */
    stat_type PrincipalComponentAnalysis::eigenvalue( size_t i ) const
    {
      if ( i >= ev_.size() )
        throw std::out_of_range( "PrincipalComponentAnalysis::eigenvalue : Specified index is too large" );

      // 対象のデータ種を指す反復子
      auto cit = ev_.begin();
      std::advance( cit, i );

      return( cit->first );
    }

    /*
      PrincipalComponentAnalysis::eigenvector : i 番目のデータ種の固有ベクトルを返す
    */
    vector< stat_type > PrincipalComponentAnalysis::eigenvector( size_t i ) const
    {
      if ( i >= ev_.size() )
        throw std::out_of_range( "PrincipalComponentAnalysis::eigenvector : Specified index is too large" );

      // 対象のデータ種を指す反復子
      auto cit = ev_.begin();
      std::advance( cit, i );
      // ベクトルをコピー
      auto col = q_.column( cit->second );
      vector< stat_type > ans( col, col.end() );

      return( ans );
    }

  } // namespace statistics

} // namespace tadah_fussy
