/**
   @file svd.h
   @brief 主成分分析・特異値分解ルーチン定義用ヘッダ

   @author tadah_fussy
   @date 2016/05/26 新規作成
   @date 2021/03/25 名前空間の変更
**/

#ifndef SVD_20210325_H
#define SVD_20210325_H

#include "statistics.h"

namespace tadah_fussy
{
  namespace statistics
  {
    /**
       @brief 主成分分析用関数オブジェクト

       データ列 X の共分散行列 V を求め、固有値分解 ( スペクトル分解 ) V = QD(Qt)を行う。
       D は固有値からなる対角行列で、Q は固有ベクトルからなる直交行列、Qt はその転置行列を表す。
       各標本の独立変数ベクトルを xi としたとき、(Qt)xi は主成分を表し、D の固有値はその分散となる。
    **/
    class PrincipalComponentAnalysis
    {
      // 固有値をキーに固有ベクトルへのインデックスを保持する型
      typedef std::multimap<
        distribution::Base::stat_type,
        SquareMatrix< distribution::Base::stat_type >::size_type,
        std::greater< distribution::Base::stat_type >
        > EigenValue;

      SymmetricMatrix< distribution::Base::stat_type > cov_; // 標本共分散行列
      size_t sampleSize_; // 標本数

      EigenValue ev_; // 固有値と固有ベクトルへのインデックス
      SquareMatrix< distribution::Base::stat_type > q_; // QR分解後の直交行列(固有ベクトル)
      distribution::Base::stat_type evTotal_; // 固有値の総和

      std::vector< SquareMatrix< distribution::Base::stat_type >::const_iterator > idx_; // 固有値を大きい順に並べた時の固有ベクトルの位置

    public:

      /// @brief 対象データを指定して構築
      ///
      /// @param data 対象データ
      template< class T >
      PrincipalComponentAnalysis( const Matrix< T >& data )
        : sampleSize_( data.rows() )
      { CovarianceMatrix( data, &cov_ ); }

      /// @brief 主成分分析を行う
      ///
      /// @param e 対角化を行うときに、非対角成分をゼロと判断するためのしきい値
      /// @param maxCnt 対角化を行うときの反復処理最大回数
      /// @return なし
      void operator()( distribution::Base::stat_type e, uint32_t maxCnt );

      /// @brief 寄与率を返す
      ///
      /// 寄与率は、対象のデータ種の固有値に対して固有値の総和との比率から求められる。
      ///
      /// - 範囲外を指定した場合、例外 std::out_of_range を投げる。
      ///
      /// @param i 対象データ種の番号
      /// @return 寄与率(範囲外を指定した場合ゼロが、固有値の総和がゼロの場合 distribution::Base::nan() がそれぞれ返される)
      distribution::Base::stat_type contribution( size_t i ) const;

      /// @brief 主成分の分散(固有値)を返す
      ///
      /// 主成分の分散は、固有値分解後の固有値そのものである。番号は、固有値の大きいものから順番に付けられる。
      ///
      /// - 範囲外を指定した場合、例外 std::out_of_range を投げる。
      ///
      /// @param i 対象の主成分の番号
      /// @return 主成分
      distribution::Base::stat_type eigenvalue( size_t i ) const;

      /// @brief 固有ベクトルを返す
      ///
      /// 番号は、固有値の大きいものから順番に付けられる。
      ///
      /// - 範囲外を指定した場合、例外 std::out_of_range を投げる。
      ///
      /// @param i 対象の主成分の番号
      /// @return 主成分
      std::vector< distribution::Base::stat_type > eigenvector( size_t i ) const;

      /// @brief 主成分の数を返す
      ///
      /// @return 主成分の数
      size_t size() const
      { return( ev_.size() ); }

      /// @brief 標本数を返す
      ///
      /// @return 標本の数
      size_t sampleSize() const
      { return( sampleSize_ ); }
    };

  } // namespace statistics

} // namespace tadah_fussy

#include "svd_inline.h"

#endif
